/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 * Name: Class4xsb.c
 *
 * Description: Implementation of diagnostics functions for 4401/4402.
 *
 * Author: Steven Siener
 *
 * $Log:: /Source/Bcm44xx/b44diag/Cl4xsb.c                         $
 * 
 * 13    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 12    7/11/02 1:00p Ahui
 * Fixed a bug in Phy Loopback set up.
 * 
 * 11    7/08/02 12:27p Ahui
 * Modified code to support new card srom,  mac address programming,
 * boot rom enable/disable and boot rom size programming.
 * 
 * 10    6/17/02 2:45p Ahui
 * Replace 16 bit packet counter to 32 bit packet counter in lbtest.
 * Also all countes will be updated periodically.
 * Mac duplex will follow phy duplex state
 * 
 * 9     6/11/02 4:06p Ahui
 * Added pattern select for blast.
 * Added pattern and max packet count selections for lbtest.
 * Enable PCI brust.
 * Modified lbtest and blast to improve performance.
 *
 * 8     5/29/02 6:38p Ahui
 * 1.  Updated error message.
 * 2.  loopback test packet size increase to 1514 bytes
 * 
 * 7     4/25/02 11:11a Ahui
 * Modified loopback test to support 100/10BT loopback
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 * 
 * 1     2/08/02 5:26p Hyfeng
 * 
 ****************************************************************************/
#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>
#include <CRTDBG.H>
#include "types.h"


#include "Cl4xsb.h"
#include "t_io.h"
#include "t3_tests.h"
#include "mm.h"
#include "t3_util.h"
#include "pci.h"
#include "t3_tests.h"
#include "sromutl.h"
#include "t3_win.h"
#include "time.h"
/* Client Mode PCI memory access space (1 GB); host
   addresses posted to the chip exist in this space. */
#define SB_PCI_DMA      0x40000000      
#define SBID_REG_PCI    0x18002000
#define SB2PCI0         0x100
#define SB2PCI1         0x104
#define SB2PCI2         0x108

#define SBTOPCI_PREF    0x04
#define SBTOPCI_BURST   0x08
/* upper 4K of register space contains sprom, pci regs, etc. */

//struct regspec Class4xsb::reset_values[] = {
//    { NULL, 0, 0, 0x0 }
//};

//TESTENTRY Class4xsb::sm_testlist[] = {
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagSuspendDrv },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestRegisters },//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestRegisters },

//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestMii },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestIntMemory },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestExtMemory },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestInterrupt },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestCPU },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestLEDs },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagTestEEPROM },
//    { IDS_TEST_FIFOLOOP,(TestFunc_t) SB4xDiagTestLoopBack },
//    { IDS_TEST_RAM,   (TestFunc_t) SB4xDiagResumeDriver },
//};

#define MIN(a,b)            (a > b ? b : a)
#define ROUNDUP(x, y)       ((((x)+((y)-1))/(y))*(y))
#define NTXD                128
#define NRXD                512
#define NRXDBUFF            48
#define RXOFF               32
#define RXBUFSIZE           2048 


/******************************************************************************/
/* Error string constants. */
/******************************************************************************/

//const char *ERR_UNABLE_TO_SUSPEND_DRIVER =
//    "Unable to suspend the driver.";
//const char *ERR_DRIVER_NOT_SUSPENDED = 
//    "Driver is not in a suspend state.";
//const char *ERR_UNABLE_TO_RESUME_DRIVER = 
//    "Unable to resume the driver.";
//const char *ERR_DIAG_NOT_INIT = 
//    "No diagnostic access right.";

#define DRV_IN_DIAG_MODE                5

/******************************************************************************/
/* Register test entry. */
/******************************************************************************/

#define TEST_RESET      0x1
#define TEST_RW         0x2
#define TEST_ALL        (TEST_RESET | TEST_RW)

typedef struct
{
	CHAR RegName[50];               // Char str of register name
    ULONG Offset;       // Register offset.
	ULONG Size;                     // Size in bytes
	ULONG ResetVal;         // value after core reset
    ULONG RdMask;       // Read only register mask.
    ULONG WrMask;       // Read/Write mask.
	ULONG RunTests;         // bits that determine which tests to run
} SB4X_REG_ENTRY, *PSB4X_REG_ENTRY;

static ULONG MallocAddr;
/******************************************************************************/
/* Local functions. */
/******************************************************************************/

/* SB-based core registers for register read/write test */
static SB4X_REG_ENTRY CoreRegTable[] = {
	/* Device and Power Control */
//      { "devctl",             0x000,  4,      0x400,  0x400,          0x007f8080, TEST_ALL },     // A1
//      { "devctl",             0x000,  4,      0x8400, 0x400,          0x007f8080, TEST_ALL },     // A0
	{ "bist",               0x00c,  4,      0x0,    0xffffffff,     0x0, TEST_ALL },
	{ "wakeuplen",  0x010,  4,      0x0,    0x0,            0xffffffff, TEST_ALL },

	/* Interrupt Control */
	{ "intstat",    0x020,  4,      0x0,    0x0,            0x0, TEST_RESET },
	{ "intmask",    0x024,  4,      0x0,    0x0,            0x1d01fcc0, TEST_RESET },
	{ "inttimer",   0x028,  4,      0x0,    0x0,            0xffffffff, TEST_RESET },

	/* Ethernet MAC Control */
	{ "enetidx",    0x090,  4,      0x0,    0x0,            0x7fc, TEST_ALL },
	{ "enetdata",   0x094,  4,      0x0,    0x0,            0xffffffff, TEST_RW },
	{ "xmtfifoctl", 0x0a0,  4,      0x010,  0x0,            0x1ff, TEST_ALL },
	{ "rcvfifoctl", 0x0a4,  4,      0x010,  0x0,            0x1ff, TEST_ALL },
//      { "emaccontrol",0x0a8,  4,      0xe0,   0x0,            0xed, TEST_RW },
	{ "emacflowctl",0x0ac,  4,      0x0,    0x0,            0xffff, TEST_ALL },
	{ "intrcvlazy", 0x100,  4,      0x0,    0x0,            0xffffffff, TEST_ALL },

	/* descriptor processor regs */
	{ "xmtctl",             0x200,  4,      0x0,    0x0,            0xf, TEST_ALL },
	{ "xmtaddr",    0x204,  4,      0x0,    0x0,            0xfffff000, TEST_ALL },
	{ "xmtptr",             0x208,  4,      0x0,    0x0,            0xfff, TEST_RESET },
	{ "xmtstat",    0x20c,  4,      0x0,    0xfffff,        0x0, TEST_ALL },
	{ "rcvctl",             0x210,  4,      0x40,   0x0,            0xff, TEST_ALL },
	{ "rcvaddr",    0x214,  4,      0x0,    0x0,            0xfffff000, TEST_ALL },
	{ "rcvptr",             0x218,  4,      0x0,    0x0,            0xfff, TEST_RESET },
	{ "rcvstat",    0x21c,  4,      0x0,    0xfffff,        0x0, TEST_ALL },

	/* Emac Control registers - in EMAC Block */
	{ "emrcvcfg",   0x400,  4,      0x0,    0x0,            0xff, TEST_ALL },
	{ "emrcvmaxl",  0x404,  4,      0x0,    0x0,            0x7ff, TEST_ALL },
	{ "emxmtmaxl",  0x408,  4,      0x5ee,  0x0,            0x7ff, TEST_ALL },
	{ "emmiictl",   0x410,  2,      0x0,    0x0,            0xff, TEST_ALL },
	{ "emmiidata",  0x414,  4,      0x0,    0x0,            0xffffffff, TEST_ALL },
	{ "emintmask",  0x418,  4,      0x0,    0x0,            0x7, TEST_ALL },
	{ "emintstat",  0x41c,  4,      0x0,    0x0,            0x7, TEST_RESET },
	{ "emcamdatl",  0x420,  4,      0x0,    0x0,            0xffffffff, TEST_ALL },
	{ "emcamdath",  0x424,  4,      0x0,    0x0,            0x1ffff, TEST_ALL },
	{ "emcamctl",   0x428,  4,      0x0,    0x0,            0x803f000f, TEST_ALL },
	/* Note: emctl has bits which don't easily fit this register test model. Fix this. */
	/*{ "emctl",    0x42c,  4,      0x0,    0x0,            0x4 }, */
	/* Note: Bit 4 of emxmtctl exists but is unconnected and provides no functionality. */
	{ "emxmtctl",   0x430,  4,      0x0,    0x0,            0x1f, TEST_ALL },
	{ "emxmtwmk",   0x434,  2,      0x10,   0x0,            0x3f, TEST_ALL },
	/* Note: Bit 1 of emmibctl exists but is unconnected and provides no functionality. */
	{ "emmibctl",   0x438,  2,      0x0,    0x0,            0x3, TEST_ALL },

	/* EMAC MIB Counters */
	{ "txgdoct",    0x500,  4,      0x0,    0x0,            0xffffffff, TEST_RW },
	{ "txgdpkt",    0x504,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txalloct",   0x508,  4,      0x0,    0x0,            0xffffffff, TEST_RW },
	{ "txallpkt",   0x50c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txbcast",    0x510,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txmcast",    0x514,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx64",               0x518,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx65_127",   0x51c,  2,      0x0,    0x0,            0xffff, TEST_RW },

	{ "tx128_255",  0x520,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx256_511",  0x524,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx512_1k",   0x528,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx1k_max",   0x52c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txjab",              0x530,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txovr",              0x534,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txfrag",             0x538,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txundrn",    0x53c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txcol",              0x540,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "tx1col",             0x544,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txmcol",             0x548,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txexcol",    0x54c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txlate",             0x550,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txdefer",    0x554,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txcrser",    0x558,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "txpaus",             0x55c,  2,      0x0,    0x0,            0xffff, TEST_RW },

	{ "rxgdoct",    0x580,  4,      0x0,    0x0,            0xffffffff, TEST_RW },
	{ "rxgdpkt",    0x584,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxalloct",   0x588,  4,      0x0,    0x0,            0xffffffff, TEST_RW },
	{ "rxallpkt",   0x58c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxbcast",    0x590,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxmcast",    0x594,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx64",               0x598,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx65_127",   0x59c,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx128_255",  0x5a0,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx256_511",  0x5a4,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx512_1k",   0x5a8,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rx1k_max",   0x5ac,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxjab",              0x5b0,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxovr",              0x5b4,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxfrag",             0x5b8,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxdrop",             0x5bc,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxcrcal",    0x5c0,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxundrn",    0x5c4,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxcrcer",    0x5c8,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxalign",    0x5cc,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxsym",              0x5d0,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxpaus",             0x5d4,  2,      0x0,    0x0,            0xffff, TEST_RW },
	{ "rxcntrl",    0x5d8,  2,      0x0,    0x0,            0xffff, TEST_RW },

	/* System Backplane Configuration Space */
	{ "sbipsflag",  0xf08,  4,      0x0,            0xffffffff,     0x0, TEST_ALL },
	{ "sbtpsflag",  0xf18,  4,      0x41,           0xffffffff,     0x0, TEST_ALL },
	{ "sbimstate",  0xf90,  4,      0x00000020,     0x600cf,        0x0, TEST_ALL },
	{ "sbimintvec", 0xf94,  4,      0x00000000,     0xffffffff,     0x0, TEST_ALL },
	{ "sbtmstateh", 0xf9c,  4,      0x0,            0xffffffff,     0x0, TEST_RESET },
	{ "sbimconfigh",0xfac,  4,      0x00000000,     0x0,            0x0, TEST_ALL },
	{ "sbaddr0",    0xfb0,  4,      0x18000058,     0xffffffff,     0x0, TEST_ALL },
	{ "sbtmconfigl",0xfb8,  4,      0x00030101,     0xffffffff,     0x0, TEST_ALL },
	{ "sbtmconfigh",0xfbc,  4,      0x00000100,     0xeff,          0x0, TEST_ALL },
	{ "sbbconfig",  0xfc0,  4,      0x00ff0002,     0xff00f0,       0x0, TEST_ALL },
	{ "sbbstate",   0xfc8,  4,      0x00000002,     0x0,            0x0, TEST_RESET },
	{ "sbidlo",             0xff8,  4,      0x000522c4,     0xfffffff,      0x0, TEST_ALL },
//      { "sbidhi",             0xffc,  4,      0x42438064,     0xffffffff,     0x0, TEST_ALL },    // A1
//      { "sbidhi",             0xffc,  4,      0x42438061,     0xffffffff,     0x0, TEST_ALL },    // A0
	/* do last & don't write bistenable/clkenable/reset bits (80010001) - 
	   it wreaks havoc */
	{ "sbtmstatel", 0xf98,  4,      0x10000,        0x0,            0x7ffe0002, TEST_ALL },
};

/* BCM5221 phy registers. */
static SB4X_REG_ENTRY PhyRegTable[] = {
	{ "miictrl",    0x00,   2,      0x3000,         0x0800,         0x7580, TEST_RESET },
	{ "miistatus",  0x01,   2,      0x7809,         0xf83f,         0x0040, TEST_ALL },
	{ "phyidhi",    0x02,   2,      0x0040,         0xffff,         0x0000, TEST_ALL },
	{ "phyidlo",    0x03,   2,      0x6360,         0xffff,         0x0000, TEST_ALL },
	{ "autonegadv", 0x04,   2,      0x05e1,         0x0000,         0xa7ff, TEST_ALL },
	{ "lpadv",              0x05,   2,      0x0000,         0xffff,         0x0000, TEST_ALL },
	{ "autonegexp", 0x06,   2,      0x0004,         0x001f,         0x0000, TEST_ALL },
	{ "nextpage",   0x07,   2,      0x2001,         0x0000,         0xb7ff, TEST_ALL },
	{ "lpnextpage", 0x08,   2,      0x0000,         0xbfff,         0x0000, TEST_ALL },
	{ "100auxctl",  0x10,   2,      0x1000,         0x0000,         0x27fe, TEST_ALL },
	{ "100auxstat", 0x11,   2,      0x0000,         0x0fbe,         0x0000, TEST_RW },
	{ "100rcverrs", 0x12,   2,      0x0000,         0x0000,         0xffff, TEST_ALL },
	{ "100carsens", 0x13,   2,      0x0000,         0x0000,         0xffff, TEST_ALL },
	{ "100discons", 0x14,   2,      0x0200,         0xc000,         0x0000, TEST_ALL },
	{ "auxctlstat", 0x18,   2,      0x003c,         0x000f,         0xc0f0, TEST_ALL },
	{ "auxstatsum", 0x19,   2,      0x0002,         0xffff,         0x0000, TEST_RESET },
	{ "interrupt",  0x1a,   2,      0x9f00,         0x000f,         0x0f00, TEST_ALL },
	{ "auxmode2",   0x1b,   2,      0x008a,         0x0000,         0x0ee6, TEST_ALL },
//      { "10terrgstat",0x1c,   2,      0x002c,         0x060f,         0x1800, TEST_ALL },
	{ "auxmode",    0x1d,   2,      0x0000,         0x0000,         0x001a, TEST_ALL },
	{ "auxmultphy", 0x1e,   2,      0x0000,         0xf8f0,         0x000a, TEST_ALL },
};


//static BUFENTRY buffers[50];
static ULONG
Allocatebuffer(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG size,
	ULONG Alig)
{
    ULONG       addrs[3];
    PBUFENTRY   pbuf, limit;

	if (Alig)
	{
		MallocAddr =  malloc(size*2);
		addrs[1] = (MallocAddr & ~(0xfff)) + 0x1000;
	}
	else
	{
		MallocAddr =  malloc(size);
		addrs[1] = MallocAddr ;
	}

	if ( addrs[ 1 ] == 0 )
	return( 0 );

    limit = ( PBUFENTRY )( ( char * )pDmabuf + uDmaBufSize );
    for ( pbuf = pDmabuf; pbuf < limit; pbuf++ )
    {
	if ( pbuf->phys_addr == 0 )
	{
	    pbuf->phys_addr = ( ULONG )addrs[ 1 ];
	    pbuf->vaddr = ( ULONG )MallocAddr;
	    pbuf->size = size;
	    break;
	}
    }

    return( addrs[ 1 ] );
}


static unsigned int
Deallocatebuffer(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG phys_addr )
{
    
   
    ULONG       addrs[2];
    PBUFENTRY   pbuf, limit;

    limit = ( PBUFENTRY )( ( char * )pDmabuf + uDmaBufSize );
    for ( pbuf = pDmabuf; pbuf < limit; pbuf++ )
    {
	if ( pbuf->phys_addr == phys_addr )
	    break;
    }

    if ( pbuf >= limit )
    {
	printf("Deallocatebuffer(): unable to find "
		"buffer \r\n" );
	return( FALSE );
    }

    addrs[ 0 ] = ( ULONG )pbuf->vaddr;
    addrs[ 1 ] = ( ULONG )pbuf->phys_addr;

    free(addrs[0]);

    memset( pbuf, 0, sizeof( BUFENTRY ) );

    return( TRUE );
}

static unsigned int
ReadBuffer(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG phys_addr,
    char *buffer_start,
    ULONG length,
    ULONG offset )
{
    PBUFENTRY   pbuf, limit;

    limit = ( PBUFENTRY )( ( char * )pDmabuf + uDmaBufSize );
    for ( pbuf = pDmabuf; pbuf < limit; pbuf++ )
    {
	if ( pbuf->phys_addr == phys_addr )
	    break;
    }

    if ( ( pbuf >= limit ) || ( length > ( pbuf->size - offset ) ) )
    {
		//printf( "ReadBuffer(): unable to find "
		//"buffer \r\n" );
		return( FALSE );
    }

    memcpy( buffer_start, ( const void * )( pbuf->phys_addr + offset), length );

    return( TRUE );
}


static unsigned int
WriteBuffer(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG phys_addr,
    char *buffer_start,
    ULONG length,
    ULONG offset )
{
    PBUFENTRY   pbuf, limit;

    limit = ( PBUFENTRY )( ( char * )pDmabuf + uDmaBufSize );
    for ( pbuf = pDmabuf; pbuf < limit; pbuf++ )
    {
	if ( pbuf->phys_addr == phys_addr )
	    break;
    }

    if ( ( pbuf >= limit ) || ( length > ( pbuf->size - offset ) ) )
    {
	printf( "WriteBuffer(): unable to find "
		"buffer \r\n" );
	return( FALSE );
    }

    memcpy( (void *)( pbuf->phys_addr + offset ), ( const void * )buffer_start,
	    length );

    return( TRUE );
}

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
ULONG EpromRd( ULONG Offset)
{
	return RegRd( SPROM_BASE + Offset);
}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
void EpromWr( U16 Offset, U16 value16)
{
	*(((u16 *)CURRCARD.base_address) + (Offset >> 2)) = value16;
}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
void
RegWr(ULONG Offset, ULONG Value32)
{
	t_writemem32(Offset,Value32);
}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
void
RegWr16(u16 Offset, u16 Value16)
{
	t_writemem16(Offset,Value16);
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
ULONG RegRd( ULONG Offset)
{
	return(t_readmem32(Offset));
}
static ULONG iRegRd( ULONG Offset)
{
	ULONG value32;
	write_configuration_dword(CURRCARD.bus_number,
			    CURRCARD.device_and_function,
			    PCI_CS_BlackDoorAddr,
			    Offset);
	read_configuration_dword(CURRCARD.bus_number,
			    CURRCARD.device_and_function,
			    PCI_CS_BlackDoorData,
			    &value32);
	return(value32);
}
static void iRegWr( ULONG Offset, ULONG value32)
{
	write_configuration_dword(CURRCARD.bus_number,
			    CURRCARD.device_and_function,
			    PCI_CS_BlackDoorAddr,
			    Offset);
	write_configuration_dword(CURRCARD.bus_number,
			    CURRCARD.device_and_function,
			    PCI_CS_BlackDoorData,
			    value32);
}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

void RegOr( ULONG Offset, ULONG Value32 )
{
	ULONG TempVal;

	TempVal = RegRd(Offset);
	RegWr(Offset, Value32 | TempVal);
}

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

void RegAnd(ULONG Offset,ULONG Value32 )
{
	ULONG TempVal;

	TempVal = RegRd(Offset);
	RegWr(Offset, Value32 & TempVal);
}

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
void PhyWr( UCHAR Offset, U16 Value16)
{
    miiwr(Offset,Value16);
}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
U16 PhyRd(UCHAR Offset)
{
    return miird(Offset);
}


/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

U32 SB4xDiagTestRegisters(u32 req_iteration, u8 indir)
{

    U32     Retcode;
    ULONG   Idx;
	ULONG Offset;
	ULONG RdMask;
	ULONG WrMask;
	ULONG Value32;
	ULONG RdValue;      // Read only value.
	ULONG Oldcfg;
	ULONG local_iteration = 0;
	int i;

	if (!global.manuf)
		printf("Begin Reg test.\n");

    Retcode = NO_ERROR;
	do 
    {
		local_iteration++;
		if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
		check_abort();
		// reset the core so we can check reset values 
		core_reset();

		// test reset values first 
		i = 0;
		if (!global.manuf)
			printf("\n");
		for(Idx = 0; Idx < sizeof(CoreRegTable)/sizeof(SB4X_REG_ENTRY); Idx++)
		{
			if (!(CoreRegTable[Idx].RunTests & TEST_RESET))
				continue;

			// Get the offset and mask from the table.
			Offset = CoreRegTable[Idx].Offset;

			// Save the original register content.
			Oldcfg = (indir)? iRegRd(Offset):RegRd(Offset);
			//printf("Reg: 0x%4x  Actual value: %4x %4x\n", Offset, (Oldcfg >> 16) & 0xffff, Oldcfg & 0xffff);
			if (!global.manuf)
			{
				printf("%08x ",Oldcfg);
				i++;
				if (i == 7) 
				{       printf("\n");
					i=0;
				}
			}
			
			// Test the reset value.
			if(Oldcfg != CoreRegTable[Idx].ResetVal)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("Register test failed for reg %s at 0x%x.  Expecting "
						"reset value of 0x%x.  The actual is 0x%x.\n",
						CoreRegTable[Idx].RegName, Offset, CoreRegTable[Idx].ResetVal, Oldcfg); 
				}
				else
				{
					test_failed3(ERR_IND_REG_ERR,Oldcfg,Offset,CoreRegTable[Idx].ResetVal);
				}
				check_abort();
				Retcode = HAS_ERROR;
				if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);

				break;
			}
		}
        if (Retcode == HAS_ERROR)
			break;
	// test read/write values 
		i=0;
		if (!global.manuf)
			printf("\n");
		for(Idx = 0; Idx < sizeof(CoreRegTable)/sizeof(SB4X_REG_ENTRY); Idx++)
		{
			if (!(CoreRegTable[Idx].RunTests & TEST_RW))
				continue;

			// Get the offset and mask from the table.
			Offset = CoreRegTable[Idx].Offset;
			RdMask = CoreRegTable[Idx].RdMask;
			WrMask = CoreRegTable[Idx].WrMask;

			// Save the original register content.
			Oldcfg = (indir)? iRegRd(Offset):RegRd(Offset);
			if (!global.manuf)
			{
				printf("%08x ",Oldcfg);
				i++;
				if (i == 7) 
				{       printf("\n");
					i=0;
				}
			}
			// Determine the read-only value.
			RdValue = Oldcfg & RdMask;

			// Write zero to the register, then make sure the read-only bits
			// are not changed and the read/write bits are all zeros.
			//RegWr( Offset, 0);
			//Value32 = RegRd( Offset);
			
			if (indir)
				iRegWr( Offset, 0);
			else
				RegWr( Offset, 0);

			Value32 = (indir)? iRegRd(Offset):RegRd(Offset);
			// Test the read-only bits.
			if((Value32 & RdMask) != RdValue)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("Register test failed for reg %s at 0x%x.  Expecting "
						"read-only value of 0x%x.  The actual is 0x%x.",
						CoreRegTable[Idx].RegName, Offset, RdValue, Value32 & RdMask);
				}
				else
				{
					test_failed2(ERR_READ_ONLY_CLEAR,test_bitString((Value32^Oldcfg) & RdMask),Offset);
				}

				check_abort();

				Retcode = HAS_ERROR;
				if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);
				//RegWr( Offset, Oldcfg);

				break;
			}
			
			// Test the read/write bits.
			if(Value32 & WrMask)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("Register test failed for reg %s at 0x%x.  Expecting "
						"read-write value of 0x%x.  The actual is 0x%x.",
						CoreRegTable[Idx].RegName, Offset, 0, Value32 & WrMask);
				}
				else
				{
					test_failed2(ERR_READ_WRITE_NOT_CLEAR,test_bitString((Value32^Oldcfg) & WrMask),Offset);
				}
				check_abort();

				Retcode = HAS_ERROR;
				if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);
				//RegWr( Offset, Oldcfg);
			
				break;
			}
			
			// Write ones to all the bits defined by RdMask and WrMask, then make
			// sure the read-only bits are not changed and the read/write bits are
			// all ones.
			//RegWr( Offset, RdMask | WrMask);
			//Value32 = RegRd( Offset);
			if (indir)
					iRegWr( Offset, RdMask | WrMask);
				else
					RegWr( Offset, RdMask | WrMask);
			Value32 = (indir)? iRegRd(Offset):RegRd(Offset);
			// Test the read-only bits.
			if((Value32 & RdMask) != RdValue)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("Register test failed for reg %s at 0x%x.  Expecting "
						"read-only value of 0x%x.  The actual is 0x%x.",
						CoreRegTable[Idx].RegName, Offset, RdValue, Value32 & RdMask);
				}
				else
				{
					test_failed2(ERR_READ_ONLY_SET,test_bitString((Value32^Oldcfg) & RdMask),Offset);
				}
				check_abort();

				Retcode = HAS_ERROR;
				if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);
				//RegWr( Offset, Oldcfg);

				break;
			}

			// Test the read/write bits.
			if((Value32 & WrMask) != WrMask)
			{       
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("Register test failed for reg %s at 0x%x.  Expecting "
						"read-write value of 0x%x.  The actual is 0x%x.\n",
						CoreRegTable[Idx].RegName, Offset, WrMask, Value32 & WrMask);
				}
				else
				{
					return test_failed2(ERR_READ_WRITE_NOT_SET,test_bitString((Value32^Oldcfg) & WrMask),Offset);
				}
				check_abort();
				
				Retcode = HAS_ERROR;;
				if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);
				//RegWr( Offset, Oldcfg);

				break;
			}
			if (indir)
					iRegWr( Offset, Oldcfg);
				else
					RegWr( Offset, Oldcfg);
			//RegWr( Offset, Oldcfg);
		}
		if (Retcode == HAS_ERROR)
			break;
	}while ((Retcode == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));


	reset();
	if (!global.manuf)
	{
		printf("\n");
		if (Retcode == NO_ERROR) 
			printf("Reg test PASSED.\n"); 
		else
			printf("Reg test FAILED.\n"); 
		return Retcode;
	}
	else
    {
		if (Retcode == NO_ERROR) 
			return test_passed0();
		else
			return Retcode;
	}
}



/******************************************************************************/
/* Description: Test phy registers                                            */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

U32 SB4xDiagTestMii(u32 req_iteration)
{
    U32     Retcode;
    ULONG   Idx;
    ULONG Offset;
    ULONG RdMask;
    ULONG WrMask;
    ULONG Value32;
    ULONG RdValue;      // Read only value.
    ULONG Oldcfg;
	ULONG local_iteration = 0;
	int i;
	if (!global.manuf)
		printf("Begin MII Reg test.\n");

    Retcode = NO_ERROR;
	do 
    {
		local_iteration++;
		if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
		check_abort();
		reset();
		// do a hard reset of the ephy 
		RegOr( REGOFFSET(devcontrol), DC_ER);
		MM_Wait(10000);
		RegAnd( REGOFFSET(devcontrol), ~DC_ER);
		MM_Wait(10000);
		i=0;
		if (!global.manuf)
			printf("\n");
		for(Idx = 0; Idx < sizeof(PhyRegTable)/sizeof(SB4X_REG_ENTRY); Idx++)
		{
			if (!(PhyRegTable[Idx].RunTests & TEST_RESET))
				continue;

			Offset = PhyRegTable[Idx].Offset;
			// Save the original register content.
			Oldcfg = PhyRd(Offset);
			if (!global.manuf)
			{
				printf("%04x ",Oldcfg);
				i++;
				if (i == 7) 
				{       printf("\n");
					i=0;
				}
			}
			// Test reset value

			if(Oldcfg != PhyRegTable[Idx].ResetVal)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("MII register test failed for reg %s at 0x%x.  Expecting "
						"reset value of 0x%x.  The actual is 0x%x.\n",
						PhyRegTable[Idx].RegName, Offset, PhyRegTable[Idx].ResetVal, Oldcfg);
				}
				else
				{
					test_failed3(ERR_IND_REG_ERR,Oldcfg,Offset,PhyRegTable[Idx].ResetVal);
				}
				check_abort();

				Retcode = HAS_ERROR;

				PhyWr( Offset, Oldcfg);

				break;
			}
		}
		if (Retcode == HAS_ERROR)
			break; //do

		i=0;
		if (!global.manuf)
			printf("\n");
		for(Idx = 0; Idx < sizeof(PhyRegTable)/sizeof(SB4X_REG_ENTRY); Idx++)
		{
			if (!(PhyRegTable[Idx].RunTests & TEST_RW))
				continue;

			// Get the offset and mask from the table.
			Offset = PhyRegTable[Idx].Offset;
			RdMask = PhyRegTable[Idx].RdMask;
			WrMask = PhyRegTable[Idx].WrMask;

			// Save the original register content.
			Oldcfg = PhyRd(Offset);
			if (!global.manuf)
			{
				printf("%04x ",Oldcfg);
				i++;
				if (i == 7) 
				{       printf("\n");
					i=0;
				}
			}
			// Determine the read-only value.
			RdValue = Oldcfg & RdMask;

			// Write zero to the register, then make sure the read-only bits
			// are not changed and the read/write bits are all zeros.
			PhyWr( Offset, 0);
			Value32 = PhyRd(Offset);

			// Test the read-only bits.
			if((Value32 & RdMask) != RdValue)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("MII register test failed for reg %s at 0x%x.  Expecting "
						"read-only value of 0x%x.  The actual is 0x%x.\n",
						PhyRegTable[Idx].RegName, Offset, RdValue, Value32 & RdMask);
				}
				else
				{
					test_failed2(ERR_READ_ONLY_CLEAR,test_bitString((Value32^Oldcfg) & RdMask),Offset);
				}


				check_abort();

				Retcode = HAS_ERROR;

				PhyWr( Offset, Oldcfg);

				break;
			}

			// Test the read/write bits.
			if(Value32 & WrMask)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("MII register test failed for reg %s at 0x%x.  Expecting "
						"read-write value of 0x%x.  The actual is 0x%x.\n",
						PhyRegTable[Idx].RegName, Offset, 0, Value32 & WrMask);

				}
				else
				{
					test_failed2(ERR_READ_WRITE_NOT_CLEAR,test_bitString((Value32^Oldcfg) & WrMask),Offset);
				}
				check_abort();

				Retcode = HAS_ERROR;

				PhyWr( Offset, Oldcfg);

				break;
			}

			// Write ones to all the bits defined by RdMask and WrMask, then make
			// sure the read-only bits are not changed and the read/write bits are
			// all ones.
			PhyWr( Offset, RdMask | WrMask);
			Value32 = PhyRd(Offset);
			
			// Test the read-only bits.
			if((Value32 & RdMask) != RdValue)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("MII register test failed for reg %s at 0x%x.  Expecting "
						"read-only value of 0x%x.  The actual is 0x%x.\n",
						PhyRegTable[Idx].RegName, Offset, RdValue, Value32 & RdMask);
				}
				else
				{
					test_failed2(ERR_READ_ONLY_SET,test_bitString((Value32^Oldcfg) & RdMask),Offset);
				}
				check_abort();

				Retcode = HAS_ERROR;

				PhyWr( Offset, Oldcfg);

				break;
			}

			// Test the read/write bits.
			if((Value32 & WrMask) != WrMask)
			{
				if (!global.manuf)
				{
					printf("\n");
					i=0;
					printf("MII register test failed for reg %s at 0x%x.  Expecting "
						"read-write value of 0x%x.  The actual is 0x%x.\n",
						PhyRegTable[Idx].RegName, Offset, WrMask, Value32 & WrMask);
				}
				else
				{
					test_failed2(ERR_READ_WRITE_NOT_SET,test_bitString((Value32^Oldcfg) & WrMask),Offset);
				}
				check_abort();

				Retcode = HAS_ERROR;

				PhyWr( Offset, Oldcfg);

				break;
			}

			PhyWr( Offset, Oldcfg);
		}
		if (Retcode == HAS_ERROR)
			break;
	}while ((Retcode == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));
	
	reset();
	if (!global.manuf)
	{
		printf("\n");
		if (Retcode == NO_ERROR) 
			printf("MII Reg test PASSED.\n"); 
		else
			printf("MII Reg test FAILED.\n"); 
		return Retcode;
	}
	else
	{
		if (Retcode == NO_ERROR)
			return test_passed0();
		else
			return Retcode;
	}
}

/******************************************************************************/
/* Description: For SB-based, 4xxx chips, run BIST instead                    */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

U32 SB4xDiagTestIntMemory(u32 req_iteration)
{
	ULONG   save_sbtmstatelow;
	ULONG   dummy, count, maxcount;
	U32     Retcode;
	ULONG local_iteration = 0;

	if (!global.manuf)
		printf("Begin BIST test.\n");

	Retcode = NO_ERROR;
	// save sbtmstatelow 
	save_sbtmstatelow = RegRd( REGOFFSET(sbconfig.sbtmstatelow));

	do{
		local_iteration++;
		if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
		check_abort();
		
		core_reset();

		// start bist with clocks on 
		RegOr( REGOFFSET(sbconfig.sbtmstatelow), SBTML_BE);

		// read back sbtmstatelow check 
		dummy = RegRd( REGOFFSET(sbconfig.sbtmstatelow));
		if (dummy != (SBTML_CLK | SBTML_BE)) 
		{
			if (!global.manuf)
				printf("BIST FAILED: sbtmstatelow failed readback\n");
			else
				test_failed0(ERR_BIST_SBTMSTATE);
			Retcode = HAS_ERROR;
			RegWr( REGOFFSET(sbconfig.sbtmstatelow), save_sbtmstatelow);
		}

		// wait a short while for bist to complete 
		dummy = 0;
		count = 0;
		maxcount = 50;
		while (1) 
		{
			dummy = RegRd( REGOFFSET(sbconfig.sbtmstatehigh));
			if (dummy & 0x80000000)
				break;

			if (++count > maxcount)
				break;

			MM_Wait( 1000 );
		}

		// restore state 
		RegWr( REGOFFSET(sbconfig.sbtmstatelow), save_sbtmstatelow);

		if (count >= maxcount) 
		{
			if (!global.manuf)
				printf("BIST FAILED: did not complete\n");
		    else
				test_failed0(ERR_BIST_NOT_DONE);
			Retcode = HAS_ERROR;
	//              return;
		}

		if (dummy & 0x40000000) 
		{
			if (!global.manuf)
				printf("BIST FAILED\n");
			else
				test_failed0(ERR_BIST);
			Retcode = HAS_ERROR;
	//              return;
		}
		if (Retcode == HAS_ERROR) break;
	}while ((Retcode == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));
	reset();
	if (!global.manuf)
	{
		if (Retcode == NO_ERROR) 
			printf("BIST test PASSED.\n"); 
		else
			printf("BIST test FAILED.\n"); 
		return Retcode;
	}
	else
	{
		if (Retcode == NO_ERROR)
			return test_passed0();
		else
			return Retcode;
	}
}



static void
dmaallocBlast(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG *pTxringaddr,
    ULONG *pRxringaddr,
	ULONG *pTxbufaddr,
    ULONG *pRxbufaddr )
//void Class4xsb::dmaalloc()
{
    int txringsize = 4096;
    int rxringsize = 4096;
	int i;
    
    // allocate tx descriptor ring
    *pTxringaddr = Allocatebuffer( pDmabuf, uDmaBufSize,
	    txringsize ,1);
    // allocate rx descriptor ring
    *pRxringaddr = Allocatebuffer( pDmabuf, uDmaBufSize,
	    rxringsize, 1);
	// allocate a bunch of tx buffers
    for ( i = 0; i < 1; i++ )
    {
	    pTxbufaddr[i] = Allocatebuffer(pDmabuf, uDmaBufSize,
		MAX_ETHERNET_PACKET_SIZE,0 );
    }
    // allocate a bunch of rx buffers
    for ( i = 0; i < NRXDBUFF; i++ )
    {
	    pRxbufaddr[i] = Allocatebuffer(pDmabuf, uDmaBufSize,
		MAX_ETHERNET_PACKET_SIZE,0 );
    }
}

static unsigned int
dmaisalloced(
    ULONG *pTxringaddr,
    ULONG *pRxringaddr )
{
   return (*pTxringaddr != NULL &&  *pRxringaddr != NULL);
}




// initialize the chip for dma
static int
dmainit(
    loopback_t loopback,
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG *pTxringaddr,
    ULONG *pRxringaddr,
	ULONG *pTxbufaddr,
    ULONG *pRxbufaddr,
    ULONG *pRxOut )
{
	int i;
	u16 LinkUp = 0x4;
	u32 last_update_time;
    u32 data;
	u32 bar0win;
	
	
	reset();
	
   
    // if first time, allocate dma resources
    if ( !dmaisalloced( pTxringaddr, pRxringaddr ) )
		dmaallocBlast(pDmabuf, uDmaBufSize, pTxringaddr, pRxringaddr, pTxbufaddr, pRxbufaddr );
		//dmaalloc(pDmabuf, uDmaBufSize, pTxringaddr, pRxringaddr,
		//pRxbufaddr );
	
	// Save Bar0Window 
	read_configuration_dword(CURRCARD.bus_number,
  	                     CURRCARD.device_and_function,
  	                     PCI_CS_BAR0WINDOW ,&bar0win);
    
	// point bar0 at pci core registers
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BAR0WINDOW,SBID_REG_PCI);
    
	//Select SB2PCITRANSLATION (offset 0x108)
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BlackDoorAddr,SB2PCI2);
    
	// Read SB2PCITRANSLATION (offset 0x108)
    read_configuration_dword(CURRCARD.bus_number,
  	                     CURRCARD.device_and_function,
  	                     PCI_CS_BlackDoorData,&data);
    
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BlackDoorAddr,SB2PCI2);
    // enable prefetch and bursts for sonics-to-pci translation 2
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                         PCI_CS_BlackDoorData,data | SBTOPCI_PREF | SBTOPCI_BURST);

    // restore bar0window
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BAR0WINDOW,bar0win);
    // set intrlazy to interrupt on each received frame
    RegWr(REGOFFSET(intrecvlazy), 1 << IRL_FC_SHIFT);

     // enable crc generation
    RegOr(REGOFFSET(emaccontrol), EMC_CG);

    // enable flow control and promisc
    RegWr(REGOFFSET(rxconfig), ERC_FE | ERC_PE);

    if (loopback == ET_MAC) 
	{
		RegOr(REGOFFSET(rxconfig), ERC_LE);
    } 
	else if (loopback == ET_FIFO) 
	{
		RegOr(REGOFFSET(dmaregs.xmtcontrol), XC_LE);
    } 
	else if (loopback == ET_MII) 
	{
		miiwr(0,0x6100);
		/*
		USHORT bmcr;
		bmcr = miird(0 );
			bmcr |= 0x4000;
		miiwr( 0, bmcr );*/
    } 
	else if (loopback == ET_EXT_100)
	{
		miiwr(4,0x1E1);
		miiwr(0,0x1300);
	}
	else if (loopback == ET_EXT_10)
	{
		miiwr(4,0x61);
		miiwr(0,0x1300);
	}

    // set max frame lengths
    RegWr(REGOFFSET(rxmaxlength), MAX_ETHERNET_PACKET_SIZE);
    RegWr(REGOFFSET(txmaxlength), MAX_ETHERNET_PACKET_SIZE);

    // half duplex
	
    //if ((loopback == ET_EXT_100) || (loopback == ET_EXT_10))
	{
       RegWr(REGOFFSET(txcontrol), 1); 
    }
	/*
	else
	{
		RegWr(REGOFFSET(txcontrol), 0);
    }*/

    // set tx watermark (bcm44xx enet ISB burstsize is only 2)
    RegWr(REGOFFSET(txwatermark), 48);

	/* init dma engines */

    // enable tx dma engine
    RegOr(REGOFFSET(dmaregs.xmtcontrol), XC_XE);

    // set the tx ring base address
    RegWr(REGOFFSET(dmaregs.xmtaddr), *pTxringaddr + SB_PCI_DMA);

    _ASSERT( RegRd(REGOFFSET(dmaregs.xmtptr)) == 0 );

    // enable the rx dma channel
    RegWr(REGOFFSET(dmaregs.rcvcontrol), RC_RE | (RXOFF << RC_RO_SHIFT));

    // set the rx ring base address
    RegWr(REGOFFSET(dmaregs.rcvaddr), *pRxringaddr + SB_PCI_DMA);

	_ASSERT( RegRd(REGOFFSET(dmaregs.rcvptr)) == 0 );
	// post a bunch of rx descriptors
    for (i = 0; i < NRXD - 1; i++) {
		postrxd( pDmabuf, uDmaBufSize, *pRxringaddr,
		*pRxOut, pRxbufaddr[i % NRXDBUFF], MAX_ETHERNET_PACKET_SIZE);
		*pRxOut = (*pRxOut + 1) % NRXD;
    }
		
    // last - turn on the emac
    RegOr(REGOFFSET(enetcontrol), EC_EE);
	
	last_update_time=clock();
	if ((loopback == ET_EXT_100) || (loopback == ET_EXT_10))
	{
		while (!(miird(1) & LinkUp))
		{
			if ((clock() - last_update_time) >= 5000)
				return 1;
		}
	}
	return 0;

}
// initialize the chip for dma
static void
dmainitBlast(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG *pTxringaddr,
    ULONG *pRxringaddr,
	ULONG *pTxbufaddr,
    ULONG *pRxbufaddr,
    ULONG *pRxOut )
{
	int i;
	u16 val16_19;
	u16 LinkUp = 0x4;
	U32 data;
	U32 bar0win;
	pTX_TEST_INFO txInfo = &CURRCARD.txInfo;
    // if first time, allocate dma resources
    if ( !dmaisalloced( pTxringaddr, pRxringaddr ) )
		dmaallocBlast(pDmabuf, uDmaBufSize, pTxringaddr, pRxringaddr, pTxbufaddr, pRxbufaddr );
    

    // Save Bar0Window 
	read_configuration_dword(CURRCARD.bus_number,
  	                     CURRCARD.device_and_function,
  	                     PCI_CS_BAR0WINDOW ,&bar0win);
    
	// point bar0 at pci core registers
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BAR0WINDOW,SBID_REG_PCI);
    
	//Select SB2PCITRANSLATION (offset 0x108)
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BlackDoorAddr,SB2PCI2);
    
	// Read SB2PCITRANSLATION (offset 0x108)
    read_configuration_dword(CURRCARD.bus_number,
  	                     CURRCARD.device_and_function,
  	                     PCI_CS_BlackDoorData,&data);
    
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BlackDoorAddr,SB2PCI2);
    // enable prefetch and bursts for sonics-to-pci translation 2
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                         PCI_CS_BlackDoorData,data | SBTOPCI_PREF | SBTOPCI_BURST);

    // restore bar0window
	write_configuration_dword(CURRCARD.bus_number,
                          CURRCARD.device_and_function,
                          PCI_CS_BAR0WINDOW,bar0win);
	
	// set intrlazy to interrupt on each received frame
    RegWr(REGOFFSET(intrecvlazy), 1 << IRL_FC_SHIFT);

    // set EMACXmtMaxBurstLen
    RegWr(REGOFFSET(emactxmaxburstlen), 0x40);

     // enable crc generation
    RegOr(REGOFFSET(emaccontrol), EMC_CG);

    // enable flow control and promisc
    RegWr(REGOFFSET(rxconfig), ERC_FE | ERC_PE);

    // set max frame lengths
    RegWr(REGOFFSET(rxmaxlength), MAX_ETHERNET_PACKET_SIZE);
    RegWr(REGOFFSET(txmaxlength), MAX_ETHERNET_PACKET_SIZE);

	val16_19 = ((miird(0x19) & 0x700) >> 8);
	miird(1);
	if (miird(1) & LinkUp)
	{
		if ((val16_19 == 1) || (val16_19 == 3))
		{
			// Half duplex
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) & ~TC_FD);
		}
		else if ((val16_19 == 2) || (val16_19 == 5))
		{
			// Full duplex
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) | TC_FD);
		}
	}

    // set tx watermark (bcm44xx enet ISB burstsize is only 2)
    RegWr(REGOFFSET(txwatermark), 48);

    // set MIB Control Reg
    //RegWr(REGOFFSET(mibcontrol), 1);

	/* init dma engines */

	if (txInfo->host_loopback)
		// enable tx dma engine
		RegOr(REGOFFSET(dmaregs.xmtcontrol), XC_XE | XC_FPRI);
	else    
		// enable tx dma engine
		RegOr(REGOFFSET(dmaregs.xmtcontrol), XC_XE);

    // set the tx ring base address
    RegWr(REGOFFSET(dmaregs.xmtaddr), *pTxringaddr + SB_PCI_DMA);

    // enable the rx dma channel
    RegWr(REGOFFSET(dmaregs.rcvcontrol), RC_RE | (RXOFF << RC_RO_SHIFT));

    // set the rx ring base address
    RegWr(REGOFFSET(dmaregs.rcvaddr), *pRxringaddr + SB_PCI_DMA);

    // post a bunch of rx descriptors
    for (i = 0; i < NRXD - 1; i++) {
		postrxd( pDmabuf, uDmaBufSize, *pRxringaddr,
		*pRxOut, pRxbufaddr[i % NRXDBUFF], MAX_ETHERNET_PACKET_SIZE);
		*pRxOut = (*pRxOut + 1) % NRXD;
    }
	
    // last - turn on the emac
    RegOr(REGOFFSET(enetcontrol), EC_EE);
}

static void
postrxd(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG pciaddr,
    ULONG index,
    ULONG bufaddr,
    ULONG flags )
{
    dmadd_t rxd;
    int offset;

    offset = index * sizeof(dmadd_t);

    // Set the end of receive flag if this is the last rxd
    if (index == (ULONG) NRXD - 1) 
		flags |= CTRL_EOT;
    
    // create the descriptor
    rxd.ctrl = flags;
    rxd.addr = (U32) bufaddr + SB_PCI_DMA;
    
    WriteBuffer(pDmabuf, uDmaBufSize, pciaddr, (char *)&rxd,
	    sizeof(dmadd_t), offset);
    
    // update the chip RcvPtr register
    // XXX since we fill the rxring with buffers
    // for 1-to-1 association of rxd and rxbuf,
    // don't increment the offset here
    RegWr(REGOFFSET(dmaregs.rcvptr),  ((index+1) % NRXD) * sizeof(dmadd_t));
    //_ASSERT(RegRd(REGOFFSET(dmaregs.rcvptr)) == ((index+1)%NRXD) * sizeof(dmadd_t));
}

static void
dmafreeBlast(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG *pTxringaddr,
    ULONG *pRxringaddr,
    ULONG *pRxbufaddr, 
	ULONG *pTxbufaddr)
{
    int i;
	if (!dmaisalloced( pTxringaddr, pRxringaddr ))
		return;

    // turn OFF the dma engines
    
	//reset();

    // free tx descriptor ring
    Deallocatebuffer( pDmabuf, uDmaBufSize, *pTxringaddr );
    *pTxringaddr = NULL;

    // free rx descriptor ring
    Deallocatebuffer( pDmabuf, uDmaBufSize, *pRxringaddr );
    *pRxringaddr = NULL;
	// free tx buffers
    for (i = 0; i < 1; i++) {
		Deallocatebuffer(pDmabuf, uDmaBufSize, pTxbufaddr[ i ] );
		pTxbufaddr[ i ] = NULL;
    }
    // free rx buffers
    for (i = 0; i < NRXDBUFF-1; i++) {
		Deallocatebuffer(pDmabuf, uDmaBufSize, pRxbufaddr[ i ] );
		pRxbufaddr[ i ] = NULL;
    }
}

void
core_disable()
{
//      HANDLE Handle = handle;
	ULONG   dummy;

	// disable the core if it is active 
	if (RegRd( REGOFFSET(sbconfig.sbtmstatelow)) & SBTML_RESET)
		return;

	// set the reject bit 
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), (SBTML_CLK | SBTML_REJ));

	// spin until reject is set 
	while ((RegRd( REGOFFSET(sbconfig.sbtmstatelow)) & SBTML_REJ) == 0)
		MM_Wait(1000);

	// spin until sbtmstatehigh.busy is clear 
	while (RegRd( REGOFFSET(sbconfig.sbtmstatehigh)) & SBTMH_BUSY)
		MM_Wait(1000);

	// set reset and reject while enabling the clocks 
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), (SBTML_FGC | SBTML_CLK | SBTML_REJ | SBTML_RESET));
	dummy = RegRd( REGOFFSET(sbconfig.sbtmstatelow));
	MM_Wait(1000);

	// leave reset and reject asserted 
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), (SBTML_REJ | SBTML_RESET));
}


void core_reset()
{
	ULONG   dummy;

	core_disable();

	/*
	 * Now do the initialization sequence.
	 */

	// set reset while enabling the clock and forcing them on throughout the core */
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), (SBTML_FGC | SBTML_CLK | SBTML_RESET));
	dummy = RegRd( REGOFFSET(sbconfig.sbtmstatelow));

	// clear reset and allow it to propagate throughout the core */
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), (SBTML_FGC | SBTML_CLK));
	dummy = RegRd( REGOFFSET(sbconfig.sbtmstatelow));

	// turn clock on */
	RegWr( REGOFFSET(sbconfig.sbtmstatelow), SBTML_CLK);
}

void reset()
{
	
	/* reset the dma engines */
	RegWr( REGOFFSET(dmaregs.xmtcontrol), 0);
	RegWr( REGOFFSET(dmaregs.rcvcontrol), 0);

	core_reset();
    
    // some mib counters must be cleared by hand
    RegWr( REGOFFSET(mibcontrol), 1);
    RegRd( REGOFFSET(mib.tx_good_pkts));
    RegRd( REGOFFSET(mib.tx_pkts));
    RegRd( REGOFFSET(mib.rx_good_pkts));
    RegRd( REGOFFSET(mib.rx_pkts));
    RegWr( REGOFFSET(mibcontrol), 0);
	    

	/* some chips have internal phy, some don't */
	if (!(RegRd( REGOFFSET(devcontrol)) & DC_IP)) {
		RegWr( REGOFFSET(enetcontrol), EC_EP);
	} else if (RegRd( REGOFFSET(devcontrol)) & DC_ER) {
		RegAnd( REGOFFSET(devcontrol), ~DC_ER);
		MM_Wait(10000);
	}
	
	/* XXX: hard-coded 4402/4401 value */
	RegWr( REGOFFSET(mdiocontrol), 0x8d);

    miireset();
}

//  ephy soft reset
void miireset()
//void Class4xsb::miireset()
{
    USHORT tmp;
    miiwr(0, 0x8000 );
    MM_Wait(1000);
    _ASSERT((miird(  0 ) & 0x8000) == 0);
	
    //  enable activity led
    tmp = miird(26);
    tmp &= ~0x8000;
    miiwr(  26, tmp );
}

// read arbitrary ephy register
U16 miird( UCHAR r )
{
    ULONG data;
    int count;
	int ephyunit;
    //_ASSERT( r < 32 );

    // clear mii_int
    RegWr( REGOFFSET(emacintstatus), EI_MII);
    //_ASSERT( (RegRd( REGOFFSET(emacintstatus)) & EI_MII) == 0);
	ephyunit = 1;
    // issue the read
    data = MD_SB_START | MD_OP_READ;
    data |= (ephyunit << MD_PMD_SHIFT);
    data |= (r << MD_RA_SHIFT);
    data |= MD_TA_VALID ;

    RegWr( REGOFFSET(mdiodata), data);

    count = 0;
    while (!(RegRd( REGOFFSET(emacintstatus)) & EI_MII) && (count++ < 60)) {
	MM_Wait(10000);
    }
    _ASSERT(RegRd( REGOFFSET(emacintstatus)) & EI_MII);
    
    return (USHORT) RegRd( REGOFFSET(mdiodata));
}

// write arbitrary ephy register
void miiwr( UCHAR r, USHORT val )
//void Class4xsb::miiwr (UCHAR r, USHORT val) 
{
//      HANDLE Handle = handle;
    ULONG data;
    int count;
	int ephyunit;
    _ASSERT(r < 32);

    // clear mii_int
    RegWr( REGOFFSET(emacintstatus), EI_MII);
    _ASSERT((RegRd( REGOFFSET(emacintstatus)) & EI_MII) == 0);
	ephyunit = 1;
    data = MD_SB_START | MD_OP_WRITE ;
    data |= (ephyunit << MD_PMD_SHIFT); 
    data |= (r << MD_RA_SHIFT);
    data |= MD_TA_VALID | val;

    RegWr( REGOFFSET(mdiodata),  data);

    count = 0;
    while (!(RegRd( REGOFFSET(emacintstatus)) & EI_MII) && (count++ < 60)) {
		MM_Wait(10000);
    }
    _ASSERT(RegRd( REGOFFSET(emacintstatus)) & EI_MII);
}

//// fillbuf - initialize binary string with monotonically-increasing data pattern
/*
static void fillbuf(void *addr, ULONG len )
{
    unsigned char *var = (unsigned char *) addr;
	int i;
	
    for (i = 0; i < (int)len; i++) {
		var[i] = i & 0xff;
    }
}
*/
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
U32
SB4xDiagTestLoopBack(u32 req_iteration, ULONG Lbtype, u32 testpack, u32 type)
{

    int                         txlen, len;
	//int							rc;
    char                        txbuf[2000];
    char                        rxbuf[2000];
	loopback_t              loopback;
	//int                             wantlen;
	U32                             status = NO_ERROR;
	int             j, i;

    u32				count;
    BUFENTRY        dmabuf[50];
    ULONG           txringaddr = NULL, rxringaddr = NULL;
    ULONG           rxbufaddr[NRXDBUFF];
	ULONG           txbufaddr[1];
    ULONG           uTxIn = 0, uTxOut = 0, uRxIn = 0, uRxOut = 0;
	pBlast                  dev = &CURRCARD.blast;
	PSWMIBCOUNTERS swmib= &CURRCARD.swmibcounts;
    unsigned int    bRet;
	
    ULONG local_iteration = 0;


    if((Lbtype != LOOPBACK_TYPE_PHY) && (Lbtype != LOOPBACK_TYPE_MAC) && (Lbtype != LOOPBACK_TYPE_EXT_100) && (Lbtype != LOOPBACK_TYPE_EXT_10))
    {
		if (!global.manuf)
		{
			printf("Not implemented (INVALID_PARAMETER).\n");
			printf("use -m for Mac loopback; -p for MII loopback\n");
			printf("use -e for Ext loopback 100BT; -a for Ext loopback 10BT\n");
		}
		return ERR_LOOPBACK_TYPE;
    }
	if (testpack <= 0) testpack = 2000;
	do 
		{
			local_iteration++;
			if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
			check_abort();
			if (!global.manuf)
			{
				printf("Loopback Test Begin...\n");
			}
			txlen = 1514;
			// init a txbuffer
   
			for (j = 0; j < 6; j++)
			{
				txbuf[j] = 0xff;
				txbuf[6+j] = dev->pDevice->PermanentNodeAddress[j];
			}    
			
			*(u16 *)(txbuf + 12) = BYTE_SWAP_S(DIAG_MAGIC_PACKET_TYPE);
			*(u16 *)(txbuf + 14) = BYTE_SWAP_S(1514);
			*(u32 *)(txbuf + 16) = BYTE_SWAP(1);
			for (j = 0; j < (MAX_ETHERNET_PACKET_SIZE_NO_CRC-20); j++)
			{
				if (type==0)
					txbuf[20+j] = 0x00;
				else if (type==1)
					txbuf[20+j] = 0xFF;
				else if (type==2)
					txbuf[20+j] = 0x55;
				else if (type==3)
					txbuf[20+j] = 0xAA;
				else if (type==4)
					txbuf[20+j] = 0x0F;
				else if (type==5)
					txbuf[20+j] = 0xF0;
				else if (type==6)
				{
					txbuf[20+j] = 0xFF;
				    j++;
					txbuf[20+j] = 0x00;
				}
				else if (type==7)
				{
					txbuf[20+j] = 0x00;
				    j++;
					txbuf[20+j] = 0xFF;
				}
				else if (type==8)
				{
					txbuf[20+j] = 0xFF;
				    j++;
					txbuf[20+j] = 0xFF;
					j++;
					txbuf[20+j] = 0x00;
				    j++;
					txbuf[20+j] = 0x00;
				}
				else if (type==9)
				{
					txbuf[20+j] = 0x00;
				    j++;
					txbuf[20+j] = 0x00;
					j++;
					txbuf[20+j] = 0xFF;
				    j++;
					txbuf[20+j] = 0xFF;
				}
				else if (type == 10)
					txbuf[20+j] = j;
				else
					txbuf[20+j] = rand() & 0xFF;
			}


			if ( Lbtype == LOOPBACK_TYPE_PHY )
				loopback = ET_MII;  /* deepest level: phy loopback */
			else if ( Lbtype == LOOPBACK_TYPE_MAC )
				loopback = ET_MAC;  /* mac loopback */
			else if ( Lbtype == LOOPBACK_TYPE_EXT_100 )
				loopback = ET_EXT_100;
			else if ( Lbtype == LOOPBACK_TYPE_EXT_10 )
				loopback = ET_EXT_10;


			// init the chip in dma mode, emac-level loopback
			memset( &dmabuf, 0, sizeof( dmabuf ) );
			if(dmainit(loopback, dmabuf, sizeof( dmabuf ), &txringaddr, &rxringaddr,txbufaddr, rxbufaddr, &uRxOut ))
			{
				if (!global.manuf)
				{
					printf("Link Error.\n");
				}
				return ERR_NO_LINK;
			}
			if ((txringaddr == 0) || (rxringaddr == 0) )
			{
				if (!global.manuf)
				{
					printf("Memory Allocation Error.\n");
				}
				return ERR_MALLOC;
			}
			for (i = 0 ; i < NRXDBUFF; i++)
			{
				 
				if (rxbufaddr[i] == 0)
				{
					if (!global.manuf)
					{
						printf("Memory Allocation Error.\n");
					}
					return ERR_MALLOC;
				}
			}
			for (i = 0 ; i < 1; i++)
			{
				 
				if (txbufaddr[i] == 0)
				{
					if (!global.manuf)
					{
						printf("Memory Allocation Error.\n");
					}
					return ERR_MALLOC;
				}
			}

			WriteBuffer(dmabuf, sizeof( dmabuf ), txbufaddr[0], &txbuf[0], txlen, 0 );
			resetcountersBlast();
			for (count = 0; count < testpack; count++)
			{
				if (kbhit())
				{
					if (toupper(getch())==KEY_ESC)
						status = ERR_ABORT;
						break;
				}
				   
				// transmit it
				bRet = txdma( dmabuf, sizeof( dmabuf ), &txbufaddr[0], txlen, &uTxIn, &uTxOut, &txringaddr );
				//if (Lbtype == LOOPBACK_TYPE_PHY) MM_Wait(1000);
				// read it back
				len  = rxdma(dmabuf, sizeof( dmabuf ), rxbuf, &rxringaddr, rxbufaddr, &uRxIn, &uRxOut );
				if (count > 2000) readcountersBlast(1);
				
			}
				
			readcountersBlast(1);

			// release dma resources
			dmafreeBlast( dmabuf, sizeof( dmabuf ), &txringaddr, &rxringaddr, rxbufaddr , txbufaddr);

			if ((swmib->rx_crc_errs_H * 0x10000 +swmib->rx_crc_errs_L) != 0)
			{
				if (!global.manuf)
					printf("Loopback test RX packet with CRC Error. %d\n",swmib->rx_crc_errs_H * 0x10000 +swmib->rx_crc_errs_L );
				status = ERR_BAD_CRC;
			}

			else if ((swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L) != testpack)
			{
				if (!global.manuf)
					printf("Loopback test txdma() failed %10lu \n" ,swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				status = ERR_TXDMA;
			}
			else if ((swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L) != testpack)
			{
				if (!global.manuf)
					printf("Loopback test Rxdma() failed. %10lu \n" ,swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				status = ERR_RXDMA;
			}
			
			else
			status = NO_ERROR;

			reset();

			if(!global.manuf)
				printf( "Loopback Test Completed.\n\n" );
	}while ((status == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));



	return status;


}

#define RXH_ERROR       (RXF_LG | RXF_NO | RXF_RXER | RXF_CRC | RXF_OV)

// read the next received packet into the given buffer and return
// the length of the received data.  Read at most maxlen data.

static int
rxdma(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    char *outbuf,
    //int maxlen,
    //int *error,
    ULONG *pRxringaddr,
    ULONG *pRxbufaddr,
    ULONG *pRxIn,
    ULONG *pRxOut )
{
	int i;
	RegWr(REGOFFSET(intstatus), I_RI);
	for (i = 0; i < 5; i++)
	{
		postrxd( pDmabuf, uDmaBufSize, *pRxringaddr, *pRxOut, pRxbufaddr[*pRxOut % NRXDBUFF], MAX_ETHERNET_PACKET_SIZE);
		*pRxOut = (*pRxOut +1) % NRXD;
		*pRxIn = (*pRxIn +1) % NRXD;
	}
	return 0;

}
// DMA write the buffer
// references the global associative array "rei"
static int
rxdmaBlast(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    char *outbuf,
    int maxlen,
    int *error,
    ULONG *pRxringaddr,
    ULONG *pRxbufaddr,
    ULONG *pRxIn,
    ULONG *pRxOut)

{
	int i;
	
    RegWr(REGOFFSET(intstatus), I_RI);

	//{
	for (i = 0; i < (NRXD -1); i ++)
	{
		postrxd( pDmabuf, uDmaBufSize, *pRxringaddr, *pRxOut, pRxbufaddr[*pRxOut % NRXDBUFF], MAX_ETHERNET_PACKET_SIZE);
		*pRxOut = (*pRxOut +1) % NRXD;
		*pRxIn = (*pRxIn +1) % NRXD;
	}
	
    return 0;
}
static int
rxdmaBlastHostBack(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    char *outbuf,
    int maxlen,
    int *error,
    ULONG *pRxringaddr,
    ULONG *pRxbufaddr,
    ULONG *pRxIn,
    ULONG *pRxOut)

{

    
    //bcmenetrxh_t rxhdr;
	unsigned int status;
    RegWr(REGOFFSET(intstatus), I_RI);

	// post the rxd that is the hole.
	postrxd( pDmabuf, uDmaBufSize, *pRxringaddr,
	*pRxOut, pRxbufaddr[*pRxOut % NRXDBUFF], MAX_ETHERNET_PACKET_SIZE);
	*pRxOut = (*pRxOut + 1) % NRXD;

    // get packet 
    status = ReadBuffer(pDmabuf, uDmaBufSize, pRxbufaddr[*pRxIn % NRXDBUFF],(char *)outbuf,maxlen, 0);

	*pRxIn = (*pRxIn + 1) % NRXD;
    
    return status;
}
static int
txdma(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
	ULONG *pTxBufaddr,
    int nbytes,
    ULONG *pTxIn,
    ULONG *pTxOut,
    ULONG *pTxringaddr )
{
    int txbufsize, resid, i, n, offset;
    ULONG flags;

    txbufsize = MAX_ETHERNET_PACKET_SIZE ;

    resid = nbytes;
    i = 0;
    
    while (resid > 0) {
		n = MIN(resid, txbufsize);
		offset = i * txbufsize;
		flags = 0;
		
		resid -= n;

		// compute the tx descriptor flags value
		if (i == 0) {
			flags |= CTRL_SOF;
		}

		if (resid == 0) {
			flags |= (CTRL_EOF | CTRL_IOC);
		}
		
		// Set the end of transmit flag if this is the last txd
		if ((*pTxOut == NTXD - 1)) {
			flags |= CTRL_EOT;
		}
		flags |= n;

		// post one tx descriptor
		posttxd(  pDmabuf, uDmaBufSize, *pTxringaddr, *pTxOut, pTxBufaddr[0], flags );
		
		// increment txout index
		*pTxOut = (*pTxOut + 1) % NTXD;
		i++;
    } 
    
    // update the chip XmtPtr register.
    RegWr(REGOFFSET(dmaregs.xmtptr), (*pTxOut % NTXD) * sizeof(dmadd_t));
    
    // wait for tx to complete
	MM_Wait(1400);
    *pTxIn = *pTxOut;

    RegWr(REGOFFSET(intstatus), I_XI);
    
    return( TRUE );
}
static int
MAX (int int1, int int2)
{
	if (int1 >= int2)
		return int1;
	else
		return int2;
}
static int
txdmaBlast(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    char buf[],
    int nbytes,
	ULONG *pTxBufaddr,
    ULONG *pTxIn,
    ULONG *pTxOut,
    ULONG *pTxringaddr,
	ULONG ipg,
	ULONG nPack)
{
	int  i;
	ULONG n;
    ULONG flags = (CTRL_SOF | CTRL_EOF | CTRL_IOC | nbytes);
    U32 last_updateCounter_time;
	U32 last_update_time;
    int blocksize = 120;
	//int ipg_start_time;

	last_update_time=clock(); 
	last_updateCounter_time=clock();   
	
	WriteBuffer( pDmabuf, uDmaBufSize, pTxBufaddr[0], &buf[0], nbytes, 0 );
    
	for (i = 0; i < NTXD ; i++)
	{
		if ((i == NTXD - 1)) 
			flags |= CTRL_EOT;
		else
			flags = (CTRL_SOF | CTRL_EOF | CTRL_IOC | nbytes);

		posttxd(  pDmabuf, uDmaBufSize, *pTxringaddr, i, pTxBufaddr[0], flags );

	}

	if (nPack == 0)
		for (;;)
		{
			i = (RegRd(REGOFFSET(dmaregs.xmtstatus)) & XS_CD_MASK) / sizeof(dmadd_t);
			
			if (*pTxOut == i)
				n=blocksize;
			else if (*pTxOut<i)
				n=(blocksize - (NTXD-1-i) - *pTxOut);
			else
				n=(blocksize - (*pTxOut -i));
			
			*pTxOut = (*pTxOut + n) % NTXD;
			
			RegWr(REGOFFSET(dmaregs.xmtptr), (*pTxOut % NTXD) * sizeof(dmadd_t));
            
			//To prevent readcounters and displaycounters interrupt rec too long. 
			if (clock() >=  (last_update_time + 100) )
			{
				readcountersBlast(0);
				last_update_time=clock();
			}
			
			if (clock() >=  (last_updateCounter_time + 5000) )
			{
				displaycountersBlast(1);
				last_updateCounter_time=clock();
			}

			if((RegRd(REGOFFSET(dmaregs.xmtstatus)) & XS_CD_MASK) == (*pTxOut * sizeof(dmadd_t)))
			{
				if ( !checkinterror() )
					goto done;
			}
			if (kbhit())
			{
				if (toupper(getch())==KEY_ESC)
					goto done;
			}
			
			
		}
	else
	{
		while (nPack > 0)
		{
			i = (RegRd(REGOFFSET(dmaregs.xmtstatus)) & XS_CD_MASK) / sizeof(dmadd_t);
			
			if (*pTxOut == i)
				n=MIN(blocksize,nPack);
			else if (*pTxOut<i)
				n=MIN((blocksize - (NTXD-1-i) - *pTxOut),nPack);
			else
				n=MIN((blocksize - (*pTxOut -i)),nPack);
			
			*pTxOut = (*pTxOut + n) % NTXD;
			
			RegWr(REGOFFSET(dmaregs.xmtptr), (*pTxOut % NTXD) * sizeof(dmadd_t));
			//To prevent readcounters and displaycounters interrupt rec too long. 
			if (clock() >=  (last_update_time + 100) )
			{
				readcountersBlast(0);
				last_update_time=clock();
			}
			
			if (clock() >=  (last_updateCounter_time + 5000) )
			{
				displaycountersBlast(1);
				last_updateCounter_time=clock();
			}
			nPack=MAX(0,nPack-n);
			if((RegRd(REGOFFSET(dmaregs.xmtstatus)) & XS_CD_MASK) == (*pTxOut * sizeof(dmadd_t)))
			{
				if ( !checkinterror() )
					goto done;
			}
			if (kbhit())
			{
				if (toupper(getch())==KEY_ESC)
					goto done;
			}
			
		}
	}               
    *pTxIn = *pTxOut;
done:

    RegWr(REGOFFSET(intstatus), I_XI);
    
    return( TRUE );
}

// check for IntStatus errors
//static unsigned int checkinterror(void)
static int
checkinterror()
{
    if (RegRd(REGOFFSET(intstatus)) & ~(I_RI | I_XI| I_EM))
    {
			printf( "Error on receive, instatus is 0x%x\r\n",
			RegRd( REGOFFSET( intstatus ) ) );
		return( FALSE );
    }

    return( TRUE );
}

// create and post a tx descriptor to the chip
static void
posttxd(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    ULONG pciaddr,
    int index,
    ULONG bufaddr,
    ULONG flags )
{
    dmadd_t txd;
    ULONG offset;

    offset = index * sizeof(dmadd_t);

    // create the descriptor
    txd.ctrl = flags;
    txd.addr = (U32) bufaddr + SB_PCI_DMA;

    WriteBuffer( pDmabuf, uDmaBufSize, pciaddr, (char *)&txd,
	    sizeof(dmadd_t), offset );
}

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
/*
U32
SB4xDiagTestCPU(
    
    CHAR *pRstr )
//void 
//Class4xsb::SB4xDiagTestCPU(Pvoid)
{
    LogMsg(BMAPI_DEBUG_INFORMATION, "### SB4xDiagTestCPU\n");

	// not supported on SB4X enet 
	return BMAPI_OK;
}
*/


/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/

U32 SB4xDiagTestLEDs( U32 req_iteration, ULONG BlinkDurationSec )
{
    ULONG OldPhyCtl;
    ULONG OldPhyAuxCtl;
	ULONG OldPhyAux2;
    ULONG j;
	U32     Retcode;
	ULONG local_iteration = 0;

    if(BlinkDurationSec == 0)
		return NO_ERROR;
    

    if (BlinkDurationSec > 120)
		BlinkDurationSec = 120;
    
	if(!global.manuf)
		printf("Begin LED test.\n");

	Retcode = NO_ERROR;
	do 
    {
		local_iteration++;
		if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
		check_abort();
		
		reset();

		/* 
		 * unfortunately can't support phy LED blink; mii shadow reg 0x1a
		 * would do it but it is not hooked up inside the 4402. 
		 * can do forcelink, force100, forceactivity, however 
		 */
		 
		OldPhyCtl = PhyRd(0x00);
		OldPhyAuxCtl = PhyRd(0x18);
		OldPhyAux2 = PhyRd(0x1b);

		for(j = 0; j < BlinkDurationSec * 2; j++)
		{
			if(j % 2)
			{
				// Turn on the LEDs.
				PhyWr( 0x00, OldPhyCtl | (1 << 13));
				PhyWr( 0x18, OldPhyAuxCtl | (1 << 14));
				PhyWr( 0x1b, OldPhyAux2 | (1 << 5));
			}
			else
			{
				// Turn off the LEDs.
				PhyWr( 0x00, OldPhyCtl);
				PhyWr( 0x18, OldPhyAuxCtl);
				PhyWr( 0x1b, OldPhyAux2);
			}

			MM_Wait(5000);
		}

    }while ((Retcode == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));

	reset();
	if(!global.manuf)
		printf("LED test Completed.\n"); 

	return Retcode;
}

/*******************************************************************************
 * crc8
 *
 * Computes a crc8 over the input data using the polynomial:
 *
 *       x^8 + x^7 +x^6 + x^4 + x^2 + 1
 *
 * The caller provides the initial value (either CRC8_INIT_VALUE
 * or the previous returned value) to allow for processing of 
 * discontiguous blocks of data.  When generating the CRC the
 * caller is responsible for complementing the final return value
 * and inserting it into the byte stream.  When checking, a final
 * return value of CRC8_GOOD_VALUE indicates a valid CRC.
 *
 * Reference: Dallas Semiconductor Application Note 27
 *   Williams, Ross N., "A Painless Guide to CRC Error Detection Algorithms", 
 *     ver 3, Aug 1993, ross@guest.adelaide.edu.au, Rocksoft Pty Ltd.,
 *     ftp://ftp.rocksoft.com/clients/rocksoft/papers/crc_v3.txt
 *
 ******************************************************************************/

static U8 crc8_table[256] = {
    0x00, 0xF7, 0xB9, 0x4E, 0x25, 0xD2, 0x9C, 0x6B,
    0x4A, 0xBD, 0xF3, 0x04, 0x6F, 0x98, 0xD6, 0x21,
    0x94, 0x63, 0x2D, 0xDA, 0xB1, 0x46, 0x08, 0xFF,
    0xDE, 0x29, 0x67, 0x90, 0xFB, 0x0C, 0x42, 0xB5,
    0x7F, 0x88, 0xC6, 0x31, 0x5A, 0xAD, 0xE3, 0x14,
    0x35, 0xC2, 0x8C, 0x7B, 0x10, 0xE7, 0xA9, 0x5E,
    0xEB, 0x1C, 0x52, 0xA5, 0xCE, 0x39, 0x77, 0x80,
    0xA1, 0x56, 0x18, 0xEF, 0x84, 0x73, 0x3D, 0xCA,
    0xFE, 0x09, 0x47, 0xB0, 0xDB, 0x2C, 0x62, 0x95,
    0xB4, 0x43, 0x0D, 0xFA, 0x91, 0x66, 0x28, 0xDF,
    0x6A, 0x9D, 0xD3, 0x24, 0x4F, 0xB8, 0xF6, 0x01,
    0x20, 0xD7, 0x99, 0x6E, 0x05, 0xF2, 0xBC, 0x4B,
    0x81, 0x76, 0x38, 0xCF, 0xA4, 0x53, 0x1D, 0xEA,
    0xCB, 0x3C, 0x72, 0x85, 0xEE, 0x19, 0x57, 0xA0,
    0x15, 0xE2, 0xAC, 0x5B, 0x30, 0xC7, 0x89, 0x7E,
    0x5F, 0xA8, 0xE6, 0x11, 0x7A, 0x8D, 0xC3, 0x34,
    0xAB, 0x5C, 0x12, 0xE5, 0x8E, 0x79, 0x37, 0xC0,
    0xE1, 0x16, 0x58, 0xAF, 0xC4, 0x33, 0x7D, 0x8A,
    0x3F, 0xC8, 0x86, 0x71, 0x1A, 0xED, 0xA3, 0x54,
    0x75, 0x82, 0xCC, 0x3B, 0x50, 0xA7, 0xE9, 0x1E,
    0xD4, 0x23, 0x6D, 0x9A, 0xF1, 0x06, 0x48, 0xBF,
    0x9E, 0x69, 0x27, 0xD0, 0xBB, 0x4C, 0x02, 0xF5,
    0x40, 0xB7, 0xF9, 0x0E, 0x65, 0x92, 0xDC, 0x2B,
    0x0A, 0xFD, 0xB3, 0x44, 0x2F, 0xD8, 0x96, 0x61,
    0x55, 0xA2, 0xEC, 0x1B, 0x70, 0x87, 0xC9, 0x3E,
    0x1F, 0xE8, 0xA6, 0x51, 0x3A, 0xCD, 0x83, 0x74,
    0xC1, 0x36, 0x78, 0x8F, 0xE4, 0x13, 0x5D, 0xAA,
    0x8B, 0x7C, 0x32, 0xC5, 0xAE, 0x59, 0x17, 0xE0,
    0x2A, 0xDD, 0x93, 0x64, 0x0F, 0xF8, 0xB6, 0x41,
    0x60, 0x97, 0xD9, 0x2E, 0x45, 0xB2, 0xFC, 0x0B,
    0xBE, 0x49, 0x07, 0xF0, 0x9B, 0x6C, 0x22, 0xD5,
    0xF4, 0x03, 0x4D, 0xBA, 0xD1, 0x26, 0x68, 0x9F
};


/* crc defines */



#define CRC_INNER_LOOP(n, c, x) \
    (c) = ((c) >> 8) ^ crc##n##_table[((c) ^ (x)) & 0xff]
u8 crc8(
    u8 *pdata,  /* pointer to array of data to process */
    u32 nbytes, /* number of input data bytes to process */
    u8 crc      /* either CRC8_INIT_VALUE or previous return value */
)
{
    while (nbytes-- > 0)
	CRC_INNER_LOOP(8, crc, *pdata++);
    return crc;
}
/*
U8 crc8(U8 *pdata,      
	U32  nbytes,    
	U8 crc  
)
{
	// hard code the crc loop instead of using CRC_INNER_LOOP macro
	// to avoid the undefined and unnecessary (U8 >> 8) operation. 
	while (nbytes-- > 0)
		crc = crc8_table[(crc ^ *pdata++) & 0xff];

	return crc;
}
*/
/* size in long words */
#define SPROM_SIZE              64
#define NTOH32(p)                   ((ULONG) ((*((PUCHAR) (p)) << 24) |     \
				    (*((PUCHAR) (p)+1) << 16) |             \
				    (*((PUCHAR) (p)+2) << 8) |              \
				    *((PUCHAR) (p)+3)))

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
U32 SB4xDiagprgEEPROM(u8* Macbuf)

{
	int             i = 0, j=0;
    u16             sprom_size = SROM_SIZE;
	u16             *image;                 // In-core copy of the srom 
	u32             val32;
	u8              crc;
	int             status;
	u32             tempread;
	u16 *           tempaddr;


	
	image = (uint16 *)malloc(sprom_size*2);
	
	for (i = 0; i < sprom_size/2; i++) 
	{
		tempread = RegRd( SPROM_BASE + (i * 4));
		image[j] = tempread & 0xffff;
		j++;
		image[j] = (tempread & 0xffff0000) / 0x10000;
		j++;
	}



	if ((global.optMacCmdLine) || (global.optMacProg))
	{
		if ((Macbuf[0] == 0) &&  (Macbuf[1] == 0xFF) &&  (Macbuf[2] == 0xFF))
		{
			image[40] = (image[40] & 0xFF00) | Macbuf[3];
			image[41] = (Macbuf[4] * 0x100) | Macbuf[5];
		}
		else
		{
			for (i = 0 ; i < 3; i++)
			{
				image[ i + 39] = (Macbuf[i*2] * 0x100) | Macbuf[i*2+1];
			}
		}
    }

    if (global.optBrom)
	{
		//disable brom
		image[SROM_COMMPW] &= ~(0x0100);
		image[SROM_COMMPW] &= ~(0xFE00);
		if (global.romsize > 0) // enable brom
		{
			image[SROM_COMMPW] |= (1 << BR_PRESSHIFT);
			image[SROM_COMMPW] |= ((global.romsize-1) << BR_SIZESHIFT);
		}
	}

	// Calculate crc's XXX: Assumes little-endian host 
	crc = ~crc8((u8 *)image, SROM_BYTES - 1, CRC8_INIT_VALUE);
	image[SROM_SIZE - 1] = (crc << 8) | (image[SROM_SIZE - 1] & 0xff);
	
    
	
	
	printf("Enabling SPROM writes.\n");

	read_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,&val32);
	write_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,val32 | SROM_WRITEABLE);
   
	MM_Wait(5000);  
	for (i = 0; i < sprom_size; i++)
		{
			   tempaddr = (U16 *)CURRCARD.base_address + (SPROM_BASE/2) + i; 
			   * tempaddr = image[i];
			MM_Wait(20000); 
		}
	
	// assumes 4-byte aligned offset 
		write_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,val32 );

	MM_Wait(5000);  

		

	// check to see if SPROM programmed correctly 
	j=0;
	for (i = 0; i < sprom_size/2; i++)
		{
			tempread = RegRd(SPROM_BASE + (i*4));
			
			if (((tempread & 0xffff) != image[j]) || 
			   (((tempread & 0xffff0000)/ 0x10000) != image[j+1]))
				break;
			j=j+2;
		}       
	if (i < sprom_size/2) 
	{
	printf("SPROM not programmed correctly!\n");
	printf("Location: %d\n", i);
		status=HAS_ERROR;
	}
	else
	util_init_pci_and_find_tigons (0, 0);

	return status;
}
U32 SB4xDiagTestEEPROM(u32 req_iteration )

{
	U16 b[SPROM_SIZE];
	int nw;
	U8 chk8;
	int i=0,j=0;
	U32 nbytes;
	U32     Retcode;
	ULONG local_iteration = 0;
	int k;
	U32 tempread;


	nbytes = sizeof (b);
	nw = ROUNDUP(nbytes, 4) / 4;
    if(!global.manuf)
		printf("Begin EEPROM test.\n");

	Retcode = NO_ERROR;
	do 
    {
		local_iteration++;
		if (req_iteration != 1) printf("Iteration : %d\n",local_iteration);
		check_abort();

		// read the sprom 4 bytes at a time - may need byte swapping
		k = 0;
		if(!global.manuf)
			printf("\n");
		for (i = 0; i < nw; i++) {
			tempread = RegRd( SPROM_BASE + (i * 4));
			b[j] = tempread & 0xffff;
			j++;
			b[j] = (tempread & 0xffff0000) / 0x10000;
			j++;
		}
		
		
		if ((chk8 = crc8((U8 *)b, nbytes, CRC8_INIT_VALUE)) != CRC8_GOOD_VALUE) 
		{
			//printf("EEPROM corrupted.CRC:%x\n",chk8);
			Retcode = ERR_EEP_CRC;
	//              return;
		}
		if(!global.manuf)
		{
			srdump(b, nbytes/2, "Srom contents:", BCM4401);
		}
	if (Retcode != NO_ERROR) break;
	}while ((Retcode == NO_ERROR) && (!req_iteration || (local_iteration < req_iteration)));
	if(!global.manuf)
	{
		if (Retcode == NO_ERROR) 
			printf("EEPROM test PASSED.\n"); 
		else
			printf("EEPROM test FAILED\n"); 
	}
	
	return Retcode;

}
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
U32
SB4xDiagTestBlast(u32 type)
{

    int txlen, rc, len;
	int j;
	pBlast dev = &CURRCARD.blast;
    char txbuf[MAX_ETHERNET_PACKET_SIZE];
    char rxbuf[MAX_ETHERNET_PACKET_SIZE];
	U32 status = NO_ERROR;
    pTX_TEST_INFO txInfo = &CURRCARD.txInfo;

    BUFENTRY        dmabuf[50];
    ULONG           txringaddr = NULL, rxringaddr = NULL;
    ULONG           rxbufaddr[NRXDBUFF];
	ULONG           txbufaddr[1];
    ULONG           uTxIn = 0, uTxOut = 0, uRxIn = 0, uRxOut = 0;
	ULONG           n=0;
	ULONG           last_update_time;
	ULONG           last_updateCounter_time;
	int				i;
    unsigned int    bRet;
	
	if (!(txInfo->tx_enable || txInfo->rx_enable || txInfo->host_loopback))
    {
		eprintf("Dev %d: Tx or Rx or Host Loopback have to be enabled!\n",global.current_card);
		return HAS_ERROR;
    }
	

	dev->current_tx_seq_num = 0;  
    dev->current_rx_seq_num = 0;  
    dev->sniffer_data = 0;
    dev->seq_num_sync = 0;
    
	txInfo->curr_tx_len = txInfo->start_tx_len;
	txlen = txInfo->curr_tx_len;
    if (txInfo->end_tx_len < txInfo->curr_tx_len)
	txInfo->end_tx_len = txInfo->curr_tx_len;

	
    memset( &dmabuf, 0, sizeof( dmabuf ) );
    dmainitBlast(dmabuf, sizeof( dmabuf ), &txringaddr, &rxringaddr, txbufaddr, rxbufaddr, &uRxOut );
	
	if ((txringaddr == 0) || (rxringaddr == 0) )
	{
		if (!global.manuf)
		{
			printf("Memory Allocation Error.\n");
		}
		return ERR_MALLOC;
	}

	for (i = 0 ; i < NRXDBUFF; i++)
	{
		 
		if (rxbufaddr[i] == 0)
		{
			if (!global.manuf)
			{
				printf("Memory Allocation Error.\n");
			}
			return ERR_MALLOC;
		}
	}

	for (i = 0 ; i < 1; i++)
	{
		 
		if (txbufaddr[i] == 0)
		{
			if (!global.manuf)
			{
				printf("Memory Allocation Error.\n");
			}
			return ERR_MALLOC;
		}
	}


	resetcountersBlast();
	readcountersBlast(0);
	displaycountersBlast(1);

	last_update_time=clock();
	last_updateCounter_time=clock();
	if (txInfo->tx_enable)

		{
			last_update_time=clock(); 
			if (txInfo->tx_tprot_pkt)
			{
				for (j = 0; j < 6; j++)
				{
				txbuf[j] = 0xff;
				txbuf[6+j] = j;
				}    

				*(u16 *)(txbuf + 12) = BYTE_SWAP_S(TPROT_MAGIC_PACKET_TYPE);
				*(u32 *)(txbuf + 14) = TPROT_PACKET_SIG;
				txbuf[18] = TPROT_PACKET_TYPE_LOOPBACK;
				txbuf[19] = TPROT_PACKET_FORMAT_ASCENDING;
				*(u16 *)(txbuf + 20) = txInfo->curr_tx_len - TPROT_PACKET_OVERHEAD_SIZE;
				*(u32 *)(txbuf + TPROT_PACKET_OVERHEAD_SIZE) = 
						 BYTE_SWAP(dev->current_tx_seq_num);
				for (j = 0; j < (MAX_ETHERNET_PACKET_SIZE-TPROT_PACKET_OVERHEAD_SIZE-4); j++)
						txbuf[TPROT_PACKET_OVERHEAD_SIZE+4+j] = j;
			}
			else
			{
				for (j = 0; j < 6; j++)
				{
					txbuf[j] = 0xff;
					txbuf[6+j] = dev->pDevice->PermanentNodeAddress[j];
				}    
		
				*(u16 *)(txbuf + 12) = BYTE_SWAP_S(DIAG_MAGIC_PACKET_TYPE);
				*(u16 *)(txbuf + 14) = BYTE_SWAP_S((u16)txInfo->curr_tx_len);
				*(u32 *)(txbuf + 16) = BYTE_SWAP(dev->current_tx_seq_num);
				for (j = 0; j < (MAX_ETHERNET_PACKET_SIZE_NO_CRC-20); j++)
				{
					if (type==0)
						txbuf[20+j] = 0x00;
					else if (type==1)
						txbuf[20+j] = 0xFF;
					else if (type==2)
						txbuf[20+j] = 0x55;
					else if (type==3)
						txbuf[20+j] = 0xAA;
					else if (type==4)
						txbuf[20+j] = 0x0F;
					else if (type==5)
						txbuf[20+j] = 0xF0;
					else if (type==6)
					{
						txbuf[20+j] = 0xFF;
						j++;
						txbuf[20+j] = 0x00;
					}
					else if (type==7)
					{
						txbuf[20+j] = 0x00;
						j++;
						txbuf[20+j] = 0xFF;
					}
					else if (type==8)
					{
						txbuf[20+j] = 0xFF;
						j++;
						txbuf[20+j] = 0xFF;
						j++;
						txbuf[20+j] = 0x00;
						j++;
						txbuf[20+j] = 0x00;
					}
					else if (type==9)
					{
						txbuf[20+j] = 0x00;
						j++;
						txbuf[20+j] = 0x00;
						j++;
						txbuf[20+j] = 0xFF;
						j++;
						txbuf[20+j] = 0xFF;
					}
					else if (type == 10)
						txbuf[20+j] = j;
					else
						txbuf[20+j] = rand() & 0xFF;

				}
			}

			if (txInfo->inc_len)
			{
				if (txInfo->num_tx_packets == 0)
				{
					for(;;)
					{
						if (++txInfo->curr_tx_len > txInfo->end_tx_len)
							txInfo->curr_tx_len = txInfo->start_tx_len;
						// transmit it
						bRet = txdmaBlastHostBack( dmabuf, sizeof( dmabuf ), txbuf, txInfo->curr_tx_len, txbufaddr, &uTxIn, &uTxOut, &txringaddr,txInfo->tx_ipg); 
						
						if ( !bRet )
						{
							if (!global.manuf)
								printf("Tramsmit Error.\n" );
							status = ERR_TXDMA;
							goto done;
						}
						if (kbhit())
						{
							if (toupper(getch())==KEY_ESC)
							goto done;
						}
						//To prevent readcounters and displaycounters interrupt rec too long. 
						if (clock() >=  (last_update_time + 1000) )
						{
							readcountersBlast(0);
							last_update_time=clock();
						}
						
						if (clock() >=  (last_updateCounter_time + 5000) )
						{
							displaycountersBlast(1);
							last_updateCounter_time=clock();
						}
					}
				}
				else
				{
					while(n<txInfo->num_tx_packets)
					{
						if (++txInfo->curr_tx_len > txInfo->end_tx_len)
							txInfo->curr_tx_len = txInfo->start_tx_len;
						// transmit it
						bRet = txdmaBlastHostBack( dmabuf, sizeof( dmabuf ), txbuf, txInfo->curr_tx_len, txbufaddr, &uTxIn, &uTxOut, &txringaddr,txInfo->tx_ipg); 
						
						if ( !bRet )
						{
							if (!global.manuf)
								printf("Tramsmit Error.\n" );
							status = ERR_TXDMA;
							goto done;
						}
						
						n++;
						if (kbhit())
						{
							if (toupper(getch())==KEY_ESC)
							goto done;
						}
						//To prevent readcounters and displaycounters interrupt rec too long. 
						if (clock() >=  (last_update_time + 1000) )
						{
							readcountersBlast(0);
							last_update_time=clock();
						}
						
						if (clock() >=  (last_updateCounter_time + 5000) )
						{
							displaycountersBlast(1);
							last_updateCounter_time=clock();
						}
					}
				}
			}
			else
			{
				// transmit it
				bRet = txdmaBlast( dmabuf, sizeof( dmabuf ), txbuf, txlen, txbufaddr,
					&uTxIn, &uTxOut, &txringaddr,txInfo->tx_ipg,txInfo->num_tx_packets );
			
				if ( !bRet )
				{
					if (!global.manuf)
						printf("Tramsmit Error.\n" );
					status = ERR_TXDMA;
					goto done;
				}

			}
					
		}

	else if (txInfo->rx_enable)
		{
			
		for (;;)
				{
					
					if (kbhit())
					{
						if (toupper(getch())==KEY_ESC)
						goto done;
					}
					
					if ((RegRd(REGOFFSET(dmaregs.rcvstatus)) & RS_CD_MASK) == (uRxIn * sizeof(dmadd_t)))
					{
						if (RegRd(REGOFFSET(intstatus)) & ~(I_RI | I_XI| I_EM ))
						{
							printf( "Error on receive, instatus is 0x%x\r\n",
									RegRd( REGOFFSET( intstatus ) ) );
							status = ERR_RX_INSTATUS;
							goto done;
						}
					}
					else
					{
						for (;;)
						{
							if (kbhit())
							{
								if (toupper(getch())==KEY_ESC)
								goto done;
							}

							len=rxdmaBlast(dmabuf, sizeof( dmabuf ), rxbuf,
								sizeof(rxbuf), &rc, &rxringaddr, rxbufaddr, &uRxIn, &uRxOut ) ;
							//To prevent readcounters and displaycounters interrupt rec too long. 
							if (clock() >=  (last_update_time + 100) )
							{
								readcountersBlast(0);
								last_update_time=clock();
							}
							
							if (clock() >=  (last_updateCounter_time + 5000) )
							{
								displaycountersBlast(1);
								last_updateCounter_time=clock();
							}
							
						}
					}
				}
		}
	else if (txInfo->host_loopback)
		{
			
			for (;;)
			{
				if (kbhit())
				{
					if (toupper(getch())==KEY_ESC)
						goto done;
				}
				
				if ((RegRd(REGOFFSET(dmaregs.rcvstatus)) & RS_CD_MASK) == (uRxIn * sizeof(dmadd_t)))
				{
					if( !checkinterror() )
						goto done;
				}
				else
				{
					// read it back
					bRet = rxdmaBlastHostBack(dmabuf, sizeof( dmabuf ), rxbuf,
							sizeof(rxbuf), &rc, &rxringaddr, rxbufaddr, &uRxIn, &uRxOut );
					len = rxbuf[1] * 0x100 + rxbuf[0] -4;

					if (!(rxbuf[2] & RXH_ERROR) && bRet && (len < MAX_ETHERNET_PACKET_SIZE))
					{
						memcpy(txbuf,rxbuf+RXOFF,len);
						bRet = txdmaBlastHostBack( dmabuf, sizeof( dmabuf ), txbuf, len, txbufaddr,
									&uTxIn, &uTxOut, &txringaddr,0);  
					}
					
				}  
				//To prevent readcounters and displaycounters interrupt rec too long. 
				if (clock() >=  (last_update_time + 1000) )
				{
					readcountersBlast(0);
					last_update_time=clock();
				}
				
				if (clock() >=  (last_updateCounter_time + 5000) )
				{
					displaycountersBlast(1);
					last_updateCounter_time=clock();
				}
			}
		}
done:
	delay(1000);
	readcountersBlast(0);
	displaycountersBlast(0);
	txInfo->tx_enable=0;
	txInfo->host_loopback =0;
	txInfo->rx_enable=0;
    // release dma resources


    dmafreeBlast( dmabuf, sizeof( dmabuf ), &txringaddr,
	    &rxringaddr, rxbufaddr, txbufaddr);
	
      
      return(status);
}

void readcountersBlast(int readall)
{
	pTX_TEST_INFO txInfo = &CURRCARD.txInfo;
	PSWMIBCOUNTERS swmib= &CURRCARD.swmibcounts;

	u32 temp;
     
	if ((txInfo->tx_enable) || (txInfo->host_loopback) || (readall))
	{
		temp = swmib->tx_good_octets_L;
		swmib->tx_good_octets_L = RegRd(REGOFFSET(mib.tx_good_octets));
		if (swmib->tx_good_octets_L < temp)
			swmib->tx_good_octets_H += 1;

		temp = swmib->tx_good_pkts_L;
		swmib->tx_good_pkts_L = RegRd(REGOFFSET(mib.tx_good_pkts));
		if (swmib->tx_good_pkts_L < temp)
			swmib->tx_good_pkts_H += 1;

		temp = swmib->tx_octets_L;
		swmib->tx_octets_L = RegRd(REGOFFSET(mib.tx_octets));
		if (swmib->tx_octets_L < temp)
			swmib->tx_octets_H += 1;

		temp=swmib->tx_pkts_L;
		swmib->tx_pkts_L = RegRd(REGOFFSET(mib.tx_pkts));
		if (swmib->tx_pkts_L < temp)
			swmib->tx_pkts_H += 1;
		
		temp=swmib->tx_broadcast_pkts_L;
		swmib->tx_broadcast_pkts_L = RegRd(REGOFFSET(mib.tx_broadcast_pkts));
		if (swmib->tx_broadcast_pkts_L <temp)
			swmib->tx_broadcast_pkts_H +=1;
		
		temp=swmib->tx_multicast_pkts_L;
		swmib->tx_multicast_pkts_L = RegRd(REGOFFSET(mib.tx_multicast_pkts));
		if (swmib->tx_multicast_pkts_L <temp)
			swmib->tx_multicast_pkts_H +=1;
		
		temp=swmib->tx_len_64_L;
		swmib->tx_len_64_L = RegRd(REGOFFSET(mib.tx_len_64));
		if (swmib->tx_len_64_L <temp)
			swmib->tx_len_64_H +=1;
		
		temp=swmib->tx_len_65_to_127_L;
		swmib->tx_len_65_to_127_L = RegRd(REGOFFSET(mib.tx_len_65_to_127));
		if (swmib->tx_len_65_to_127_L <temp)
			swmib->tx_len_65_to_127_H +=1;
		
		temp=swmib->tx_len_128_to_255_L;
		swmib->tx_len_128_to_255_L = RegRd(REGOFFSET(mib.tx_len_128_to_255));
		if (swmib->tx_len_128_to_255_L <temp)
			swmib->tx_len_128_to_255_H +=1;
		
		temp=swmib->tx_len_256_to_511_L;
		swmib->tx_len_256_to_511_L = RegRd(REGOFFSET(mib.tx_len_256_to_511));
		if (swmib->tx_len_256_to_511_L <temp)
			swmib->tx_len_256_to_511_H +=1;
		
		temp=swmib->tx_len_512_to_1023_L;
		swmib->tx_len_512_to_1023_L = RegRd(REGOFFSET(mib.tx_len_512_to_1023));
		if (swmib->tx_len_512_to_1023_L <temp)
			swmib->tx_len_512_to_1023_H +=1;
		
		temp=swmib->tx_len_1024_to_max_L;
		swmib->tx_len_1024_to_max_L = RegRd(REGOFFSET(mib.tx_len_1024_to_max));
		if (swmib->tx_len_1024_to_max_L <temp)
			swmib->tx_len_1024_to_max_H +=1;
		
		temp=swmib->tx_jabber_pkts_L;
		swmib->tx_jabber_pkts_L = RegRd(REGOFFSET(mib.tx_jabber_pkts));
		if (swmib->tx_jabber_pkts_L <temp)
			swmib->tx_jabber_pkts_H +=1;
		
		temp=swmib->tx_oversize_pkts_L;
		swmib->tx_oversize_pkts_L = RegRd(REGOFFSET(mib.tx_oversize_pkts));
		if (swmib->tx_oversize_pkts_L <temp)
			swmib->tx_oversize_pkts_H +=1;
		
		temp=swmib->tx_fragment_pkts_L;
		swmib->tx_fragment_pkts_L = RegRd(REGOFFSET(mib.tx_fragment_pkts));
		if (swmib->tx_fragment_pkts_L <temp)
			swmib->tx_fragment_pkts_H +=1;
		
		temp=swmib->tx_underruns_L;
		swmib->tx_underruns_L = RegRd(REGOFFSET(mib.tx_underruns));
		if (swmib->tx_underruns_L <temp)
			swmib->tx_underruns_H +=1;


		temp=swmib->tx_total_cols_L;
		swmib->tx_total_cols_L = RegRd(REGOFFSET(mib.tx_total_cols));
		if (swmib->tx_total_cols_L <temp)
			swmib->tx_total_cols_H +=1;

		temp=swmib->tx_single_cols_L;
		swmib->tx_single_cols_L = RegRd(REGOFFSET(mib.tx_single_cols));
		if (swmib->tx_single_cols_L <temp)
			swmib->tx_single_cols_H +=1;

		temp=swmib->tx_multiple_cols_L;
		swmib->tx_multiple_cols_L = RegRd(REGOFFSET(mib.tx_multiple_cols));
		if (swmib->tx_multiple_cols_L <temp)
			swmib->tx_multiple_cols_H +=1;

		temp=swmib->tx_excessive_cols_L;
		swmib->tx_excessive_cols_L = RegRd(REGOFFSET(mib.tx_excessive_cols));
		if (swmib->tx_excessive_cols_L <temp)
			swmib->tx_excessive_cols_H +=1;

		temp=swmib->tx_late_cols_L;
		swmib->tx_late_cols_L = RegRd(REGOFFSET(mib.tx_total_cols));
		if (swmib->tx_late_cols_L <temp)
			swmib->tx_late_cols_H +=1;

		temp=swmib->tx_defered_L;
		swmib->tx_defered_L = RegRd(REGOFFSET(mib.tx_defered));
		if (swmib->tx_defered_L <temp)
			swmib->tx_defered_H +=1;

		temp=swmib->tx_carrier_lost_L;
		swmib->tx_carrier_lost_L = RegRd(REGOFFSET(mib.tx_carrier_lost));
		if (swmib->tx_carrier_lost_L <temp)
			swmib->tx_carrier_lost_H +=1;

		temp=swmib->tx_pause_pkts_L;
		swmib->tx_pause_pkts_L = RegRd(REGOFFSET(mib.tx_pause_pkts));
		if (swmib->tx_pause_pkts_L <temp)
			swmib->tx_pause_pkts_H +=1;
		
    }

	if ((txInfo->rx_enable) || (txInfo->host_loopback) || (readall))
	{
		temp = swmib->rx_good_octets_L;
		swmib->rx_good_octets_L = RegRd(REGOFFSET(mib.rx_good_octets));
		if (swmib->rx_good_octets_L < temp)
			swmib->rx_good_octets_H += 1;

		temp = swmib->rx_good_pkts_L;
		swmib->rx_good_pkts_L = RegRd(REGOFFSET(mib.rx_good_pkts));
		if (swmib->rx_good_pkts_L < temp)
			swmib->rx_good_pkts_H += 1;

		temp = swmib->rx_octets_L;
		swmib->rx_octets_L = RegRd(REGOFFSET(mib.rx_octets));
		if (swmib->rx_octets_L < temp)
			swmib->rx_octets_H += 1;

		temp=swmib->rx_pkts_L;
		swmib->rx_pkts_L = RegRd(REGOFFSET(mib.rx_pkts));
		if (swmib->rx_pkts_L < temp)
			swmib->rx_pkts_H += 1;
		
		temp=swmib->rx_broadcast_pkts_L;
		swmib->rx_broadcast_pkts_L = RegRd(REGOFFSET(mib.rx_broadcast_pkts));
		if (swmib->rx_broadcast_pkts_L <temp)
			swmib->rx_broadcast_pkts_H +=1;
		
		temp=swmib->rx_multicast_pkts_L;
		swmib->rx_multicast_pkts_L = RegRd(REGOFFSET(mib.rx_multicast_pkts));
		if (swmib->rx_multicast_pkts_L <temp)
			swmib->rx_multicast_pkts_H +=1;
		
		temp=swmib->rx_len_64_L;
		swmib->rx_len_64_L = RegRd(REGOFFSET(mib.rx_len_64));
		if (swmib->rx_len_64_L <temp)
			swmib->rx_len_64_H +=1;
		
		temp=swmib->rx_len_65_to_127_L;
		swmib->rx_len_65_to_127_L = RegRd(REGOFFSET(mib.rx_len_65_to_127));
		if (swmib->rx_len_65_to_127_L <temp)
			swmib->rx_len_65_to_127_H +=1;
		
		temp=swmib->rx_len_128_to_255_L;
		swmib->rx_len_128_to_255_L = RegRd(REGOFFSET(mib.rx_len_128_to_255));
		if (swmib->rx_len_128_to_255_L <temp)
			swmib->rx_len_128_to_255_H +=1;
		
		temp=swmib->rx_len_256_to_511_L;
		swmib->rx_len_256_to_511_L = RegRd(REGOFFSET(mib.rx_len_256_to_511));
		if (swmib->rx_len_256_to_511_L <temp)
			swmib->rx_len_256_to_511_H +=1;
		
		temp=swmib->rx_len_512_to_1023_L;
		swmib->rx_len_512_to_1023_L = RegRd(REGOFFSET(mib.rx_len_512_to_1023));
		if (swmib->rx_len_512_to_1023_L <temp)
			swmib->rx_len_512_to_1023_H +=1;
		
		temp=swmib->rx_len_1024_to_max_L;
		swmib->rx_len_1024_to_max_L = RegRd(REGOFFSET(mib.rx_len_1024_to_max));
		if (swmib->rx_len_1024_to_max_L <temp)
			swmib->rx_len_1024_to_max_H +=1;
		
		temp=swmib->rx_jabber_pkts_L;
		swmib->rx_jabber_pkts_L = RegRd(REGOFFSET(mib.rx_jabber_pkts));
		if (swmib->rx_jabber_pkts_L <temp)
			swmib->rx_jabber_pkts_H +=1;
		
		temp=swmib->rx_oversize_pkts_L;
		swmib->rx_oversize_pkts_L = RegRd(REGOFFSET(mib.rx_oversize_pkts));
		if (swmib->rx_oversize_pkts_L <temp)
			swmib->rx_oversize_pkts_H +=1;
		
		temp=swmib->rx_fragment_pkts_L;
		swmib->rx_fragment_pkts_L = RegRd(REGOFFSET(mib.rx_fragment_pkts));
		if (swmib->rx_fragment_pkts_L <temp)
			swmib->rx_fragment_pkts_H +=1;

		temp=swmib->rx_missed_pkts_L;
		swmib->rx_missed_pkts_L = RegRd(REGOFFSET(mib.rx_missed_pkts));
		if (swmib->rx_missed_pkts_L <temp)
			swmib->rx_missed_pkts_H +=1;

		temp=swmib->rx_crc_align_errs_L;
		swmib->rx_crc_align_errs_L = RegRd(REGOFFSET(mib.rx_crc_align_errs));
		if (swmib->rx_crc_align_errs_L <temp)
			swmib->rx_crc_align_errs_H +=1;

		temp=swmib->rx_undersize_L;
		swmib->rx_undersize_L = RegRd(REGOFFSET(mib.rx_undersize));
		if (swmib->rx_undersize_L <temp)
			swmib->rx_undersize_H +=1;

		temp=swmib->rx_crc_errs_L;
		swmib->rx_crc_errs_L = RegRd(REGOFFSET(mib.rx_crc_errs));
		if (swmib->rx_crc_errs_L <temp)
			swmib->rx_crc_errs_H +=1;


		temp=swmib->rx_align_errs_L;
		swmib->rx_align_errs_L = RegRd(REGOFFSET(mib.rx_align_errs));
		if (swmib->rx_align_errs_L <temp)
			swmib->rx_align_errs_H +=1;

		temp=swmib->rx_symbol_errs_L;
		swmib->rx_symbol_errs_L = RegRd(REGOFFSET(mib.rx_symbol_errs));
		if (swmib->rx_symbol_errs_L <temp)
			swmib->rx_symbol_errs_H +=1;



		temp=swmib->rx_pause_pkts_L;
		swmib->rx_pause_pkts_L = RegRd(REGOFFSET(mib.rx_pause_pkts));
		if (swmib->rx_pause_pkts_L <temp)
			swmib->rx_pause_pkts_H +=1;

		temp=swmib->rx_nonpause_pkts_L;
		swmib->rx_nonpause_pkts_L = RegRd(REGOFFSET(mib.rx_nonpause_pkts));
		if (swmib->rx_nonpause_pkts_L <temp)
			swmib->rx_nonpause_pkts_H +=1;

	}

}         
void resetcountersBlast(void)
{
	PSWMIBCOUNTERS swmib= &CURRCARD.swmibcounts;
    
    swmib->tx_good_octets_L = 0;
	swmib->tx_good_pkts_L = 0;
	swmib->tx_octets_L = 0;
	swmib->tx_pkts_L = 0;
	swmib->tx_broadcast_pkts_L = 0;
	swmib->tx_multicast_pkts_L = 0;
	swmib->tx_len_64_L = 0;
	swmib->tx_len_65_to_127_L = 0;
	swmib->tx_len_128_to_255_L = 0;
	swmib->tx_len_256_to_511_L = 0;
	swmib->tx_len_512_to_1023_L = 0;
	swmib->tx_len_1024_to_max_L = 0;
	swmib->tx_jabber_pkts_L = 0;
	swmib->tx_oversize_pkts_L = 0;
	swmib->tx_fragment_pkts_L = 0;
	swmib->tx_underruns_L = 0;
	swmib->tx_total_cols_L = 0;
	swmib->tx_single_cols_L = 0;
	swmib->tx_multiple_cols_L = 0;
	swmib->tx_excessive_cols_L = 0;
	swmib->tx_late_cols_L = 0;
	swmib->tx_defered_L = 0;
	swmib->tx_carrier_lost_L = 0;
	swmib->tx_pause_pkts_L = 0;


    swmib->rx_good_octets_L = 0;
	swmib->rx_good_pkts_L = 0;
	swmib->rx_octets_L = 0;
	swmib->rx_pkts_L = 0;
	swmib->rx_broadcast_pkts_L = 0;
	swmib->rx_multicast_pkts_L = 0;
	swmib->rx_len_64_L = 0;
	swmib->rx_len_65_to_127_L = 0;
	swmib->rx_len_128_to_255_L = 0;
	swmib->rx_len_256_to_511_L = 0;
	swmib->rx_len_512_to_1023_L = 0;
	swmib->rx_len_1024_to_max_L = 0;
	swmib->rx_jabber_pkts_L = 0;
	swmib->rx_oversize_pkts_L = 0;
	swmib->rx_fragment_pkts_L = 0;
	swmib->rx_missed_pkts_L = 0;
	swmib->rx_crc_align_errs_L = 0;
	swmib->rx_undersize_L= 0;
	swmib->rx_crc_errs_L = 0;
	swmib->rx_align_errs_L = 0;
	swmib->rx_symbol_errs_L = 0;
	swmib->rx_pause_pkts_L = 0;
	swmib->rx_nonpause_pkts_L = 0;

	swmib->tx_good_octets_H = 0;
	swmib->tx_good_pkts_H = 0;
	swmib->tx_octets_H = 0;
	swmib->tx_pkts_H = 0;
	swmib->tx_broadcast_pkts_H = 0;
	swmib->tx_multicast_pkts_H = 0;
	swmib->tx_len_64_H = 0;
	swmib->tx_len_65_to_127_H = 0;
	swmib->tx_len_128_to_255_H = 0;
	swmib->tx_len_256_to_511_H = 0;
	swmib->tx_len_512_to_1023_H = 0;
	swmib->tx_len_1024_to_max_H = 0;
	swmib->tx_jabber_pkts_H = 0;
	swmib->tx_oversize_pkts_H = 0;
	swmib->tx_fragment_pkts_H = 0;
	swmib->tx_underruns_H = 0;
	swmib->tx_total_cols_H = 0;
	swmib->tx_single_cols_H = 0;
	swmib->tx_multiple_cols_H = 0;
	swmib->tx_excessive_cols_H = 0;
	swmib->tx_late_cols_H = 0;
	swmib->tx_defered_H = 0;
	swmib->tx_carrier_lost_H = 0;
	swmib->tx_pause_pkts_H = 0;


    swmib->rx_good_octets_H = 0;
	swmib->rx_good_pkts_H = 0;
	swmib->rx_octets_H = 0;
	swmib->rx_pkts_H = 0;
	swmib->rx_broadcast_pkts_H = 0;
	swmib->rx_multicast_pkts_H = 0;
	swmib->rx_len_64_H = 0;
	swmib->rx_len_65_to_127_H = 0;
	swmib->rx_len_128_to_255_H = 0;
	swmib->rx_len_256_to_511_H = 0;
	swmib->rx_len_512_to_1023_H = 0;
	swmib->rx_len_1024_to_max_H = 0;
	swmib->rx_jabber_pkts_H = 0;
	swmib->rx_oversize_pkts_H = 0;
	swmib->rx_fragment_pkts_H = 0;
	swmib->rx_missed_pkts_H = 0;
	swmib->rx_crc_align_errs_H = 0;
	swmib->rx_undersize_H = 0;
	swmib->rx_crc_errs_H = 0;
	swmib->rx_align_errs_H = 0;
	swmib->rx_symbol_errs_H = 0;
	swmib->rx_pause_pkts_H = 0;
	swmib->rx_nonpause_pkts_H = 0;

	RegWr(REGOFFSET(mib.tx_good_octets), 0 );
	RegWr16(REGOFFSET(mib.tx_good_pkts), 0 );
	RegWr(REGOFFSET(mib.tx_octets), 0 );
	RegWr16(REGOFFSET(mib.tx_pkts), 0 );
	RegWr16(REGOFFSET(mib.tx_broadcast_pkts), 0 );
	RegWr16(REGOFFSET(mib.tx_len_64), 0 );
	RegWr16(REGOFFSET(mib.tx_len_65_to_127), 0 );
	RegWr16(REGOFFSET(mib.tx_len_128_to_255), 0 );
	RegWr16(REGOFFSET(mib.tx_len_256_to_511), 0 );
	RegWr16(REGOFFSET(mib.tx_len_512_to_1023), 0 );
	RegWr16(REGOFFSET(mib.tx_len_1024_to_max), 0 );
	RegWr16(REGOFFSET(mib.tx_jabber_pkts), 0 );
	RegWr16(REGOFFSET(mib.tx_oversize_pkts), 0 );
	RegWr16(REGOFFSET(mib.tx_fragment_pkts), 0 );
	RegWr16(REGOFFSET(mib.tx_underruns), 0 );
	RegWr16(REGOFFSET(mib.tx_total_cols), 0 );
	RegWr16(REGOFFSET(mib.tx_single_cols), 0 );
	RegWr16(REGOFFSET(mib.tx_multiple_cols), 0 );
	RegWr16(REGOFFSET(mib.tx_excessive_cols), 0 );
	RegWr16(REGOFFSET(mib.tx_late_cols), 0 );
	RegWr16(REGOFFSET(mib.tx_defered), 0 );
	RegWr16(REGOFFSET(mib.tx_carrier_lost), 0 );
	RegWr16(REGOFFSET(mib.tx_pause_pkts), 0 );


    RegWr(REGOFFSET(mib.rx_good_octets), 0 );
	RegWr16(REGOFFSET(mib.rx_good_pkts), 0 );
	RegWr(REGOFFSET(mib.rx_octets), 0 );
	RegWr16(REGOFFSET(mib.rx_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_broadcast_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_len_64), 0 );
	RegWr16(REGOFFSET(mib.rx_len_65_to_127), 0 );
	RegWr16(REGOFFSET(mib.rx_len_128_to_255), 0 );
	RegWr16(REGOFFSET(mib.rx_len_256_to_511), 0 );
	RegWr16(REGOFFSET(mib.rx_len_512_to_1023), 0 );
	RegWr16(REGOFFSET(mib.rx_len_1024_to_max), 0 );
	RegWr16(REGOFFSET(mib.rx_jabber_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_oversize_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_fragment_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_missed_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_crc_align_errs), 0 );
	RegWr16(REGOFFSET(mib.rx_undersize), 0 );
	RegWr16(REGOFFSET(mib.rx_crc_errs), 0 );
	RegWr16(REGOFFSET(mib.rx_align_errs), 0 );
	RegWr16(REGOFFSET(mib.rx_symbol_errs), 0 );
	RegWr16(REGOFFSET(mib.rx_pause_pkts), 0 );
	RegWr16(REGOFFSET(mib.rx_nonpause_pkts), 0 );
}       
unsigned int displaycountersBlast(int runtime)
{
	pTX_TEST_INFO txInfo = &CURRCARD.txInfo;
	PSWMIBCOUNTERS swmib= &CURRCARD.swmibcounts;
	U32 last_time;
	int tempchar;
		win_clearscreen();

		if (!runtime)
		{
			if (txInfo->tx_enable)
			{               
				
				printf("                TX Good Bytes Counts: %10lu\n",swmib->tx_good_octets_H * 0x100000000 + swmib->tx_good_octets_L);
				printf("              TX Good Packets Counts: %10lu\n",swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				printf("               TX Total Bytes Counts: %10lu\n",swmib->tx_octets_H * 0x100000000 + swmib->tx_octets_L);
				printf("             TX Total Packets Counts: %10lu\n",swmib->tx_pkts_H * 0x10000 + swmib->tx_pkts_L);
				printf("         TX Broadcast Packets Counts: %10lu\n",swmib->tx_broadcast_pkts_H * 0x10000 + swmib->tx_broadcast_pkts_L);
				printf("         TX Multicast Packets Counts: %10lu\n",swmib->tx_multicast_pkts_H * 0x10000 + swmib->tx_multicast_pkts_L);
				printf("            TX Len_64 Packets Counts: %10lu\n",swmib->tx_len_64_H * 0x10000 +swmib->tx_len_64_L,swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L);
				printf("     TX Len_65_to_127 Packets Counts: %10lu\n",swmib->tx_len_65_to_127_H * 0x10000 + swmib->tx_len_65_to_127_L);
				printf("    TX Len_128_to_255 Packets Counts: %10lu\n",swmib->tx_len_128_to_255_H * 0x10000 +swmib->tx_len_128_to_255_L);
				printf("    TX Len_256_to_511 Packets Counts: %10lu\n",swmib->tx_len_256_to_511_H * 0x10000 +swmib->tx_len_256_to_511_L);
				printf("   TX Len_512_to_1023 Packets Counts: %10lu\n",swmib->tx_len_512_to_1023_H * 0x10000 +swmib->tx_len_512_to_1023_L);
				printf("   TX Len_1024_to_Max Packets Counts: %10lu\n",swmib->tx_len_1024_to_max_H * 0x10000 +swmib->tx_len_1024_to_max_L);
				printf("            TX Jabber Packets Counts: %10lu\n",swmib->tx_jabber_pkts_H * 0x10000 +swmib->tx_jabber_pkts_L);
				printf("          TX OverSize Packets Counts: %10lu\n",swmib->tx_oversize_pkts_H * 0x10000 + swmib->tx_oversize_pkts_L);
				printf("          TX Fragment Packets Counts: %10lu\n",swmib->tx_fragment_pkts_H * 0x10000 + swmib->tx_fragment_pkts_L);

				printf("                 TX Under Run Counts: %10lu\n",swmib->tx_underruns_H * 0x10000 +swmib->tx_underruns_L);
				printf("                TX Total Cols Counts: %10lu\n",swmib->tx_total_cols_H * 0x10000 +swmib->tx_total_cols_L);
				printf("                TX Single Col Counts: %10lu\n",swmib->tx_single_cols_H * 0x10000 +swmib->tx_single_cols_L);
				printf("              TX Mutiple Cols Counts: %10lu\n",swmib->tx_multiple_cols_H * 0x10000 +swmib->tx_multiple_cols_L);
				printf("            TX Excessive Cols Counts: %10lu\n",swmib->tx_excessive_cols_H * 0x10000 +swmib->tx_excessive_cols_L);
				printf("                 TX Late Cols Counts: %10lu\n",swmib->tx_late_cols_H * 0x10000 +swmib->tx_late_cols_L);
				printf("                   TX Defered Counts: %10lu\n",swmib->tx_defered_H * 0x10000 +swmib->tx_defered_L);
				printf("         TX Carrier_lost Cols Counts: %10lu\n",swmib->tx_carrier_lost_H * 0x10000 +swmib->tx_carrier_lost_L);
				printf("              TX Pause Packet Counts: %10lu\n",swmib->tx_pause_pkts_H * 0x10000 + swmib->tx_pause_pkts_L);
			}

			else if (txInfo->rx_enable)
			{
				
				printf("                 RX Good Bytes Counts: %10lu\n",swmib->rx_good_octets_H * 0x100000000 + swmib->rx_good_octets_L);
				printf("               RX Good Packets Counts: %10lu\n",swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				printf("                RX Total Bytes Counts: %10lu\n",swmib->rx_octets_H * 0x100000000 + swmib->rx_octets_L);
				printf("              RX Total Packets Counts: %10lu\n",swmib->rx_pkts_H * 0x10000 + swmib->rx_pkts_L);
				printf("          RX Broadcast Packets Counts: %10lu\n",swmib->rx_broadcast_pkts_H * 0x10000 + swmib->rx_broadcast_pkts_L);
				printf("          RX Multicast Packets Counts: %10lu\n",swmib->rx_multicast_pkts_H * 0x10000 + swmib->rx_multicast_pkts_L);
				printf("             RX Len_64 Packets Counts: %10lu\n",swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L,swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L);
				printf("      RX Len_65_to_127 Packets Counts: %10lu\n",swmib->rx_len_65_to_127_H * 0x10000 + swmib->rx_len_65_to_127_L);
				printf("     RX Len_128_to_255 Packets Counts: %10lu\n",swmib->rx_len_128_to_255_H * 0x10000 +swmib->rx_len_128_to_255_L);
				printf("     RX Len_256_to_511 Packets Counts: %10lu\n",swmib->rx_len_256_to_511_H * 0x10000 +swmib->rx_len_256_to_511_L);
				printf("    RX Len_512_to_1023 Packets Counts: %10lu\n",swmib->rx_len_512_to_1023_H * 0x10000 +swmib->rx_len_512_to_1023_L);
				printf("    RX Len_1024_to_Max Packets Counts: %10lu\n",swmib->rx_len_1024_to_max_H * 0x10000 +swmib->rx_len_1024_to_max_L);
				printf("             RX Jabber Packets Counts: %10lu\n",swmib->rx_jabber_pkts_H * 0x10000 +swmib->rx_jabber_pkts_L);
				printf("           RX OverSize Packets Counts: %10lu\n",swmib->rx_oversize_pkts_H * 0x10000 + swmib->rx_oversize_pkts_L);
				printf("           RX Fragment Packets Counts: %10lu\n",swmib->rx_fragment_pkts_H * 0x10000 + swmib->rx_fragment_pkts_L);
				
				
				printf("             RX Missed Packets Counts: %10lu\n",swmib->rx_missed_pkts_H * 0x10000 +swmib->rx_missed_pkts_L);
				printf("              RX CRC Align Err Counts: %10lu\n",swmib->rx_crc_align_errs_H * 0x10000 +swmib->rx_crc_align_errs_L);
				printf("                 RX Under Size Counts: %10lu\n",swmib->rx_undersize_H * 0x10000 + swmib->rx_undersize_L);
				printf("                    RX CRC Err Counts: %10lu\n",swmib->rx_crc_errs_H * 0x10000 +swmib->rx_crc_errs_L);
				printf("                  RX Align Err Counts: %10lu\n",swmib->rx_align_errs_H * 0x10000 + swmib->rx_align_errs_H);
				printf("                 RX Symbol Err Counts: %10lu\n",swmib->rx_symbol_errs_H * 0x10000 +swmib->rx_symbol_errs_L);
				printf("               RX Pause Packet Counts: %10lu\n",swmib->rx_pause_pkts_H * 0x10000 +swmib->rx_pause_pkts_L);
				printf("           RX Non_Pause Packet Counts: %10lu\n",swmib->rx_nonpause_pkts_H * 0x10000 +swmib->rx_nonpause_pkts_L);
			}
			
			else //if (txInfo->host_loopback)
			{
				
				printf("                TX Good Bytes Counts: %10lu\n",swmib->tx_good_octets_H * 0x100000000 + swmib->tx_good_octets_L);
				printf("              TX Good Packets Counts: %10lu\n",swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				printf("               TX Total Bytes Counts: %10lu\n",swmib->tx_octets_H * 0x100000000 + swmib->tx_octets_L);
				printf("             TX Total Packets Counts: %10lu\n",swmib->tx_pkts_H * 0x10000 + swmib->tx_pkts_L);
				printf("         TX Broadcast Packets Counts: %10lu\n",swmib->tx_broadcast_pkts_H * 0x10000 + swmib->tx_broadcast_pkts_L);
				printf("         TX Multicast Packets Counts: %10lu\n",swmib->tx_multicast_pkts_H * 0x10000 + swmib->tx_multicast_pkts_L);
				printf("            TX Len_64 Packets Counts: %10lu\n",swmib->tx_len_64_H * 0x10000 +swmib->tx_len_64_L,swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L);
				printf("     TX Len_65_to_127 Packets Counts: %10lu\n",swmib->tx_len_65_to_127_H * 0x10000 + swmib->tx_len_65_to_127_L);
				printf("    TX Len_128_to_255 Packets Counts: %10lu\n",swmib->tx_len_128_to_255_H * 0x10000 +swmib->tx_len_128_to_255_L);
				printf("    TX Len_256_to_511 Packets Counts: %10lu\n",swmib->tx_len_256_to_511_H * 0x10000 +swmib->tx_len_256_to_511_L);
				printf("   TX Len_512_to_1023 Packets Counts: %10lu\n",swmib->tx_len_512_to_1023_H * 0x10000 +swmib->tx_len_512_to_1023_L);
				printf("   TX Len_1024_to_Max Packets Counts: %10lu\n",swmib->tx_len_1024_to_max_H * 0x10000 +swmib->tx_len_1024_to_max_L);
				printf("            TX Jabber Packets Counts: %10lu\n",swmib->tx_jabber_pkts_H * 0x10000 +swmib->tx_jabber_pkts_L);
				printf("          TX OverSize Packets Counts: %10lu\n",swmib->tx_oversize_pkts_H * 0x10000 + swmib->tx_oversize_pkts_L);
				printf("          TX Fragment Packets Counts: %10lu\n",swmib->tx_fragment_pkts_H * 0x10000 + swmib->tx_fragment_pkts_L);

				printf("                Hit Enter to see more! Hit Esc to end  \n");
				
				last_time=clock();
				for (;;)
				{
					if (clock() >=(last_time+1000))
					{
						if (kbhit())
						{
							tempchar = getch();
							if (toupper(tempchar )==KEY_RETURN)
								break;
							if (toupper(tempchar )==KEY_ESC)
								goto done;
							
						}
					}
				}
				win_clearscreen();
				printf("                 TX Under Run Counts: %10lu\n",swmib->tx_underruns_H * 0x10000 +swmib->tx_underruns_L);
				printf("                TX Total Cols Counts: %10lu\n",swmib->tx_total_cols_H * 0x10000 +swmib->tx_total_cols_L);
				printf("                TX Single Col Counts: %10lu\n",swmib->tx_single_cols_H * 0x10000 +swmib->tx_single_cols_L);
				printf("              TX Mutiple Cols Counts: %10lu\n",swmib->tx_multiple_cols_H * 0x10000 +swmib->tx_multiple_cols_L);
				printf("            TX Excessive Cols Counts: %10lu\n",swmib->tx_excessive_cols_H * 0x10000 +swmib->tx_excessive_cols_L);
				printf("                 TX Late Cols Counts: %10lu\n",swmib->tx_late_cols_H * 0x10000 +swmib->tx_late_cols_L);
				printf("                   TX Defered Counts: %10lu\n",swmib->tx_defered_H * 0x10000 +swmib->tx_defered_L);
				printf("         TX Carrier_lost Cols Counts: %10lu\n",swmib->tx_carrier_lost_H * 0x10000 +swmib->tx_carrier_lost_L);
				printf("              TX Pause Packet Counts: %10lu\n",swmib->tx_pause_pkts_H * 0x10000 + swmib->tx_pause_pkts_L);
				
				printf("                Hit Enter to see more! Hit Esc to end  \n");
				
				last_time=clock();
				for (;;)
				{
					if (clock() >=(last_time+1000))
					{
						if (kbhit())
						{
							tempchar = getch();
							if (toupper(tempchar )==KEY_RETURN)
								break;
							if (toupper(tempchar )==KEY_ESC)
								goto done;
							
						}
					}
				}
				win_clearscreen();
				printf("                 RX Good Bytes Counts: %10lu\n",swmib->rx_good_octets_H * 0x100000000 + swmib->rx_good_octets_L);
				printf("               RX Good Packets Counts: %10lu\n",swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				printf("                RX Total Bytes Counts: %10lu\n",swmib->rx_octets_H * 0x100000000 + swmib->rx_octets_L);
				printf("              RX Total Packets Counts: %10lu\n",swmib->rx_pkts_H * 0x10000 + swmib->rx_pkts_L);
				printf("          RX Broadcast Packets Counts: %10lu\n",swmib->rx_broadcast_pkts_H * 0x10000 + swmib->rx_broadcast_pkts_L);
				printf("          RX Multicast Packets Counts: %10lu\n",swmib->rx_multicast_pkts_H * 0x10000 + swmib->rx_multicast_pkts_L);
				printf("             RX Len_64 Packets Counts: %10lu\n",swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L,swmib->rx_len_64_H * 0x10000 +swmib->rx_len_64_L);
				printf("      RX Len_65_to_127 Packets Counts: %10lu\n",swmib->rx_len_65_to_127_H * 0x10000 + swmib->rx_len_65_to_127_L);
				printf("     RX Len_128_to_255 Packets Counts: %10lu\n",swmib->rx_len_128_to_255_H * 0x10000 +swmib->rx_len_128_to_255_L);
				printf("     RX Len_256_to_511 Packets Counts: %10lu\n",swmib->rx_len_256_to_511_H * 0x10000 +swmib->rx_len_256_to_511_L);
				printf("    RX Len_512_to_1023 Packets Counts: %10lu\n",swmib->rx_len_512_to_1023_H * 0x10000 +swmib->rx_len_512_to_1023_L);
				printf("    RX Len_1024_to_Max Packets Counts: %10lu\n",swmib->rx_len_1024_to_max_H * 0x10000 +swmib->rx_len_1024_to_max_L);
				printf("             RX Jabber Packets Counts: %10lu\n",swmib->rx_jabber_pkts_H * 0x10000 +swmib->rx_jabber_pkts_L);
				printf("           RX OverSize Packets Counts: %10lu\n",swmib->rx_oversize_pkts_H * 0x10000 + swmib->rx_oversize_pkts_L);
				printf("           RX Fragment Packets Counts: %10lu\n",swmib->rx_fragment_pkts_H * 0x10000 + swmib->rx_fragment_pkts_L);
				
				printf("                Hit Enter to see more! Hit Esc to end  \n");
				
				last_time=clock();
				for (;;)
				{
					if (clock() >=(last_time+1000))
					{
						if (kbhit())
						{
							tempchar = getch();
							if (toupper(tempchar )==KEY_RETURN)
								break;
							if (toupper(tempchar )==KEY_ESC)
								goto done;
							
						}
					}
				}
				win_clearscreen();
				printf("             RX Missed Packets Counts: %10lu\n",swmib->rx_missed_pkts_H * 0x10000 +swmib->rx_missed_pkts_L);
				printf("              RX CRC Align Err Counts: %10lu\n",swmib->rx_crc_align_errs_H * 0x10000 +swmib->rx_crc_align_errs_L);
				printf("                 RX Under Size Counts: %10lu\n",swmib->rx_undersize_H * 0x10000 + swmib->rx_undersize_L);
				printf("                    RX CRC Err Counts: %10lu\n",swmib->rx_crc_errs_H * 0x10000 +swmib->rx_crc_errs_L);
				printf("                  RX Align Err Counts: %10lu\n",swmib->rx_align_errs_H * 0x10000 + swmib->rx_align_errs_H);
				printf("                 RX Symbol Err Counts: %10lu\n",swmib->rx_symbol_errs_H * 0x10000 +swmib->rx_symbol_errs_L);
				printf("               RX Pause Packet Counts: %10lu\n",swmib->rx_pause_pkts_H * 0x10000 +swmib->rx_pause_pkts_L);
				printf("           RX Non_Pause Packet Counts: %10lu\n",swmib->rx_nonpause_pkts_H * 0x10000 +swmib->rx_nonpause_pkts_L);


			}
		}
		else
		{
			if ((txInfo->tx_enable) && (!txInfo->rx_enable))
			{
				printf("                                          TX\n");
				printf("                TX Good Bytes Counts: %10lu\n",swmib->tx_good_octets_H * 0x100000000 + swmib->tx_good_octets_L);
				printf("              TX Good Packets Counts: %10lu\n",swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				printf("               TX Total Bytes Counts: %10lu\n",swmib->tx_octets_H * 0x100000000 + swmib->tx_octets_L);
				printf("             TX Total Packets Counts: %10lu\n",swmib->tx_pkts_H * 0x10000 + swmib->tx_pkts_L);
				printf("\n");
				printf("\n");
				printf("                        Press ESC Key to Stop!  \n");
			}
			else if ((txInfo->rx_enable) && (!txInfo->tx_enable))
			{
				printf("                                          RX\n");
				printf("                RX Good Bytes Counts: %10lu\n",swmib->rx_good_octets_H * 0x100000000 + swmib->rx_good_octets_L);
				printf("              RX Good Packets Counts: %10lu\n",swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				printf("               RX Total Bytes Counts: %10lu\n",swmib->rx_octets_H * 0x100000000 + swmib->rx_octets_L);
				printf("             RX Total Packets Counts: %10lu\n",swmib->rx_pkts_H * 0x10000 + swmib->rx_pkts_L);
				printf("\n");
				printf("\n");
				printf("                        Press ESC Key to Stop!  \n");
			}/*
			else if ((txInfo->rx_enable) && (txInfo->tx_enable))
			{
				
				printf("                                          TX\n");
				printf("                TX Good Bytes Counts: %10lu\n",swmib->tx_good_octets_H * 0x100000000 + swmib->tx_good_octets_L);
				printf("              TX Good Packets Counts: %10lu\n",swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				printf("               TX Total Bytes Counts: %10lu\n",swmib->tx_octets_H * 0x100000000 + swmib->tx_octets_L);
				printf("             TX Total Packets Counts: %10lu\n",swmib->tx_pkts_H * 0x10000 + swmib->tx_pkts_L);
				printf("\n");
				printf("\n");
				
				printf("                                          RX\n");
				printf("                RX Good Bytes Counts: %10lu\n",swmib->rx_good_octets_H * 0x100000000 + swmib->rx_good_octets_L);
				printf("              RX Good Packets Counts: %10lu\n",swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				printf("               RX Total Bytes Counts: %10lu\n",swmib->rx_octets_H * 0x100000000 + swmib->rx_octets_L);
				printf("             RX Total Packets Counts: %10lu\n",swmib->rx_pkts_H * 0x10000 + swmib->rx_pkts_L);
				printf("\n");
				printf("\n");
				printf("                        Press ESC Key to Stop!  \n");

			}*/
			else if (txInfo->host_loopback)
			{
				printf("                                          TX\n");
				printf("                TX Good Bytes Counts: %10lu\n",swmib->tx_good_octets_H * 0x100000000 + swmib->tx_good_octets_L);
				printf("              TX Good Packets Counts: %10lu\n",swmib->tx_good_pkts_H * 0x10000 + swmib->tx_good_pkts_L);
				printf("               TX Total Bytes Counts: %10lu\n",swmib->tx_octets_H * 0x100000000 + swmib->tx_octets_L);
				printf("             TX Total Packets Counts: %10lu\n",swmib->tx_pkts_H * 0x10000 + swmib->tx_pkts_L);
				printf("\n");
				printf("\n");
				printf("                                          RX\n");
				printf("                RX Good Bytes Counts: %10lu\n",swmib->rx_good_octets_H * 0x100000000 + swmib->rx_good_octets_L);
				printf("              RX Good Packets Counts: %10lu\n",swmib->rx_good_pkts_H * 0x10000 + swmib->rx_good_pkts_L);
				printf("               RX Total Bytes Counts: %10lu\n",swmib->rx_octets_H * 0x100000000 + swmib->rx_octets_L);
				printf("             RX Total Packets Counts: %10lu\n",swmib->rx_pkts_H * 0x10000 + swmib->rx_pkts_L);
				printf("\n");
				printf("\n");
				printf("                        Press ESC Key to Stop!  \n");

			}
		}
done:
	return 0;
}           
static int
txdmaBlastHostBack(
    BUFENTRY *pDmabuf,
    ULONG uDmaBufSize,
    char buf[],
    int nbytes,
	ULONG *pTxBufaddr,
    ULONG *pTxIn,
    ULONG *pTxOut,
    ULONG *pTxringaddr,
	ULONG ipg
	)
{
    int ipg_start_time;
	ULONG flags = (CTRL_SOF | CTRL_EOF | CTRL_IOC | nbytes);
	
	WriteBuffer( pDmabuf, uDmaBufSize, pTxBufaddr[0], &buf[0], nbytes, 0 );
	
	if ((*pTxOut == NTXD - 1)) 
		flags |= CTRL_EOT;
	
	posttxd(  pDmabuf, uDmaBufSize, *pTxringaddr, *pTxOut, pTxBufaddr[0], flags );
	*pTxOut = (*pTxOut + 1) % NTXD;
	RegWr(REGOFFSET(dmaregs.xmtptr), (*pTxOut % NTXD) * sizeof(dmadd_t));
	if (ipg > 0)
	{
		ipg_start_time = clock();
		while(clock() < ipg_start_time + ipg)
		{
			if (kbhit())
			{
				if (toupper(getch())==KEY_ESC)
					break;
			}
		}	
	}
	if ( !checkinterror() )
		return( FALSE );
	
    *pTxIn = *pTxOut;
    
    RegWr(REGOFFSET(intstatus), I_XI);
    
    return( TRUE );
}
