/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 * Name: Class4xsb.h
 *
 * Description: Broadcom Management API.
 *
 * Author: Steven Siener
 *
 * $Log:: /Source/Bcm44xx/b44diag/Cl4xsb.h                              $
 * 
 * 8     6/11/02 4:04p Ahui
 * 
 * 7     4/25/02 11:11a Ahui
 * Modified loopback test to support 100/10BT loopback
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 * 
 * 1     2/08/02 5:26p Hyfeng
 * 
 ****************************************************************************/
#ifndef __CL4XSB_H__
#define __CL4XSB_H__



//#include "ClassNic.h"
#include "types.h"
#include "general.h"


#define SPROM_BASE		4096

#include "sbconfig.h"
#include "enet47xx.h"

/****************************************************************************/
/* Loopback type.                                                           */
/****************************************************************************/
#define BMAPI_LOOPBACK_TYPE_MAC         0
#define BMAPI_LOOPBACK_TYPE_PHY         1
#define BMAPI_LOOPBACK_TYPE_TWO_NODE    2
#define BMAPI_LOOPBACK_TYPE_EXT_100     3
#define BMAPI_LOOPBACK_TYPE_EXT_10      4
#define LOOPBACK_TYPE_MAC               BMAPI_LOOPBACK_TYPE_MAC
#define LOOPBACK_TYPE_PHY               BMAPI_LOOPBACK_TYPE_PHY
#define LOOPBACK_TYPE_TWO_NODE          BMAPI_LOOPBACK_TYPE_TWO_NODE
#define LOOPBACK_TYPE_EXT_100			BMAPI_LOOPBACK_TYPE_EXT_100
#define LOOPBACK_TYPE_EXT_10			BMAPI_LOOPBACK_TYPE_EXT_10
#ifdef REGOFFSET
#undef REGOFFSET
#endif

#define REGOFFSET(n) ((USHORT)&((bcmenetregs_t  *)NULL)->n)


/****************************************************************************/
/* External functions in BMAPI environment.                                 */
/****************************************************************************/
#define BMAPI_DEBUG_INFORMATION         0x00000001
#define BMAPI_DEBUG_WARNING             0x00000002
#define BMAPI_DEBUG_ERROR               0x00000004
#define BMAPI_DEBUG_LOG_TO_FILE         0x00000008

#define CRC8_INIT_VALUE  0xff		/* Initial CRC8 checksum value */
#define CRC8_GOOD_VALUE  0x9f		/* Good final CRC8 checksum value */
#define CRC16_INIT_VALUE 0xffff		/* Initial CRC16 checksum value */
#define CRC16_GOOD_VALUE 0xf0b8		/* Good final CRC16 checksum value */
#define CRC32_INIT_VALUE 0xffffffff /* Initial CRC32 checksum value */
#define CRC32_GOOD_VALUE 0xdebb20e3 /* Good final CRC32 checksum value */

/****************************************************************************/
/* Error string constants.                                                  */
/****************************************************************************/
extern const char *ERR_UNABLE_TO_SUSPEND_DRIVER;
extern const char *ERR_DRIVER_NOT_SUSPENDED;
extern const char *ERR_UNABLE_TO_RESUME_DRIVER;
extern const char *ERR_DIAG_NOT_INIT;


typedef enum {ET_MAC, ET_FIFO, ET_MII, ET_EXT_10, ET_EXT_100 } loopback_t;
typedef struct {
    ULONG phys_addr;
    ULONG vaddr;
    ULONG size;
} BUFENTRY, *PBUFENTRY;

typedef struct {
	
    u32 tx_good_octets_H;
    u16 tx_good_pkts_H;
	u32 tx_octets_H;
    u16 tx_pkts_H;
	u16 tx_broadcast_pkts_H;
	u16 tx_multicast_pkts_H;
	u16 tx_len_64_H;
	u16 tx_len_65_to_127_H;
	u16 tx_len_128_to_255_H;
	u16 tx_len_256_to_511_H;
	u16 tx_len_512_to_1023_H;
	u16 tx_len_1024_to_max_H;
	u16 tx_jabber_pkts_H;
	u16 tx_oversize_pkts_H;
	u16 tx_fragment_pkts_H;
	u16 tx_underruns_H;
	u16 tx_total_cols_H;
	u16 tx_single_cols_H;
	u16 tx_multiple_cols_H;
	u16 tx_excessive_cols_H;
	u16 tx_late_cols_H;
	u16 tx_defered_H;
	u16 tx_carrier_lost_H;
	u16 tx_pause_pkts_H;

	u32 rx_good_octets_H;
	u16 rx_good_pkts_H;
	u32 rx_octets_H;
	u16 rx_pkts_H;
	u16 rx_broadcast_pkts_H;
	u16 rx_multicast_pkts_H;
	u16 rx_len_64_H;
	u16 rx_len_65_to_127_H;
	u16 rx_len_128_to_255_H;
	u16 rx_len_256_to_511_H;
	u16 rx_len_512_to_1023_H;
	u16 rx_len_1024_to_max_H;
	u16 rx_jabber_pkts_H;
	u16 rx_oversize_pkts_H;
	u16 rx_fragment_pkts_H;
	u16 rx_missed_pkts_H;
	u16 rx_crc_align_errs_H;
	u16 rx_undersize_H;
	u16 rx_crc_errs_H;
	u16 rx_align_errs_H;
	u16 rx_symbol_errs_H;
	u16 rx_pause_pkts_H;
	u16 rx_nonpause_pkts_H;


    u32 tx_good_octets_L;
    u16 tx_good_pkts_L;
	u32 tx_octets_L;
    u16 tx_pkts_L;
	u16 tx_broadcast_pkts_L;
	u16 tx_multicast_pkts_L;
	u16 tx_len_64_L;
	u16 tx_len_65_to_127_L;
	u16 tx_len_128_to_255_L;
	u16 tx_len_256_to_511_L;
	u16 tx_len_512_to_1023_L;
	u16 tx_len_1024_to_max_L;
	u16 tx_jabber_pkts_L;
	u16 tx_oversize_pkts_L;
	u16 tx_fragment_pkts_L;
	u16 tx_underruns_L;
	u16 tx_total_cols_L;
	u16 tx_single_cols_L;
	u16 tx_multiple_cols_L;
	u16 tx_excessive_cols_L;
	u16 tx_late_cols_L;
	u16 tx_defered_L;
	u16 tx_carrier_lost_L;
	u16 tx_pause_pkts_L;

	u32 rx_good_octets_L;
	u16 rx_good_pkts_L;
	u32 rx_octets_L;
	u16 rx_pkts_L;
	u16 rx_broadcast_pkts_L;
	u16 rx_multicast_pkts_L;
	u16 rx_len_64_L;
	u16 rx_len_65_to_127_L;
	u16 rx_len_128_to_255_L;
	u16 rx_len_256_to_511_L;
	u16 rx_len_512_to_1023_L;
	u16 rx_len_1024_to_max_L;
	u16 rx_jabber_pkts_L;
	u16 rx_oversize_pkts_L;
	u16 rx_fragment_pkts_L;
	u16 rx_missed_pkts_L;
	u16 rx_crc_align_errs_L;
	u16 rx_undersize_L;
	u16 rx_crc_errs_L;
	u16 rx_align_errs_L;
	u16 rx_symbol_errs_L;
	u16 rx_pause_pkts_L;
	u16 rx_nonpause_pkts_L;




} SWMIBCOUNTERS, *PSWMIBCOUNTERS;
U32 SB4xDiagTestRegisters(u32 req_iteration, u8 indir);
U32 SB4xDiagTestMii(u32 req_iteration);
U32 SB4xDiagTestIntMemory(u32 req_iteration);
U32 SB4xDiagTestLoopBack(u32 req_iteration, ULONG Lbtype,u32,u32);
U32 SB4xDiagTestLEDs( U32 req_iteration, ULONG BlinkDurationSec);
U32 SB4xDiagTestEEPROM(u32 req_iteration );
void			core_reset(void);
void			core_disable(void);
void			reset(void);
void			miireset(void);
U16				miird(UCHAR r);
void			miiwr(UCHAR r, USHORT val );
void			RegOr( ULONG Offset, ULONG Value32 );
void			RegAnd( ULONG Offset, ULONG Value32 );
//unsigned int    InDiagMode( CHAR *pRstr );
U8				crc8( U8 *pdata, U32 nbytes, U8 crc );
U16				PhyRd(UCHAR Offset);
void			PhyWr(UCHAR Offset, U16 Value16);
ULONG           RegRd(ULONG Offset);
void			RegWr(ULONG Offset, ULONG Value32);
void			RegWr16(u16 Offset, u16 Value16);
ULONG			EpromRd( ULONG Offset);
void			EpromWr( U16 Offset, U16 Value16);
U32				SB4xDiagTestBlast(u32);
void			resetcountersBlast(void);
void			readcountersBlast(int);
unsigned int	displaycountersBlast(int);
U32 SB4xDiagprgEEPROM(u08* Macbuf);


#endif // __CL4XSB_H__
