/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ U T I L . H
 *
 *  Description: This file contains utility routines for T3.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *
 *******************************************************************/
#ifndef __PCI_H__
#define __PCI_H__

#include "t3_gdata.h"

/****************************************************************************/
/*                                                                          */
/* Module: PCI.H                                                            */
/*                                                                          */
/* Purpose: Definitions for PCI Library                                     */
/*                                                                          */
/****************************************************************************/

#define CONFIG_CMD(bus,device_fn,where) (0x80000000 | (bus << 16) | (device_fn << 8) | (where & ~0x3))

#define CARRY_FLAG 0x01         /* 80x86 Flags Register Carry Flag bit */

#define NOCFG() (CURRCARD.base_address_offset==OFFSETOF(T3_STD_MEM_MAP,PciCfg.MemBaseAddrLow1))

/****************************************************************************/
/* PCI Functions                                                            */
/****************************************************************************/

#define PCI_FUNCTION_ID          0xb1
#define PCI_BIOS_PRESENT         0x01
#define FIND_PCI_DEVICE          0x02
#define FIND_PCI_CLASS_CODE      0x03
#define READ_CONFIG_BYTE         0x08
#define READ_CONFIG_WORD         0x09
#define READ_CONFIG_DWORD        0x0a
#define WRITE_CONFIG_BYTE        0x0b
#define WRITE_CONFIG_WORD        0x0c
#define WRITE_CONFIG_DWORD       0x0d

/****************************************************************************/
/* PCI Return Code List                                                     */
/****************************************************************************/

#define SUCCESSFUL               0x00
#define NOT_SUCCESSFUL           0x01
#define FUNC_NOT_SUPPORTED       0x81
#define BAD_VENDOR_ID            0x83
#define DEVICE_NOT_FOUND         0x86
#define BAD_REGISTER_NUMBER      0x87

/****************************************************************************/
/* PCI Configuration Space Registers                                        */
/****************************************************************************/

#define PCI_CS_VENDOR_ID         0x00
#define PCI_CS_DEVICE_ID         0x02
#define PCI_CS_COMMAND           0x04
#define PCI_CS_STATUS            0x06
#define PCI_CS_REVISION_ID       0x08
#define PCI_CS_CLASS_CODE        0x09
#define PCI_CS_CACHE_LINE_SIZE   0x0c
//#define PCI_CS_BIST              0x0c
#define PCI_CS_MASTER_LATENCY    0x0d
#define PCI_CS_HEADER_TYPE       0x0e

#define PCI_CS_BASE_ADDRESS_0    0x10
#define PCI_CS_BASE_ADDRESS_1    0x14
//#define PCI_CS_BASE_ADDRESS_2    0x18
//#define PCI_CS_BASE_ADDRESS_3    0x1c
#define PCI_CS_INTSTATUS	     0x20
#define PCI_CS_INTMASK           0x24
//#define PCI_CS_RESERVED_0        0x28
//#define PCI_CS_RESERVED_1        0x2c
#define PCI_CS_SUBSYSTEMVENDORID 0x2c
#define PCI_CS_SUBSYSTEMID       0x2e
#define PCI_CS_EXPANSION_ROM     0x30
#define PCI_CS_CAPPTR            0x34
//#define PCI_CS_RESERVED_3        0x38
#define PCI_CS_INTERRUPT_LINE    0x3c
#define PCI_CS_INTERRUPT_PIN     0x3d
#define PCI_CS_MIN_GNT           0x3e
#define PCI_CS_MAX_LAT           0x3f
#define PCI_CS_BAR0WINDOW        0x80
#define PCI_CS_PCIINTMMASK		 0x94
#define PCI_CS_PCI2SBMB		     0x98
#define PCI_CS_BlackDoorAddr	 0xA0
#define PCI_CS_BlackDoorData	 0xA4




/****************************************************************************/
/*   PCILIB Prototypes                                                      */
/****************************************************************************/

int pci_bios_present(u08 *hardware_mechanism,
                     u16 *interface_level_version,
                     u08 *last_pci_bus_number);

int find_pci_device(u16 device_id,
                    u16 vendor_id,
                    u16 index,
                    u08 *bus_number,
                    u08 *device_and_function);

int find_pci_class_code(u32 class_code,
                        u16 index,
                        u08 *bus_number,
                        u08 *device_and_function);

int read_configuration_byte(u08 bus_number,
                            u08 device_and_function,
                            u08 register_number,
                            u08 *byte_read);

int read_configuration_word(u08 bus_number,
                            u08 device_and_function,
                            u08 register_number,
                            u16 *word_read);

int read_configuration_dword(u08 bus_number,
                             u08 device_and_function,
                             u08 register_number,
                             u32 *dword_read);

int write_configuration_byte(u08 bus_number,
                             u08 device_and_function,
                             u08 register_number,
                             u08 byte_to_write);

int write_configuration_word(u08 bus_number,
                             u08 device_and_function,
                             u08 register_number,
                             u16 word_to_write);

int write_configuration_dword(u08 bus_number,
                              u08 device_and_function,
                              u08 register_number,
                              u32 dword_to_write);

#ifndef EFI
void insb(u16 port, void *buf, int count);
void insw(u16 port, void *buf, int count);
void insd(u16 port, void *buf, int count);

void outsb(u16 port, void *buf, int count);
void outsw(u16 port, void *buf, int count);
void outsd(u16 port, void *buf, int count);
#endif

#endif /* __PCI_H__ */
