/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: P R O T O C O L . H
 *
 *  Description: This file contains macro and data structure definition
 *  relating to protocol stuffs.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *
 *******************************************************************/

#ifndef __PROTOCOL_H__
#define __PROTOCOL_H__

typedef union IP_ADDR
{
  u08 byte[4];
  u32 word;
}IP_ADDR,*pIP_ADDR;

#define DEFAULT_IP_TTL  32
#define IP_VER          0x4

typedef struct IP_HDR
{
  u08 ver_hlen;
  u08 tos;
  u16 total_len;
  u16 iden;
 #define IP_HDR_FRAG_MF    (1 << 13)
 #define IP_HDR_FRAG_DF    (1 << 14)
  u16 frag_offset;
  u08 ttl;
  u08 protocol;
  u16 chksum;
  IP_ADDR src_ip;
  IP_ADDR dest_ip;
}IP_HDR,*pIP_HDR;

typedef struct UDP_HDR
{
  u16 src_port;
  u16 dest_port;
  u16 length;
  u16 chksum;
}UDP_HDR,*pUDP_HDR;

typedef struct TCP_HDR
{
  u16 src_port;
  u16 dest_port;
  u32 seq_num;
  u32 ack_seq_num;
  u16 offset_code;
  u16 window;
  u16 chksum;
  u16 urg_ptr;
}TCP_HDR,*pTCP_HDR;

typedef u08 MAC_ADDR[6];

typedef struct PACKET
{
  MAC_ADDR dest_mac;
  MAC_ADDR src_mac;
  u16 len_type;
  union 
  {
    IP_HDR  ip;
  }layer2_prot;

  union
  {
    UDP_HDR udp;
    TCP_HDR tcp;
  }layer3_prot;

  u08 data[1];
}PACKET,*pPACKET;

#define  LAYER2_TYPE_IP  0x800
#define  LAYER2_TYPE_ARP 0x806
#define  LAYER2_TYPE_BRCM_MAGIC 0x1234

#define LAYER3_PROT_UDP     17
#define LAYER3_PROT_TCP     6
#define TX_STD_BUFSIZE     9018

#define HOST_TO_NET16(x) (u16)(((x >> 8) & 0xff) | (x << 8) & 0xff00)
#define HOST_TO_NET32(x) (u32)(((x >> 24) & 0xff) | (x << 8) & 0xff0000 | \
                               ((x >> 8) & 0xff00) | (x << 24) & 0xff000000)
#define NET_TO_HOST(x) (u16)(((x >> 8) & 0xff) | (x << 8) & 0xff00)
#define NET_TO_HOST32(x) (u32)(((x >> 24) & 0xff) | (x << 8) & 0xff0000 | \
                               ((x >> 8) & 0xff00) | (x << 24) & 0xff000000)

#endif /* __PROTOCOL_H__ */
