/*
 * sromutl.c: Program the srom attached to a BCM4210 chip.
 *
 * Copyright 1999, Broadcom Corporation.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Broadcom Corporation.
 *
 * $Id: sromutl.c,v 1.21 2001/07/27 23:20:27 stafford Exp $
 */


#include "sromutl.h"
#include "t_io.h"
//char	*myname;



int
hexstring(char *hstr, uint16 *hex, int n, int byteswap)
{
	int	len, sz, curr, i;
	char	tmpstr[8];		/* Only need 5, but round it up */
	uint16 temp;

	memset(tmpstr, 0, sizeof (tmpstr));
	len = strlen(hstr);

	/* Skip leading 0x  if any */
	if ((len > 2) && (hstr[0] == '0') &&
	    ((hstr[1] == 'x') || (hstr[1] == 'X'))) {
		curr = 2;
		len -= 2;
	} else
		curr = 0;

	if (len > (n * 4)) {
		/* Ignore leading chars if more than words to receive them */
		curr += len - (n * 4);
		len = n * 4;
	} else {
		/* Else calculate how many words we'll use */
		n = (len + 3) / 4;
	}

	for (i = 0; i < n; i++) {
		sz = len - ((n - i - 1) * 4);
		strncpy(tmpstr, hstr + curr, sz);
		tmpstr[sz] = '\0';
		if (byteswap) {
			temp = (uint16)strtoul(tmpstr, NULL, 16);
			hex[i] = temp << 8;
			hex[i] += temp >> 8;
		} else
			hex[i] = (uint16)strtoul(tmpstr, NULL, 16);
		len -= sz;
		curr += sz;
	}

	return (n);
}


void
srdump(uint16 *srom, int nw, char *header, chip_type_t ChipType)
{
	uint16  Enet_MAC[3];

	int i;
	int j;

	printf("%s", header);
	for (i = 0; i < nw; i++) {
		if ((i % 8) == 0)
			printf("\n  srom[%03d] =", i);
		printf("  0x%04x", srom[i]);
	}
	printf("\n");

    Enet_MAC[0] = srom[SROM_E0_MACHI];
    Enet_MAC[1] = srom[SROM_E0_MACMID];
    Enet_MAC[2] = srom[SROM_E0_MACLO];

    /* Print out this info... */
    printf("SPROM version: %d\n", (srom[SROM_SIZE - 1] & 0xFF));
    if (ChipType == BCM4401) {
        printf("Eth0 MAC Address: %02X:%02X:%02X:%02X:%02X:%02X\n",(Enet_MAC[0] >> 8),
            (Enet_MAC[0] & 0xff), (Enet_MAC[1] >> 8), (Enet_MAC[1] & 0xff),
            (Enet_MAC[2] >> 8), (Enet_MAC[2] & 0xff) );
    }
	printf("Vendor ID: %04X\n", BROADCOM_VENDOR_ID);
	printf("Device ID: %04X\n", srom[4]);
    printf("SubSystem Vendor ID: %04X\n", srom[SROM_VID]);
    printf("SubSystem Device ID: %04X\n", srom[SROM_SSID]);
	printf("Boot ROM %senabled\n", (srom[SROM_COMMPW] & (1 << BR_PRESSHIFT) ? "" : "not "));
	if (srom[SROM_COMMPW] & (1 << BR_PRESSHIFT))
	{
		j=(srom[SROM_COMMPW] >> 9) & 0x7F; //bit 15:9
		switch (j)
		{
			case 0: 
				printf("Boot ROM Size = 16KB\n");
				break;
			case 1:
				printf("Boot ROM Size = 32KB\n");
				break;
			case 2:
				printf("Boot ROM Size = 64KB\n");
				break;
			case 3:
				printf("Boot ROM Size = 128KB\n");
				break;
		}
		
	}
    
}

/* 
	write global sprom header bits
*/
void
wrhdr (uint16 *rom, chip_info_t *ci, uint16 deviceid, uint16 subvendor, uint16 subdevice, uint16 brsize)
{
	uint16 val;

	/* Hardware control */
	val = ci->funmask & HW_FUNMSK;
	if (ci->fclk != 0) {
	    val |= HW_FCLK;
	}
	if (ci->cbm != 0) {
	    val |= HW_CBM;
	}

	val |= ((ci->pi << HW_PISHIFT) & HW_PIMSK);
	rom[SROM_HWCTL] = val;

	/* Common Power */
	val = ci->cpwr;

	/* boot rom bits */
	if (brsize != 0)
	{
		val |= (1 << BR_PRESSHIFT);
		val |= (brsize-1 << BR_SIZESHIFT);
	}
	rom[SROM_COMMPW] = val;

	/* Subsytem Id */
	rom[SROM_SSID] = subdevice;

	/* Subsytem Vendor */
	rom[SROM_VID] = subvendor;
}

/*
	write bits for a specified function
*/
void
wrfun (uint16 *rom, uint fn, fn_info_t *finfo) 
{
	uint fbase;
	uint16 b1;
	uint i;
	uint32 bar0val;

	fbase = SROM_FN0 + (fn * SROM_FNSZ);
	bar0val = SBCORES_BASE + (fn * SBCORES_EACH);

	/* Device id */
	rom[fbase + SRFN_DID] = finfo->device;

	/* Class code & D0 Power */
	rom[fbase + SRFN_CCL] = finfo->classlo;
	rom[fbase + SRFN_CCHD0] = finfo->classhi;

	/* PMEsource D1D2D3 Power */
	rom[fbase + SRFN_PMED123] = finfo->pmesrc;

	/* Bar1 enable/size, pme reports & Bar0 hi */
	if (finfo->bar1sz == 0) {
		b1 = 0;
	} else {
		b1 = 0xffff;
		for (i = 0; i <= (B1SZMSK >> B1SZSH); i++) {
			if ((1 << i) == finfo->bar1sz) {
				b1 = i;
			}
		}
		if (b1 == 0xffff) {
			printf ("panic: Error determining bar1 sprom value\n");
			exit (8);
		}
		b1 = ((b1 << B1SZSH) | B1E);
	}
	b1 = b1 | ((finfo->pmerep & PMERMSK) | (bar0val & B0LMSK));
	rom[fbase + SRFN_B1PMER] = b1 | (bar0val & B0LMSK);

	/* Bar0 value hi */
	rom[fbase + SRFN_B0H] = bar0val >> 16;

	/* CIS Pointer */
	rom[fbase + SRFN_CISL] = 0;
	rom[fbase + SRFN_CISH] = 0;
}

