/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ E E C F G . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_eecfg.c 12    7/19/02 10:07a Ahui $
 *
 *  Description: This file contains utility routines for Serial 
 *  EEPROM Configuration.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        07/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_eecfg.c $
 * 
 * 12    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 11    7/11/02 1:02p Ahui
 * Clean up unused code.
 * 
 * 10    7/08/02 12:24p Ahui
 * Modified code to support new card srom,  mac address programming, boot
 * rom enable/disable and boot rom size programming.
 * 
 * 9     6/13/02 10:04a Ahui
 * Clean up unused code
 * 
 * 8     5/16/02 5:57p Mnima
 * use input eeprom name
 * 
 * 7     4/25/02 11:06a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

/** include files **/

#include <string.h>
#include "general.h"
#include "eeprom.h"
#include "t3_eprom.h"
#include "sromutl.h"
#include "pci.h"

int eecfg_read(int verbose, pEEPROM_IMAGE eeprom)
{
    // Read current Serial EERPOM info. 
    Vprintf(verbose,"Reading current serial eeprom ... ");
    if (eprom_read_blk(0,(u32 *)eeprom,sizeof(eeprom_info_t)/4) < 0)
	return test_failed0(ERR_EEP_CANNOT_READ);
    Vprintf(verbose,"OK\n");
    return NO_ERROR;
}


/* len,offset - in byte */
int eecfg_write(int verbose, u08 * buffer, u32 offset, u32 len)
{
    u32 val32;
    int             i = 0, j=0;
    u16 *   tempaddr;             
	u16 sprom_size = SROM_SIZE;
	u16             *image;
	chip_type_t     chip_type = BCM4401;
	u32             tempread;
	int             status;

	image = (uint16 *)malloc(sprom_size*2);
    /* Round up to words */
    len = ((len + 3)/4) * 4;

    for (i = 0; i < (len/4); i++)
	{
		image[i] = (buffer[i*2+1 ] << 8) |  (buffer[i*2]);
	}
	free(buffer);
	if (verbose)
		printf("Enabling SPROM writes.\n");

	read_configuration_dword(CURRCARD.bus_number,
			      CURRCARD.device_and_function,
			      CFG_SROM_WRITABLE_OFFSET,&val32);
	write_configuration_dword(CURRCARD.bus_number,
			      CURRCARD.device_and_function,
			      CFG_SROM_WRITABLE_OFFSET,val32 | SROM_WRITEABLE);
   
	MM_Wait(5000);  
	for (i = 0; i < len/4; i++)
		{
		       tempaddr = (U16 *)CURRCARD.base_address + (SPROM_BASE/2) + i; 
		       * tempaddr = image[i];
				MM_Wait(20000); 
		}
	
	// assumes 4-byte aligned offset 
		write_configuration_dword(CURRCARD.bus_number,
			      CURRCARD.device_and_function,
			      CFG_SROM_WRITABLE_OFFSET,val32 );

	 MM_Wait(5000);  

    if (verbose)
		srdump(image, sprom_size, "Programmed:", chip_type);

    // check to see if SPROM programmed correctly 
	j=0;
	for (i = 0; i < sprom_size/2; i++)
		{
			tempread = RegRd(SPROM_BASE + (i*4));
			
			if (((tempread & 0xffff) != image[j]) || 
			   (((tempread & 0xffff0000)/ 0x10000) != image[j+1]))
				break;
			j=j+2;
		}       
    if (i < sprom_size/2) 
	{
	printf("SPROM not programmed correctly!\n");
	printf("Location: %d\n", i);
		status=HAS_ERROR;
    }
	//util_init_pci_and_find_tigons (0, 0);
	free (image);
	return status;
	
}
