/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_eprom.h 8     7/19/02 10:07a Ahui $
 *
 ****************************************************************************/
#ifndef __T3_EPROM_H__
#define __T3_EPROM_H__

#include "general.h"

#define CLK_OE	   	        (1 << 0)
#define CLK_BIT	   	        (1 << 1)
#define DATA_OE_BIT	   	(1 << 3)
#define DATA_OUT_BIT	   	(1 << 4)
#define DATA_IN_BIT	   	(1 << 5)

#define _5_VOLT

#ifdef _5_VOLT		// 5 volt operation
#define THIGH		10		/* > 0.6 us */
#define H_THIGH		5
#define TLOW		16		/* > 1.2 us */
#define H_TLOW		8
#define START_SU	10		/* > 0.6 us (setup time) */
#define START_HO	10		/* > 0.6 us (hold time) */
#define STOP_SU		10		/* > 0.6 us (setup time) */
#define TBUF		16		/* > 1.2 us */
#else			// 3.3 volt operation
#define THIGH		80		/* > 4.0 us */
#define H_THIGH		40
#define TLOW		64		/* > 4.7 us */
#define H_TLOW		32
#define START_SU	80		/* > 4.0 us (setup time) */
#define START_HO	80		/* > 4.0 us (hold time) */
#define STOP_SU		80		/* > 4.0 us (setup time) */
#define TBUF		64		/* > 4.7 us */
#endif

#define SERIAL_WRITE (u08)0xa0
#define SERIAL_READ  (u08)0xa1

#define VPD_WAIT           1000


/* function prototyping */
int  eprom_read_word(u32 eeprom_addr, u32 * data);
u32  eprom_read_seprom(u32 addr);
int  eprom_read_blk(u32 start,u32 *buffer,u32 num_words);
int  eprom_write_blk(u32 start,u32 *buffer,u32 num_words);
void eprom_init();
void eprom_wait (u32);
int  eprom_write_word(u32 device,u32 eeprom_addr,u32 wr_data);
int  eprom_dump(s08 * fn, u32 length);
void eprom_fill_blk(u32 start, u32 end,u32 wr_data);
//void eprom_vpd_fill_blk(u32 start, u32 end,u32 wr_data);
void eprom_EepSetClock(int);
void eprom_EepSetData(int);


#endif /* __T3_EPROM_H__ */

 /******************************************************************
 *
 *  Description: This file contains test control routines
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_eprom.h $
 * 
 * 8     7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 7     5/29/02 6:33p Ahui
 * Enabled eprom_write_word().
 * 
 * 6     4/25/02 11:06a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

