/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ I O . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_io.c 4     7/19/02 10:07a Ahui $
 *
 *  Description: This file contains utility routines
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_io.c $
 * 
 * 4     7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

#include <stdio.h>
#include "general.h"
#include "t3_util.h"
#include "t_io.h"
#include "pci.h"
#include "t3_gdata.h"

/*
 *  Functions to access roadrunner tigon via shared memory and via
 *  configuration space.   Shared memory access functions assume linear
 *  address space where physical_address === logical_address.
 *  Configuration functions use functions in "pcilib.c".
 *  The function "t_io_init()" must be called each time a new tigon card is
 *  selected, this enables the setup of a global pointer to the tigon shared
 *  memory so subsequent calls to read and/or write the memory do NOT need
 *  to pass a pointer.
 */
#define SHARED_MEM_SIZE         0x1000L


/*
 * pointer to rr shared memory block
 */
void ikos_delay();

#ifdef IKOS
#define IKOS_DELAY ikos_delay()
#else
#define IKOS_DELAY
#endif

/************************************************************************
 *  Functions to write/read shared memory by words/bytes                *
 ************************************************************************/
void
t_writemem32 (u16 addr, u32 value)
{
  IKOS_DELAY;

  if (CURRCARD.indirect_access)
    {
      util_write_isram((u32)addr,value);
    }
  else
    {
      iprintf("reg wr: 0x%04x <-- 0x%08x\n",addr,value);
#ifdef EFI
        b57IoFuncs->Mem.Write(b57IoFuncs, IO_UINT32,
                    (UINT64) (CURRCARD.base_address) + addr, 
                    1, &value);
#else
      *(((u32 *)CURRCARD.base_address) + (addr >> 2)) = value;
#endif
    }
}

void
t_writemem32b (u16 addr, u32 value)
{
  IKOS_DELAY;

  iprintf("reg wr: 0x%04x <-- %08x\n",addr,value);
#ifdef EFI
        b57IoFuncs->Mem.Write(b57IoFuncs, IO_UINT32,
                    (UINT64) (CURRCARD.base_address) + addr, 
                    1, &value);
#else
  *(((u32 *)CURRCARD.base_address) + (addr >> 2)) = value;
#endif
}

void
t_writemem16 (u16 addr, u16 value)
{
    int shiftbits = (addr&0x2) ? 16 : 0;
    u32 val;
    
    addr &= 0xfffc;
    val = t_readmem32 (addr);
    val = val & ~(0xffff << shiftbits) | (value << shiftbits);
    t_writemem32 (addr, val);
}

void
t_writemem08 (u16 addr, u08 value)
{
	int shiftbits = (addr&0x3) << 3;
	u32 val;
    
    addr &= 0xfffc;
    val = t_readmem32 (addr);
	val = val & ~(0xff << shiftbits) | (value << shiftbits);
	t_writemem32 (addr, val);
}

u32
t_readmem32 (u16 addr)
{
  u32 data;

  IKOS_DELAY;

  if (CURRCARD.indirect_access)
    {
      data = util_read_isram((u32)addr);
    }
  else
    {
#ifdef EFI
        b57IoFuncs->Mem.Read(b57IoFuncs, IO_UINT32,
                    (UINT64) (CURRCARD.base_address) + addr, 
                    1, &data);
#else
      data = *(((u32 *)CURRCARD.base_address) + (addr >> 2));
      iprintf("reg rd: 0x%04x --> %08x\n",addr,data);
#endif
    }


  return data;
}

u16 t_readmem16 (u16 addr)
{
    u32 val = t_readmem32 (addr&0xfffc);
    return (val >> (addr&0x2 ? 16 : 0));
}

u08 t_readmem08 (u16 addr)
{
  u32 val = t_readmem32 (addr&0xfffc);
  return (val >> ((addr&0x3) << 3));
}


/************************************************************************
 *  Functions to write/read configuration space by words/halfwords/bytes*
 ************************************************************************/
void
t_writeconf08 (u16 addr, u08 value)
{
	card_t *tigon = get_currcard ();
	if (write_configuration_byte (tigon->bus_number,
				      tigon->device_and_function,
				      addr,
				      value) != SUCCESSFUL)
	   printf("Error - cannot write PCI config space\n");
}
void
t_writeconf16 (u16 addr, u16 value)
{
	card_t *tigon = get_currcard ();
	if (write_configuration_word (tigon->bus_number,
				      tigon->device_and_function,
				      addr,
				      value) != SUCCESSFUL)
	   printf("Error - cannot write PCI config space\n");
}
void
t_writeconf32 (u16 addr, u32 value)
{
	card_t *tigon = get_currcard ();
	if (write_configuration_dword (tigon->bus_number,
				       tigon->device_and_function,
				       addr,
				       value) != SUCCESSFUL)
	   printf("Error - cannot write PCI config space\n");
}

u08
t_readconf08 (u16 addr)
{
	u08 val08;
	card_t *tigon = get_currcard ();
	if (read_configuration_byte (tigon->bus_number,
				     tigon->device_and_function,
				     addr,
				     &val08) != SUCCESSFUL)
	   printf("Error - cannot read PCI config space\n");
	return val08;
}
u16
t_readconf16 (u16 addr)
{
	u16 val16;
	card_t *tigon = get_currcard ();
	if (read_configuration_word (tigon->bus_number,
				     tigon->device_and_function,
				     addr,
				     &val16) != SUCCESSFUL)
	   printf("Error - cannot read PCI config space\n");
	return val16;
}
u32
t_readconf32 (u16 addr)
{
	u32 val32;
	card_t *tigon = get_currcard ();
	if (read_configuration_dword (tigon->bus_number,
				      tigon->device_and_function,
				      addr,
				      &val32) != SUCCESSFUL)
	   printf("Error - cannot read PCI config space\n");
	return val32;
}

void ikos_delay()
{
  int i;
  static int j;

  if (global.verbose == 2)
    return;

  for (i =0 ; i < 10000;i++)
    {
      j++;
    }
}


/* end of t_io.c */


