/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ M A I N . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_main.c 12    7/19/02 10:07a Ahui $
 *
 *  Description: This file contains BCM5700 Diagnostics main entry 
 *  point.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_main.c $
 * 
 * 12    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 11    7/08/02 12:24p Ahui
 * Modified code to support new card srom,  mac address programming, boot
 * rom enable/disable and boot rom size programming.
 * 
 * 10    6/13/02 10:04a Ahui
 * Fixed a bug in -fmac option.
 * 
 * 9     6/11/02 11:53a Ahui
 * Removed -z option 
 * Added  fmac option
 * 
 * 8     5/29/02 6:32p Ahui
 * Enabled -mac option (t3_main.c)
 * 
 * 7     5/16/02 5:58p Mnima
 * use input eeprom name
 * 
 * 6     4/18/02 3:46p Ahui
 * 1. Modified loopback test to tx 200 1000byte packets.
 * 2. Added External loop back test.
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "general.h"
#include "t3_gdata.h"
#include "t3_tx.h"
#include "t3_mdiag.h"
#include "t3_cli.h"
#include "t3_scrn.h"
#include "t3_tests.h"
#include "t3_util.h"
#include "t3_hist.h"
#include "t3_eecfg.h"
#include "t3_comm.h"
#include "Cl4xsb.h"


typedef enum arg_tag {
    none, 
    OPTcardnum,         // -c
    OPTlogfile,         // -l
//    OPTeepromfile,         // -z
    //OPTverbose,         // -v         -- not in help 
    OPTradix,           // -r         -- not in help
    //OPTMemBaseAddr,     // -a         -- not in help
    //OPTVideoMode,       // -h         -- not in help
    OPManuf,            // -b44eng    -- not in help
    //OPWOL,              // -w
    //OPPXE,              // -x
    //OPTForceVendorID,   // -i         -- not in help
    OPTDisableTest,     // -t
    OPTEnableTest,      // -T
    OPTLoop,            // -I
#ifndef EFI
    OPTPrint,           // -p
#endif
    OPTeepPrg,          // -f         -- not in help
    OPTManufLoop,       // -n         -- not in help
    OPTmacPrg,          // -m         -- not in help
    OPTmanufCust,       // -e         -- not in help
#ifndef EFI
    //OPTusedmpi,         // -dmpi
#endif
    //OPTslot,            // -bus xx:xx     
    //OPTVersion,         // -ver                 -- option to check version of software
    //OPTGeneep,          // -geneep    -- not in help -- generate eeprom.bin
    //OPTGeneepkey,       // -geneepkey -- for internal use only
    //OPTDebug,           // -debug     -- for internal use -- jump into cmd mode if test fails
    //OPTChip,            // -chip      -- for internal use only -- for chip test option
    //OPTNoCRC,           // -nocrc     -- for internal use only -- turn off CRC check on incoming packets
    OPTMacCmdLine,      // -mac       -- to program mac address right on command line
    //OPTSkipEECheck,     // -s         -- skip eeprom check/program process
#ifndef EFI
	OPTComPort,             // -com           -- option to enable com port
#endif
    //OPTDelay,                 // -delay         -- for a specific debug purpose only
	//OPTContinueOnError, // -coe             -- Continue test if error occurs
	//OPTHoursLimit,                // -hrs           -- for internal use only -- Set time in hours to run
	//OPTShowRealDevice,  // -srev    -- for internal use only -- show real device revision
    //OPTDetectAll,       // -all       -- for internal use only -- detect all cards instead check for ODI driver
    //OPTVPDWrite,        // -vpdwrite  -- force vpd write test (instead auto detect) operation in vpd test
    //OPTNoInit,          // -noinit    -- do not reset/initialization after detecting the device and jump to prompt right away
    //OPTProgPxe,                       // -ppxe          -- option to program PXE code into the eeprom
	//OPTVerifyPxe,         // -vpxe          -- option to verify PXE code in eeprom
    //OPTModEEpFile,            // -meep          -- for internal use only -- option to modify eeprom binary file
	//OPTNegLinkPol,                // -npol          -- option to use negative Link Polarity
	//OPTViewEEpFile,               // -veep          -- option to view eeprom binary file
	//OPTExtMemSize,                // -ems           -- option to enter external memory test size
    //OPTRef,             // -ref       -- specify reference card slot number
    //OPTFindRef,         // -findref   -- search for reference board and creat file refinfo.txt
    //OPTDevID,           // -devID
    OPTFmac,            // -fmac      -- Read mac address from file
	OPTBrom,			//-b   -- enable bootrom
    not_read
} arg;

typedef struct mainVar {
    int     startcardnum;
    U32     optExt;
    U32     iteration;
    s08    *progName;
    int     force_overwrite_vendor_id;
    U32     testAll;
    u08     optPrgEep;
	u32     cardSelect;
} mainVar;
    

/*********************************************************
 * global variables 
 *********************************************************/
s08 *args[] = { "-c",
		 "-l",
//		 "-z",
		 //"-v",
		 "-r",
		 //"-a",
		 //"-h",

		 "-b44eng",

		 //"-w",
		 //"-x",
		 //"-i",
		 "-t",
		 "-T",
		 "-I",
#ifndef EFI
		 "-p",
#endif
		 "-f",
		 "-n",
		 "-m",
		 "-e",
#ifndef EFI
		 //"-dpmi",
#endif
		 //"-bus",
		 //"-ver",
		 //"-geneep",
		 //"-geneepkey",
		 //"-debug",
		 //"-chip",
		 //"-nocrc",
		 "-mac",
		 //"-s",
#ifndef EFI
		 "-com",
#endif
				 //"-delay",
				 //"-coe",
				 //"-hrs",
				 //"-srev",
		 //"-all",
		 //"-vpdwrite",
		 //"-noinit",
				 //"-ppxe",
				 //"-vpxe",
				 //"-meep",
				 //"-npol",
				 //"-veep",
				 //"-ems",
		 //"-ref",
		 //"-findref",
		 //"-devid",
          "-fmac",
		  "-b",
		  NULL };

static mainVar var = { 
    0,      /* startcardnum */
    1,      /* optExt */
    1,      /* optIteration */
    NULL,   /* progName */
    0,      /* force_overwrite_vendor_id */
    1,      /* test All */
    0,      /* force program eep */
	0,      /* cardSelect */
};

/*************************************************************
 * function init_global_var
 * Initialize global structure variables
 * 
 * input: none
 * 
 * output: none
 *
 * return: none
 *
 * side effects: changes global structure data
 *************************************************************/
static void init_global_var(void)
{
    memset((s08 *) &global, 0, sizeof(global));
    global.verbose = V_CONSOLE | V_ERROR | V_WARNING; 
    global.radix = 16;  
    global.worst_interrupt_latency=0.0;
    global.G_ifile = NULL;
    global.logfile = NULL;
	global.optBrom = FALSE;
    
    global.defLogFileName = "b44diag.log";
    global.defBinFileName = "eeprom.bin";
	global.defTxtFileName = "eeprom.txt";
    strcpy(global.binFileName,global.defBinFileName);
    strcpy(global.txtFileName,global.defTxtFileName);

    global.manuf = 1;
    global.tx_wait = 1;
}

/**************************************************************
 * function next_arg
 * reads a single command line argument and convert it to 'arg'
 *
 * input: values - the argument string from command line
 *
 * output: none
 *
 * return: The converted enum argument value for the input string
 *
 * side effects: none
 *************************************************************/
static arg next_arg (s08** values)
{
    arg p = none;

    if (*values)
    {
	for (p = 0; args[p]; p++)
	{
	    if (!strcmp(args[p], *values))
	    {
		return p+1;
	    }    
	}
	p = not_read;
    }
    return p;
}

/*************************************************************
 *************************************************************/

static void checkOption(s08 * opt)
{
    if (!global.optManufCust)
    {
	printf("ERROR: option -e must be entered before -%s can be used\n",opt);
	util_system_exit(0);
    }
}
/*************************************************************/
static void GetFileName(s08 **ptr, s08 *filename)
{
    if (!sscanf(*ptr, "%s", filename))
	{
	    printf("ERROR: %s filename is not valid\n", filename);
	    util_system_exit(0);
	}
    
    if(isalnum(filename[0])==0)
    {
        printf("\nIllegal filename");
        util_system_exit(0);
    }
}
  
/*************************************************************
 *************************************************************/
/*
static s08 *GetFileName(s08 **ptr)
{
	s08 *filename;

	filename = (BYTE *)malloc(strlen(*ptr));
	if (filename == NULL)
	{
		printf("\nUnable to allocate memory for %s filename", filename);
		util_system_exit(0);
	}

	if (!sscanf(*ptr, "%s", filename))
	{
	    printf("ERROR: %s filename is not valid\n",filename);
	    util_system_exit(0);
	}
	
	return filename;
}
*/
/*************************************************************
 * function parse_args()
 * reads command line arguments, looking for -p, -f, etc   
 * 
 * input: command line string parameter
 *        p - command line argument string
 *
 * output: none
 *
 * return: none
 *
 * side effects: Changes global variable
 *
 *************************************************************/
static void parse_args (s08** p)
{
    arg     a;
    u32     tests;
    s08  tempFileName[MAX_FILE_NAME_SIZE]; 
    var.startcardnum = 0;
    p++;                 /* skip program name */

    while ((a = next_arg(p)))
    {
	p++;             /* skip to next argument */
	switch (a) 
	{
	case OPTBrom:
		if (!global.optCardSelect ||!global.optManufCust)
        {
            printf("ERROR: Option -e and -c must be entered prior to this option\n");
            util_system_exit(0);
        }
		else
		{
			if (!sscanf(*p, "%u", &global.romsize))
			{
				printf("ERROR: Missing input for -b option.\n");
                util_system_exit(0);
			}
			else if (global.romsize > 4) 
			{
				printf("ERROR: -b Option input : 0=Disable, 1=16KB, 2=32KB, 3=64KB, 4=128KB.\n");
                util_system_exit(0);
			}
			else
			{
				global.optBrom = TRUE;
				p++;
			}
		}
		
		break;
	case OPTFmac:
            if (!global.optCardSelect ||!global.optManufCust)
            {
                printf("ERROR: Option -e and -c must be entered prior to this option\n");
                util_system_exit(0);
            }
            else if ((global.optSkipEECheck) || (global.optMacProg) || (global.optMacCmdLine))
            {
                printf("ERROR: Option -fmac does not coexist with option -m, -mac and -s\n");
                util_system_exit(0);
            }
            
            GetFileName(p, global.MacAddrFileName);
            if (!util_fileExists(global.MacAddrFileName))
            {
                printf("Cannot open file %s\n",global.MacAddrFileName);
                util_system_exit(0);
            }
            
            if(util_AccessMacAddrFile(0))
                util_system_exit(0);
                
            global.optMacFile = TRUE;
			global.optMacCmdLine = TRUE;
            p++;
			break;
/*              
		case OPTExtMemSize:
	    if (!sscanf(*p, "%X", &global.ExtMemSize))
		global.ExtMemSize = 0x400000;
	    p++;
	    break; 
*/
/*               
		case OPTViewEEpFile:
	    global.EEpFileName = GetFileName(p);
	    seeprom_cfg(0,3);
			util_system_exit(0); 
*/
/*
		case OPTNegLinkPol:
			global.optNegLinkPol = TRUE;
			break;
*/ 
/*
		case OPTModEEpFile:
	    {
		#define PASSLEN 80
		s08 buff[PASSLEN];
		
c		util_getPass("Broadcom Password?",buff,PASSLEN);
		if (!strcmp(buff,"bcm57swe"))
				{
			global.EEpFileName = GetFileName(p);
			seeprom_cfg(0,2);
				}
		else
					printf("Invalid password\n");
	    } 
			util_system_exit(0);
*/
/*
	 case OPTVerifyPxe:
			global.optVerifyPxe = TRUE;
	    global.PxeFileName = GetFileName(p);
	    
	    p++;
			break;
*/
/*
		case OPTProgPxe:
			checkOption("ppxe");
			global.optProgPxe = TRUE;
	    global.PxeFileName = GetFileName(p);
	    
	    p++;
			break;
*/
/*
		case OPTShowRealDevice:
			global.optShowRealDevice = TRUE;
		util_system_init();
	    util_print_banner();
		util_load_config();
	    util_init_pci_and_find_tigons (var.startcardnum, var.force_overwrite_vendor_id);
	    util_system_exit(0);
*/
/*
		case OPTHoursLimit:
	    if (!sscanf(*p, "%D", &global.RunTimeInHrs))
		global.RunTimeInHrs = TRUE;
	    p++;
	    break; 
*/
/*
		case OPTContinueOnError:
			global.optContinueOnError = TRUE;
			break;
*/
/*
		case OPTDelay:
	    if (!sscanf(*p, "%X", &global.isrDelay))
		global.isrDelay = 0x4000;
	    p++;
	    break;
*/
/*
		case OPTDevID:
	    sscanf(*p, "%x", &global.pciDev);
	    p++;
	    break;
*/
#ifndef EFI
	case OPTComPort:
	    {
		u32 port;
		u32 speed = 115200;
		
		global.Rs232 = TRUE;
		if (!sscanf(*p, "%u", &port))
		{
		    printf("ERROR: no serial port specified\n");
		    util_system_exit(0);
		}
		if (!port || (port > 4))
		{
		    printf("ERROR: com port must be 1..4\n");
		    util_system_exit(0);
		}
		p++;
		if (*p && **p != '-') // must be speed 
		{
		    sscanf(*p, "%u", &speed);
		    if (speed != 9600 && speed!=19200 && speed !=38400 && speed!=57600 && speed != 115200)
		    {
			printf("ERROR: com speed %d is not supported\n",speed);
			util_system_exit(0);
		    }
		    p++;
		}
		com_init(--port,speed);
	    }    
	    break;
#endif
/*
	case OPTSkipEECheck:
	    checkOption("s");            
	    if (global.optMacProg||var.optPrgEep||global.optSkipEECheck)
	    {
		printf("ERROR: Option -s does not coexist with option -m, -f, or -mac\n");
		util_system_exit(0);
	    }
	    
	    else 
			global.optSkipEECheck = TRUE;
	    break; 
*/

	case OPTMacCmdLine:
	    if (!global.optCardSelect||!global.optManufCust)
	    {
		printf("ERROR: Option -e and -c must be entered prior to this option\n");
		util_system_exit(0);
	    }

	    else if ((global.optSkipEECheck) || (global.optMacFile) || (global.optMacProg))
        {
            printf("ERROR: Option -mac does not coexist with option -fmac, -m, and -s\n");
            util_system_exit(0);
		}
	    else if (!sscanf(*p, "%s", global.mac_address))
	    {
		printf("ERROR: MAC address is not valid\n");
		util_system_exit(0);
	    }
	    
	    else
		global.optMacCmdLine = TRUE;
	    
	    p++;
	    break;
			

/*
	case OPTNoCRC:
	    global.NoCRC = TRUE;
	    break;
*/
/*
	case OPTChip:
	    global.ChipTest =TRUE;
	    break;
*/
/*
	case OPTDebug:
	    global.verbose |= V_DEBUG;
	   break;
*/
/*
	case OPTDetectAll:
	    global.detectAll = TRUE;
	    break;
*/
/*
	case OPTVPDWrite:
	    global.VPDWrite = TRUE;
	    break;
*/
/*            
	case OPTNoInit:
	    global.noinit = TRUE;
	    break;    
*/
/*
	case OPTGeneepkey:
	    checkOption("geneepkey");
	    {
		#define PASSLEN 80
		s08 buff[PASSLEN];
		
		util_getPass("Broadcom Password?",buff,PASSLEN);
		if (!strcmp(buff,"bcm57password"))
		    printf("The key for %s is %08X\n",manuf_cust_str[global.optManufCust-1],util_geneepkey());
		else
		    printf("Invalid password\n");
	    } 
	    util_system_exit(0);
*/
/*                      
	case OPTGeneep:
	    checkOption("geneep");
	    util_geneep();
	    util_system_exit(0);
*/
/*
	case OPTVersion:
	    util_system_init();
	    util_print_banner();
	    util_load_config();
	    util_init_pci_and_find_tigons (var.startcardnum, var.force_overwrite_vendor_id);
	    util_system_exit(0);
*/
/*
	case OPTslot:
	    if (!sscanf(*p, "%02X:%02X", &global.test_bus, &global.test_dev) || *p[0]=='-')
	    {
		printf("ERROR: Invalid bus:dev parameter\n");
		util_system_exit(0);
	    }
	    global.test_dev<<=3;
	    p++;
	    break;
*/
/*
	case OPTRef:
	    global.ref = 0xffffffff;
	    break;
*/
/*
	case OPTFindRef:
	    {
		FILE * ref_fn;
		u08 i;
		
		util_init_pci_and_find_tigons (0, 0);
		if (global.device_detected)
		{
		    if ((ref_fn = fopen(REF_FILENAME, "w+")) != NULL)
		    {
			for (i=0;i< global.device_detected; i++)
			{
			    global.current_card = i;
			    fprintf(ref_fn,"%02x %02x\n",CURRCARD.bus_number,CURRCARD.device_and_function);
			}
			printf("%d reference device%s detected\n", global.device_detected, global.device_detected==1?"":"s");
			fclose(ref_fn);
		    }    
		    else
			printf("Cannot open file "REF_FILENAME"\n");
		} 
		else
		    printf("Cannot detect any reference device\n");
		util_system_exit(0);        
	    }    
	    break;
*/          
	case OPTlogfile:
	    util_openLog(*p++);
	    break;
/*
    case OPTeepromfile:
        util_openBin(*p++);
*/
	case OPTManufLoop:
	    checkOption("n");
	    global.optManufLoop = TRUE;
	    break;

			
	case OPTmanufCust:
	    {
		int i = 0;
		while (1)
		{
		    if (manuf_cust_str[i]==NULL)
		    {
				printf("ERROR: Invalid -e parameter entered\n");
				util_system_exit(0);
		    }
		    if (!strcmp(*p,manuf_cust_str[i++]))
		    {
				global.optManufCust = i;
				break;
		    }
		}
	    }    
	    p++;
	    break;

	    
	case OPTmacPrg:
	    checkOption("m");
		if (!global.optCardSelect ||!global.optManufCust)
        {
            printf("ERROR: Option -e and -c must be entered prior to this option\n");
            util_system_exit(0);
        }

        else if ((global.optSkipEECheck) || (global.optMacFile) || (global.optMacCmdLine) )
        {
            printf("ERROR: Option -m does not coexist with option -fmac, -mac and -s\n");
            util_system_exit(0);
		}
	    else  
			global.optMacProg = TRUE;
	    break;
     
#ifndef EFI 
			
/*
	case OPTusedmpi:
	    global.use_dpmi = TRUE;
	    break;
*/
/*
	case OPTPrint:
	    global.optPrint = TRUE;
	    break;
*/
#endif
		
	case OPTeepPrg:         // program eeprom content 
	    checkOption("f");
		
		if (!global.optCardSelect ||!global.optManufCust)
        {
            printf("ERROR: Option -e and -c must be entered prior to this option\n");
            util_system_exit(0);
        }

		GetFileName(p, tempFileName);

        if (util_fileExists(tempFileName))
        {
			var.optPrgEep = TRUE;
			strcpy(global.binFileName,tempFileName);
			p++;
			break;
        }/*
		else if (!util_fileExists(global.binFileName))
        {
			var.optPrgEep = TRUE;
			break;
        }*/
		else 
		{
			printf("Cannot open eeprom file %s \n",  global.binFileName);
			util_system_exit(0);
	    }

	    break;
	case OPTcardnum:
	    if (!sscanf(*p, "%u", &var.startcardnum))
	    {
			printf("ERROR: no card specified, using 0\n");
			var.startcardnum = 0;
	    }
	    else
	    {
			global.optCardSelect = TRUE;
			p++;
	    }    
	    // only test this card instead test all for manufacturing test 
	    var.testAll = 0;              
	    break;

/*
	case OPTverbose:
	    if (!sscanf(*p, "%u", &global.verbose))
	    {
		printf("ERROR: no verbose level specified, using default\n");
	    }
	    else
		p++;
	    break;
*/
	case OPTradix:
	    if (!sscanf(*p, "%u", &global.radix))
	    {
		printf("ERROR: no radix specified, using default (16)\n");
	    } 
	    else if (global.radix && (global.radix < 2 || global.radix > 36))
	    {
		printf("ERROR: radix must be 2-36 or 0, using default (16)\n");
	    }
	    p++;
	    break;
/*
	case OPTVideoMode:
	    global.video_mode = TRUE;
	    break;
	
*/
/*
	case OPTForceVendorID:
	    var.force_overwrite_vendor_id = TRUE;
	    break; 
*/
/*
	case OPTMemBaseAddr:
	    if (!sscanf(*p, "%lx", &global.mem_base_addr))
	    {
		printf("ERROR: Memory Base address is not valid\n");
		global.mem_base_addr = 0;
	    }
	    p++;
	    break;
*/
	case OPManuf:
	    global.manuf = 0;
	    break;
/*              
	case OPWOL:
	    global.optUpdateWol = TRUE;
	    if (!sscanf(*p, "%u", &global.optWOL))
	    {
		printf("ERROR: No WOL value specified\n");
		util_system_exit(0);
	    }
	    p++;
	    break; 
*/
/*
	case OPPXE:
	    global.optUpdatePxe = TRUE;
	    if (!sscanf(*p, "%u", &global.optPXE))
	    {
		printf("ERROR: No PXE value specified\n");
		util_system_exit(0);
	    }
	    p++;
	    break;
*/
			
	case OPTDisableTest:
	    tests = test_checkTestStr(0,*p++);
	    break; 


	case OPTEnableTest:
	    tests = test_checkTestStr(1,*p++);
	    break;



	case OPTLoop:
	    if (!sscanf(*p, "%u", &var.iteration))
	    {
		printf("ERROR: Iteration number is not valid\n");
		var.iteration = TRUE;
	    }
	    p++;
	    break;


	default:
	    printf("usage: %s [options]\n",var.progName);
	    printf("       -c <num>    : Specify the card to be tested\n");
	    //printf("       -h          : High Resolution (80x50) Video Mode\n");
	    printf("       -I <num>    : Iteration number\n");
	    printf("       -l <file>   : Log file\n");
//	    printf("       -z <file>   : Eeprom Contents file\n");
#ifndef EFI
	    //printf("       -p          : Print on error\n");
#endif
	    printf("       -t <id>     : Disable test, example: -t A13\n");
	    printf("       -T <id>     : Enable test, example: -T B143\n");
	    //printf("       -w <value>  : Enable/Disable (value = 1/0) WOL in manufacture mode\n");
	    //printf("       -x <value>  : Enable/Disable (value = 1/0) PXE in manufacture mode\n");
	    //printf("       -bus bb:dd  : Specify with UUT to test\n");
	    //printf("       -coe        : Continue on error\n");
#ifndef EFI
	    printf("       -com <value>: enable com port, value(1..4) (Internal use only!)\n");
	    //printf("       -dpmi       : Use dpmi method for memory allocation\n");
#endif
	    //printf("       -ems <size> : Enter external memory size in HEX to test\n");
	    //printf("       -npol       : Select Negative Link Polarity in TBI test\n");
	    //printf("       -srev       : Silicon revision\n");
	    //printf("       -ver        : Version of the current test software/eeprom.bin\n");
	    //printf("       -ref        : Run test with reference device\n");
	    //printf("       -findref    : Detect reference device\n");
//            printf("       -a <num>    : Memory base address\n");
	    printf("       -e          : Run program in engineering mode\n");
        printf("       -f          : Program Eeprom specified Eeprom File before test\n");
//            printf("       -i          : Force overwrite vendor ID\n");
	    printf("       -m          : Program mac address\n");
		printf("       -n          : Run in manufacture loop\n");
		printf("       -r <num>    : Input radix\n");
//            printf("       -s          : Skip EEPROM checking/programming process\n");
//            printf("       -v <num>    : Verbose level\n");
//            printf("       -chip       : For chip test option only\n");
//            printf("       -debug      : Entering debug mode which provides debug messages\n");
        printf("       -mac <addr> : Program MAC address from command line\n");
//                        printf("		 -meep <file>: Modify specified eeprom binary file\n");
//            printf("       -nocrc      : Force no CRC check mode for debugging purpose\n");
//            printf("       -ppxe       : Programming PXE from file\n");
//            printf("       -veep       : View specified eeprom binary file\n");
//            printf("       -vpxe       : Verify PXE content from referenced file\n");
        printf("       -fmac       : Option to input MAC addresses from a file\n");
		printf("       -b          : Option to enable/disable Boot Rom size\n");
	    util_system_exit(0);
	}
    }
}

/*************************************************************
 * function main()
 * 
 * input: command line parameter
 *        argc  - number of argument
 *        argv  - argument string list
 *
 * output: none
 *
 * return: none
 *
 * side effects: none
 *
 *************************************************************/
void main (int argc, s08** argv)
{
    int errorCode=NO_ERROR;

    var.progName = argv[0]; /* program name */
    /* parse arguement line, check PCI bus, and print banner */
    init_global_var();
    util_init_default_cfg();
    parse_args (argv);  //austin need to look into this
    util_system_init(); //setup screen
    util_print_banner();  
    //util_load_config();  //read miireg and pci reg from file to array
    init_break();
    util_init_pci_and_find_tigons (var.startcardnum, var.force_overwrite_vendor_id);

    if (global.RunTimeInHrs)
		time (&global.start_time);
     
    if (global.manuf)
    {
	if ((global.test_bus || global.test_dev) && var.testAll)
	{
	    u32 dev;
	    
	    set_current_test(-2,0);
	    printf("\nBus verification");
	    test_showDot();
	    for (dev=0;dev<get_device_detected();dev++)
	    {
		util_switch_card(dev);
		if (CURRCARD.bus_number==global.test_bus && CURRCARD.device_and_function == global.test_dev)
		{
		    var.startcardnum = dev;
		    var.testAll = 0;
		    break;
		}
	    }
	    if (var.testAll)
	    {
		test_failed2(ERR_BUS,global.test_bus, global.test_dev >> 3);
		util_system_exit (ERR_BUS);                    
	    }
	    test_passed();    
	}
	errorCode = test_doAllTests(var.iteration, 
			var.testAll ? (~global.ref & 0xffffffff) : (1<< get_currcard_index()), 
			global.optUpdateWol||global.optUpdatePxe, var.optPrgEep);
    }
    else
    {
#ifndef WIN_CLI        
	cmd_line_interface(NULL);
#else      
		//not used
	util_open_device_windows();
	win_main();
#endif        
    }
    util_system_exit (errorCode);                    /* done */
}

/* end of main.c */


