/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_tests.c 14    7/19/02 10:07a Ahui $
 *
 *  Description: This file contains each individual tests
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_tests.c $
 * 
 * 14    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 13    7/11/02 1:02p Ahui
 * Clean up unused code.
 * 
 * 12    7/08/02 12:22p Ahui
 * Modify code to support new card srom and mac address progarmming.
 * 
 * 11    6/13/02 10:01a Ahui
 * skip reset and modified display message for fmac option error handling
 * 
 * 10    6/11/02 11:52a Ahui
 * Added test_pasted0 function.
 * Added error code to support fmac option.
 * 
 * 9     5/31/02 10:39a Ahui
 * 
 * 8     5/29/02 6:29p Ahui
 * 1. Added Error codes descriptions. (t3_tests.c)
 * 2. In manufacturing mode, external loopback test has been separated to
 * 2 tests, C3 and C4.  C3 is 100BT and C4 is 10BT. (t3_tests.c)
 * 3. Mod restorePCIConfig() to support manufacturing loop (-n) option.
 * (t3_test.c)
 * 4. Fixed bug in continuously prompting for mac address. (t3_test.c)
 * 
 * 
 * 7     5/16/02 5:59p Mnima
 * use input eeprom name
 * 
 * 6     4/25/02 11:04a Ahui
 * Enabled C3 external tests.
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *****************************************************************/

#ifndef EFI
#include <io.h>
#include <conio.h>
#endif
#include <stdarg.h>
#include <time.h>
#include <ctype.h>

#include "general.h"
#include "t3_gdata.h"
#include "t3_tests.h"
#include "t3_util.h"
#include "t3_diag.h"
#include "t_io.h"
#include "t3_win.h"
#include "t3_mii.h"
#include "t3_eprom.h"
#include "t3_eecfg.h"
#include "pci.h"
#include "t3_cli.h"
#include "t3_util.h"
#include "Cl4xsb.h"


typedef enum {
    TEST_ENABLED = 0x000000001,
    TEST_FAILED  = 0x000000002,
} TEST_FLAGS;

typedef struct {
    int  flags;
    s08 * name;
    int (* func)(void *);
} test_t;

typedef struct {
    s08 * name;
    u32    test_count;
    test_t * subTest;
} test_group_t;

static test_t reg_tests[] = {
    {TEST_ENABLED,"Indir Ctrl Register", indi_reg_test},
    {TEST_ENABLED,"Control Register",  ctrl_reg_test},
    {TEST_ENABLED,"Interrupt",         interrupt_test},
    {TEST_ENABLED,"Built In Self",     diag_bist_test},
};

static test_t misc_tests[] = {
	{TEST_ENABLED,"LED",			   LED_test},
    {TEST_ENABLED,"EEPROM",            eeprom_rw_test},
	{TEST_ENABLED,"MII",               MII_test},
	{TEST_ENABLED,"LINK Status",       Link_test},
    //{			0,"DMA",               DMA_test},
    //{			0,"VPD",               VPD_test},
    //{			0,"ASF",               ASF_test},
    //{			0,"Expansion ROM",     ROM_test},
	
};


static test_t data_tests[] = {
    {TEST_ENABLED,"MAC Loopback", 			IntLoop_test},
    {TEST_ENABLED,"PHY Loopback", 			PhyLoop_test},
    {0,"External Loopback 100BT",     	External_Loop_test_100},
	{0,"External Loopback 10BT",     	External_Loop_test_10},
    //{			0,"MSI",                    MSI_test},
};

static test_group_t tests[] = {
    {"Register", MAX_REG_TEST, reg_tests}, 
    {"Miscellaneous", MAX_MISC_TEST,misc_tests},
    {"Data", MAX_DATA_TEST,data_tests},
	//{"Memory", MAX_MEM_TEST, mem_tests},
};

static int _current_grp,_current_id, _failed;
static u32 iteration;
static s08 returnStr[90];
static s08 * errorMsg[] = {
/* NO_ERROR                   0 */ "PASS",
/* ERR_IND_REG_ERR            1 */ "Got 0x%08X @ 0x%08X. Expected 0x%08X",
/* ERR_CHIP_RUNNING           2 */ "Cannot perform task while chip is running",
/* ERR_BAD_NIC                3 */ "Invalid NIC device",
/* ERR_READ_ONLY_CLEAR        4 */ "Read only bit %s got changed after writing zero at offset 0x%X",
/* ERR_READ_ONLY_SET          5 */ "Read only bit %s got changed after writing one at offset 0x%X",
/* ERR_READ_WRITE_NOT_CLEAR   6 */ "Read/Write bit %s did not get cleared after writing zero at offset 0x%X",
/* ERR_READ_WRITE_NOT_SET     7 */ "Read/Write bit %s did not get set after writing one at offset 0x%X",
/* ERR_BIST                   8 */ "BIST failed",
/* ERR_INTERRUPT              9 */ "Could not generate interrupt",
/* ERR_ABORT                 10 */ "Aborted by user",
/* ERR_DMA_TXDATA            11 */ "Tx DMA:Got 0x%08X @ 0x%08X. Expected 0x%08X",    
/* ERR_DMA_RXDATA            12 */ "Rx DMA:Got 0x%08X @ 0x%08X. Expected 0x%08X",    
/* ERR_TXDMA                 13 */ "Tx DMA failed",
/* ERR_RXDMA                 14 */ "Rx DMA failed",
/* ERR_MEM                   15 */ "Data error, got 0x%08X at 0x%08X, expected 0x%08X",
/* ERR_MEM2                  16 */ "Second read error, got 0x%08X at 0x%08X, expected 0x%08X",
/* ERR_EEP_WRITE             17 */ "Failed writing EEPROM at 0x%04X",
/* ERR_EEP_READ              18 */ "Failed reading EEPROM at 0x%04X",
/* ERR_EEP_DATA              19 */ "EEPROM data error, got 0x08X at 0x04X, expected 0x%08X",
/* ERR_FILE_OPEN             20 */ "Cannot open file %s",
/* ERR_LOOPBACK_TYPE         21 */ "Loopback type not supported",
/* ERR_IMAGE_SIZE            22 */ "Invalid CPU image size %d",
/* ERR_MALLOC                23 */ "Cannot allocate memory",
/* ERR_CPU_RESET             24 */ "Cannot reset CPU",    
/* ERR_CPU_RELEASE           25 */ "Cannot release CPU", 
/* ERR_CPU_TEST              26 */ "%cx CPU test failed",       
/* ERR_DMA_RANGE             27 */ "Invalid Test Address Range\nValid NIC address is 0x%08X-0x%08X and exclude 0x%08X-0x%08X",
/* ERR_DMA_DATA              28 */ "DMA:Got 0x%08X @ 0x%08X. Expected 0x%08X", 
/* ERR_RX_LEN                29 */ "Invalid RX packet length",
/* ERR_PHY_TOO_MANY_REG      30 */ "Too many registers specified in the file, max is %d",
/* ERR_VPD_WRITE             31 */ "Cannot write to VPD address %04X",
/* ERR_VPD_DATA              32 */ "VPD data error, got %08X @ 0x%04X, expected %08X",
/* ERR_NO_LINK               33 */ "No good link! Check Loopback plug",
/* ERR_DATA_TX               34 */ "Cannot TX Packet!", 
/* ERR_DATA_TX_MISSING       35 */ "Requested to Tx %d. Only %d is transmitted",       
/* ERR_DATA_RX_MISSING       36 */ "Expected %d packets. Only %d good packet(s) have been received\n%d unknown packets have been received.\n%d bad packets have been received.",
/* ERR_INVALID_TEST          37 */ "%c%d is an invalid Test",
/* ERR_EEPROM_CHECKSUM       38 */ "EEPROM checksum error",
/* ERR_READING_WOL_PXE       39 */ "Error in reading WOL/PXE",
/* ERR_READING_WOL_PXE       40 */ "Error in writing WOL/PXE",
/* ERR_NO_EXT_SRAM           41 */ "No external memory detected",
/* ERR_DMA_LEN               42 */ "DMA buffer %04X is too large, size must be less than %04X",  
/* ERR_FILE_TOO_BIG          43 */ "File size %d is too big, max is %d",
/* ERR_INVALID               44 */ "Invalid %s",
/* ERR_WRITE                 45 */ "Failed writing 0x%x to 0x%x",
/* CMD_QUIT                  46 */ "",
/* ERR_CPU_MEM_ERR           47 */ "%s CPU access error @ %08X, expected %08X but got %08X",
/* ERR_COMMAND               48 */ "Unknown command",
/* ERR_OPTION                49 */ "Invalid option",
/* ERR_CHIP_NOT_RUNNING      50 */ "Cannot perform task while chip is not running. (need driver)",
/* ERR_NO_REG_DEF            51 */ "Cannot open register define file or content is bad",
/* ERR_ASF_RST               52 */ "ASF Reset bit did not self-cleared",
/* ERR_ASF_ATTN_LOC          53 */ "ATTN_LOC %d cannot be mapped to %cX CPU event bit %d",
/* ERR_ASF_RST_VAL           54 */ "%s Regsiter is not cleared to zero after reset",
/* ERR_ASF_PA_TIMER          55 */ "Cannot start poll_ASF Timer",
/* ERR_ASF_PA_CLEAR          56 */ "poll_ASF bit did not get reset after acknowleged",
/* ERR_ADF_NO_STAMP          57 */ "Timestamp Counter is not counding",
/* ERR_ADF_NO_TIMER          58 */ "%s Timer is not working",
/* ERR_ASF_EVENT             59 */ "Cannot clear bit %s in %cX CPU event register",
/* ERR_EEP_FILESIZE          60 */ "Invalid "EEPROM_FILENAME" file size, expected %d but only can read %d bytes",
/* ERR_MAGIC_VALUE           61 */ "Invalid magic value in %s, expected %08x but found %08x",
/* ERR_EEP_FMT               62 */ "Invalid manufature revision, expected %c but found %c",
/* ERR_EEP_BOOTVER           63 */ "Invalid Boot Code revision, expected %d.%d but found %d.%d",
/* ERR_EEP_CANNOT_WRITE      64 */ "Cannot write to EEPROM",
/* ERR_EEP_CANNOT_READ       65 */ "Cannot read from EEPROM",
/* ERR_BAD_CHECKSUM          66 */ "Invalid Checksum",
/* ERR_BAD_MAGIC_VALUE       67 */ "Invalid Magic Value",
/* ERR_MAC                   68 */ "Invalid MAC address, expected %02X-%02X-%02X-%02X-%02X-%02X",
/* ERR_BUS                   69 */ "Slot error, expected an UUT to be found at location %02X:%02X:00",
/* ERR_SPEC_MEM              70 */ "Adjacent memory has been corrupted while testing block 0x%08x-0x%08x\nGot 0x%08x @ address 0x%08x. Expected 0x%08x",
/* ERR_NOT_SUPPORT           71 */ "The function is not Supported in this chip",
/* ERR_BAD_CRC               72 */ "Packets received with CRC error",
/* ERR_MII_ERR_BITS_SET      73 */ "MII error bits set: %04x",
/* ERR_INIT_MAC              74 */ "CPU does not initialize MAC address register correctly",
/* ERR_FW_FILE_FORMAT        75 */ "Invalid firmware file format",
/* ERR_RESET_TX_CPU          76 */ "Resetting TX CPU Failed",
/* ERR_RESET_RX_CPU          77 */ "Resetting RX CPU Failed",
/* ERR_INVALID_MAC_ADDR      78 */ "Invalid MAC address",
/* ERR_MAC_REG               79 */ "Mac address registers are not initialized correctly",
/* ERR_BOOTCHECKSUM          80 */ "EEPROM Bootstrap checksum error",
/* ERR_VPD_READONLY          81 */ "Write operation changed VPD read only area data at %04X",
/* ERR_VPD_READ              82 */ "Cannot read data from VPD address %04X",
/* ERR_MEM_READ              83 */ "Memory read and compare error",
/* ERR_MEM_WRITE             84 */ "Memory write error",
/* ERR_PXE_PGM               85 */ "PXE Programming Error",
/* ERR_PXE_VFY               86 */ "PXE Verification Error",
/* ERR_EXT_MEM_EXE_TIMOUT    87 */ "Cannot execute code from external memory, pc=%08X",
/* ERR_EXT_MEM_SIZE          88 */ "External memory size detection error",
/* ERR_RESET_TIMEOUT         89 */ "Reset Time",
/* ERR_MSI_ERR_NOTCLEAR      90 */ "MSI Error bits are not cleared after reset",
/* ERR_MSI_DATA              91 */ "MSI expected %04X, but read %04X at %08X",
/* ERR_MEM_INIT              92 */ "mem pool initialization failed",
/* ERR_MEM_UNINIT            93 */ "mem pool un-initialization failed",
/* ERR_PCI_REGS_WIDTH        94 */ "Read/Write PCI regs width %d affects wider than expected at offset 0x%X",
/* ERR_LINK_STATUS           95 */ "Link status error in auto-polling mode",
/* ERR_PHY_INTERRUPT         96 */ "Phy interrupt did not happen",
/* ERR_EEP_BIT_BANG          97 */ "EEprom test fails in bit-bang mode at address %X",
/* ERR_ROM_SIZE              98 */ "ROM size error\nExpected %08X but read %08X at\nROM Bar (0x30) register with %d written to ROM size reg.(0x88)",
/* ERR_ROM_DATA              99 */ "Data Error\nExpected %08X but read %08X at %08X",
/* ERR_ROM_ENABLE           100 */ "Expansion ROM Desired bit is not set after loading firmware",
/* ERR_GPIO                 101 */ "GPIO%d Error, write=%d, read=%d",
/* ERR_GPIO5704             102 */ "Dev:%d Expected GPIO 0/1/2 = %d/%d/%d, but read as %d/%d/%d",
/* ERR_BIST_NOT_DONE        103 */ "Bist test did not complete internally",
/* ERR_BIST_DATA_MISCMP     104 */ "Bist data miscompared at bit: %d out: %d exp: %d",
/* ERR_CPU_NO_RESPONSE      105 */ "No Response from firmware",
/* ERR_CPU_ERR_CODE         106 */ "%s CPU returned result %d, key = %d",
/* ERR_RX_CORR				107 */ "Corrupted RX packet contents", 
/* ERR_BAD_CPU_CFG			108 */ "Bad CPU Configuration: %s",
/* ERR_BIST_SBTMSTATE       109 */ "Sbtmstatelow failed readback",
/* ERR_RX_HEADER            110 */ "Rx packet header error",
/* ERR_RX_CONT              111 */ "Corrupt Rx packet contents",
/* ERR_RX_INSTATUS          112 */ "Error on receive interrupt status",
/* ERR_EEP_CRC				113 */ "EEprom CRC Error",
/* ERR_MAC_ADDR_ENDED       114 */ "Runs out of Mac Address",
/* ERR_ILLEGAL_MAC_ADDR     115 */ "Illegal Mac Address"
};



/*****************************************************************
 * private functions 
 *****************************************************************/

/*************************************************
 * function test_grp
 *   do a set of tests on a group
 *
 * input: grp - the test group number
 *
 * output: none
 *
 * return: test result
 *
 *************************************************/
static int test_grp(int grp)
{
    int id;
    int retc = NO_ERROR; 
    
    for (id=0;(id < tests[grp].test_count) && (retc==NO_ERROR);id++)
    {
        if (IS_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED))
        {
            retc=do_test(1,grp,id,NULL,0);
        }    
    }
    return retc;
}



/*************************************************
 * function do_device_tests
 *   do a set of tests on current active device
 *
 * input: none
 *
 * output: none
 *
 * return: test result
 *
 *************************************************/
static int do_device_tests()
{
    int grp, retc=NO_ERROR;
    
    for (grp=0; (grp < MAX_TEST_GROUP) && (retc == NO_ERROR); grp++)
    {
        if (test_selectedGrpTests(grp)) /* if any device is selected */
        {
            _settextcolor (0xf);
            printf("\nGroup %c. %s Tests",'A'+grp,tests[grp].name);
            _settextcolor (GRAY);
            
            
            retc=test_grp(grp);
        }
    }

    
    return retc;    
}

/************************************************************************
 * function do_test
 *   performs the test
 *
 * input: grp -- test group
 *        id - test id
 *        para -- test parameter
 *        abs -- if true, always run test regardless of enable status
 *
 * output: none
 *
 * return: test result
 *
 * side effect: Once this function is called, it set "tested" flag
 *              internally to indicate this test has been run before
 *
 ************************************************************************/
static int do_test(u32 req_iteration, int grp, int id, void * para, int abs)
{
    int errorCode = NO_ERROR;
    u32 local_iteration = 0;

    do 
    {
        showIteration(++local_iteration, req_iteration);
        if (grp < MAX_TEST_GROUP)
            if (id < tests[grp].test_count)
            {
                if (IS_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED) || abs)
                { 
                    check_abort();
//                    SET_FLAG(tests[grp].subTest[id].flags,TEST_TESTED);
                    printf("\n  %c%d. %s Test",'A'+grp,id+1,tests[grp].subTest[id].name);
                    util_DbgPrintf("(%d)", iteration);
                    set_current_test(grp,id);
                	errorCode = tests[grp].subTest[id].func(para);
                    continue;
                }
            }
        set_current_test(0,0);
        errorCode = test_failed2(ERR_INVALID_TEST,'A'+grp,++id);
    } while ((errorCode == NO_ERROR) && (!req_iteration || local_iteration < req_iteration));
    return errorCode;
}


            

/*************************************************
 * function showMac                              *
 *************************************************/
static showMac()
{
    u08 mac[8];
    eprom_read_blk(76,(u32 *)&mac,2);
	printf("\nMac Address");
	test_showDot();
	printf("%02X-%02X-%02X-%02X-%02X-%02X",mac[3], mac[2], mac[5], mac[4], mac[7], mac[6]);
}                      


/*************************************************
 * function checkEepromContent                   *
 *************************************************/

static int checkEepromContent(u32 update_WOL_PXE, int loadFromFile)
{
  
    int progMac = global.optMacProg;
	int cmdLineMac = global.optMacCmdLine;
    int cmdFileMac = global.optMacFile;
	int cmdBrom = global.optBrom;

	u08 * buffer = NULL;
    u32 len = 0;
    int retc;
    s08 * fn;

	fn= global.binFileName;
	
	if (loadFromFile)
	{
		// load eeprom content from file 
        printf("Loading EEPROM contents.....\n");
		
		if ((retc=util_readFileMemAlloc(fn, 0, len, 256, 
			   &len, &buffer, 1))==NO_ERROR)
		{
		eecfg_write(0, buffer,0, len);
		free(buffer);
		}
	}
	
	if (progMac || cmdLineMac || cmdBrom)
	{
		printf("\nProgramming MAC Addres\n");
		while (!util_readMacAddress())
        {
            if (scrn_yes("The MAC Address must be entered, do you want to quit?"))
                return test_failed0(ERR_ABORT);
        }
		if (cmdFileMac)
		{
			if(util_AccessMacAddrFile(1)) return ERR_FILE_OPEN;
		}
	}

	
    return NO_ERROR;
}

/*************************************************
 *************************************************/
static int restorePCIConfig(void)
{
    int dev;
    u16 vendor_id;
    u16 device_id;
    card_t *card;
    
    for (dev=0; dev<get_device_detected(); dev++)
    {
        util_switch_card(dev);
        card = get_currcard();
        
        read_configuration_word(card->bus_number,
                          card->device_and_function,
                          PCI_CS_VENDOR_ID,
                          &vendor_id);
        read_configuration_word(card->bus_number,
                          card->device_and_function,
                          PCI_CS_DEVICE_ID,
                          &device_id);

        if ((vendor_id == 0xffff) || (device_id == 0xffff))
        {
            scrn_dispWait("Cannot detect UUT for device %d",dev);
            return 0;
        }

        write_configuration_dword(card->bus_number,
                            card->device_and_function,
                            card->base_address_offset,
                            card->base_address);
        write_configuration_dword(card->bus_number,
                            card->device_and_function,
                            card->base_address_offset+4,
                            0);
        /* Enable Memory space and bus master */
        write_configuration_word(card->bus_number,
                            card->device_and_function,
                            PCI_CS_COMMAND,
                            0x6);
        write_configuration_byte(card->bus_number,
                            card->device_and_function,
                            PCI_CS_INTERRUPT_LINE,
                            card->interrupt_line);
        util_hw_init(1);
    }
    return 1;
}

/*************************************************
 * function powerOnOff
 *   prompt user to power off and on the UUT. Restore PCI configuration registers
 *
 * return: true if user wants to quit program 
 *************************************************/
static int powerOnOff()
{
    while (1)
    {
        if (scrn_dispWait("Please exchange UUT and press any key to continue... (ESC to quit)")==KEY_ESC)
        {
            if (scrn_yes("Quit program?"))
                return 1;
        }
        else
        {
            if (restorePCIConfig())
            {
                win_clearscreen();
                return 0;
            }
        }
    }
}

/*************************************************
 * function startAllTests
 *   performs all tests
 *
 * input: req_iteration -- requested iteration
 *        devMap -- it passes the device need to be tested. This passes in
 *                  bit map. If bit0 is set, it indicates device 0 is selected
 * output: none
 *
 * return: test result
 *************************************************/
static int startAllTests(u32 req_iteration, u32 devMap, u32 update_WOL_PXE, int eepProg)
{
    int errorCode = NO_ERROR;

    U32 testMap;
	u32 temp;
    //u16 phyreg, ophyreg;
    time_t stop_time;
    
    iteration = 0;
    do 
    {
        if (global.optManufLoop && iteration)
            if (powerOnOff())
                break;
        
        showIteration(++iteration, req_iteration);
        
        /* start to test from device 0 */
        util_switch_card(0);
        testMap = devMap;
        errorCode = NO_ERROR;
        while (testMap && ((errorCode==NO_ERROR) ||	\
              (global.optManufLoop && (errorCode != ERR_ABORT))))
        {
            /* check if we need to do test on this device */
            if (testMap & 1)
            {
                
                
                if (get_device_detected() > 1)
                {
                    _settextcolor (HELP_COLOR);
                    printf("\nTesting Device:\n");
                    _settextcolor (GRAY);
                    util_show_device(get_currcard_index());
                }
                printf("\n");
                global.optSkipEECheck=0; //dummy
                if ((iteration==1 || global.optManufLoop) && !global.optSkipEECheck)
				{
					temp=get_currcard_index();
                	errorCode = checkEepromContent(update_WOL_PXE, eepProg);
					//since checkEepromContent will set currcard_index = 0; I have to reset it back to the real currcard_index
					util_switch_card(temp);

				}
						
                
                showMac();
                errorCode = do_device_tests();
                
                printf("\n");
                
            }
            
            /* end of device? */
            if ((get_currcard_index()+1 == get_device_detected()) || util_user_abort())
                break;

            /* next device */
            util_switch_card(get_currcard_index()+1);
            testMap >>= 1;
        }
        
    	/* Continue on error and reset iteration if option -coe entered */
    	if ((errorCode != NO_ERROR) && (errorCode != ERR_ABORT) && global.optContinueOnError)
		{
			iteration = 0;
			errorCode = NO_ERROR;
		}

        time(&stop_time);
        if (global.RunTimeInHrs)
            if (difftime (stop_time, global.start_time) > global.RunTimeInHrs * 60 * 60)
                break;
        
    } while (global.optManufLoop || 
    		((!req_iteration || (iteration < req_iteration)) && (errorCode==NO_ERROR)));
    
    return errorCode;             
}        

/*****************************************************************
 *****************************************************************
   public functions 
 ***************************************************************** 
 *****************************************************************/ 
 
void startTest(void)
{
    _failed = 0;
}

/*************************************************
 * function test_setGrpTests
 *   Use the parameter as is to disable or enable the tests
 *
 * input: grp, id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
void test_setGrpTests(int grp, u32 tests)
{
    test_disableGrpTests(grp,0xffffffff);
    test_enableGrpTests(grp,tests);
}

/*****************************************************************
 * test_selectedGrpTests(int grp)
 *   returns the selected test bit map
 *   
 *****************************************************************/
u32 test_selectedGrpTests(int grp)
{
    int id;
    u32 sel = 0;
    
    for (id=0; id<tests[grp].test_count; id++)
        if (IS_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED))
            sel |= (1 << id);
    return sel;
}

/*************************************************
 * test_bitString 
 *    convert bits into a string
 *
 * side effect: Since this routine uses static buffer to return the string, 
 *              it cannot be called in one single statement:
 *       
 *              printf("the bit string is %s and %s",bitSTring(1), test_bitString(0xff)); -- won't work
 *
 *              should be changed to:
 *
 *              printf("the bit string is %s", test_bitString(1));        -- Good 
 *              printf(" and %s",test_bitString(0xff));
 *************************************************/
const s08 * test_bitString(u32 data32)
{
	s08 number[10];
	int bitPos = 0;

	returnStr[0] = '\0'; // empty string 
	while (data32) 
	{
		if (data32 & 1)
		{
			if (strlen(returnStr)) 
				strcat(returnStr,",");
			sprintf(number,"%d",bitPos);
			strcat(returnStr,number);
		}
		bitPos++;
		data32 >>= 1;
	}
	return returnStr;
}

/*************************************************
 * Show current test setting's dots
 *************************************************/
void test_showDot(void)
{
    int row, col;
    
    win_wherexy(&row,&col);
    for (;col < DOT_POSITION; col++)
        printf(".");
    printf(": ");
    startTest();
}

/*************************************************
 * function test_failed
 *   performs all tests
 *
 * input: errCode - error code
 *        para list - arguments
 *
 * output: none
 *
 * return: error code
 *
 *************************************************/
#define ERROR_R1 3
#define ERROR_C1 10
#define ERROR_HIGH 16
#define ERROR_WIDTH 60

int test_failed(int errCode, int p1, int p2, int p3, int p4, int p5, int p6, int p7)
{
    time_t t = time(NULL);
    s08 time_buffer[40];
    u16 ch;
    pWin w;
    pScreen s;
    
    
	if ((errCode!=ERR_MAC_ADDR_ENDED) && (errCode!=ERR_INVALID_MAC_ADDR) && (errCode!=ERR_FILE_OPEN)) 
		reset();    
    _failed = 1;
    strcpy(time_buffer,ctime(&t));
    for (ch=0;ch<40;ch++)
        if (time_buffer[ch]=='\n')
            time_buffer[ch]='\0';
    ch = 0;        
    util_beep();
    
    /* output to regular screen & output file */
    _settextcolor (RED);
    eprintf("Failed\n");
    _settextcolor (GRAY);
    if (global.manuf)
    {
        eprintf("\nFailed!");
        if (_current_grp!=-2)
            eprintf(" Error detected at device %d (PCI slot %02X:%02X:%X)",
                get_currcard_index(),
                CURRCARD.bus_number, 
                CURRCARD.device_and_function >> 3,
                CURRCARD.device_and_function & 0x7);
        if (iteration>1 && !global.optManufLoop)
            eprintf(", iteration %d",iteration);                
    

        eprintf("\nOS: DOS");
        eprintf("\nSource: B44DIAG");
        eprintf("\nRevision: %s",versionStr);
        eprintf("\nUnit Under Test: NIC");
        
        if (_current_grp==-2)
        {
            eprintf("\nModule: NIC test");
            eprintf("\nTest Name: Miscellaneous");
            eprintf("\nTest #: A0");
        }
        else if (_current_grp==-1)
        {
            eprintf("\nModule: EEPROM content test");
            eprintf("\nTest Name: EEPROM Content Check");
            eprintf("\nTest #: A0");
        }
		else if ((errCode==ERR_MAC_ADDR_ENDED) || (errCode==ERR_INVALID_MAC_ADDR) || (errCode==ERR_FILE_OPEN))
        {
			eprintf("\nModule: N/A");
            eprintf("\nTest Name: N/A");
            eprintf("\nTest #: N/A");
		}
		else
        {
            eprintf("\nModule: %s Test",tests[_current_grp].name);
            eprintf("\nTest Name: %s Test",tests[_current_grp].subTest[_current_id].name);
            eprintf("\nTest #: %c%d",'A'+_current_grp,_current_id+1);
        }    
        eprintf("\nError #: %d",errCode);
        eprintf("\nTime Stamp: %s",time_buffer);
        eprintf("\nMsg: ");
        eprintf(errorMsg[errCode],p1,p2,p3,p4,p5,p6);

        if (global.optPrint)
        {
            /* show on dialog box */
            w = win_newBasic(ERROR_R1,ERROR_C1,ERROR_HIGH,ERROR_WIDTH,ATTRIB(WHITE,RED));
            s = &w->scrn;

            scrn_xyPrintf(s,1,1,"OS: DOS");


            scrn_xyPrintf(s,2,1,"Source: B44DIAG");

            scrn_xyPrintf(s,3,1,"Revision: %s",versionStr);
            scrn_xyPrintf(s,4,1,"Unit Under Test: NIC");
            if (_current_grp==-2)
            {
                eprintf("\nModule: NIC test");
                eprintf("\nTest Name: Miscellaneous");
                eprintf("\nTest #: A0");
                scrn_xyPrintf(s,5,1,"Module: NIC test");
                scrn_xyPrintf(s,6,1,"Test Name: Miscellaneous");
                scrn_xyPrintf(s,7,1,"Test #: A0");
            }
            else if (_current_grp==-1)
            {
                eprintf("\nModule: EEPROM content test");
                eprintf("\nTest Name: EEPROM Content Check");
                eprintf("\nTest #: A0");
                scrn_xyPrintf(s,5,1,"Module: EEPROM content test");
                scrn_xyPrintf(s,6,1,"Test Name: EEPROM Content Check");
                scrn_xyPrintf(s,7,1,"Test #: A0");
            }
            else
            {
                scrn_xyPrintf(s,5,1,"Module: %s Test",tests[_current_grp].name);
                scrn_xyPrintf(s,6,1,"Test Name: %s Test",tests[_current_grp].subTest[_current_id].name);
                scrn_xyPrintf(s,7,1,"Test #: %c%d",'A'+_current_grp,_current_id+1);
            }    
            scrn_xyPrintf(s,8,1,"Error #: %d",errCode);
            scrn_xyPrintf(s,9,1,"Time Stamp: %s",time_buffer);
			scrn_xyPrintf(s,10,1,"Iteration: ",iteration);
            scrn_xyPrintf(s,11,1,"Msg: ");
            scrn_printf(s,errorMsg[errCode],p1,p2,p3,p4,p5,p6,p7);
            scrn_xyPrintf(s,ERROR_HIGH-3,1,"P-Print, C-Continue, D-Disable&Continue Others-quit");
  //        scrn_setAttrib(s,attr);
            win_refresh(w);
            while (1)
            {
                if ((global.optPrint || ch=='P') && util_printer_ready())
                {
                    ScreenPtr pbuff = scrn_dispMsg("Printing...");

                    pprintf("\nOS: DOS");


                    pprintf("\nSource: B44DIAG");

                    pprintf("\nRevision: %s",versionStr);
                    pprintf("\nUnit Under Test: NIC");
                    if (_current_grp==-1)
                    {
                        pprintf("\nModule: EEPROM content test");
                        pprintf("\nTest Name: EEPROM Content Check",tests[_current_grp].subTest[_current_id].name);
                        pprintf("\nTest #: A0");
                    }
                    else
                    {
                        pprintf("\nModule: %s Test",tests[_current_grp].name);
                        pprintf("\nTest Name: %s Test",tests[_current_grp].subTest[_current_id].name);
                        pprintf("\nTest #: %c%d",'A'+_current_grp,_current_id+1);
                    }    
                    pprintf("\nError #: %d",errCode);
                    pprintf("\nTime Stamp: %s",time_buffer);
					pprintf("\nIteration: %d",iteration);
                    pprintf("\nMsg: ");
                    pprintf(errorMsg[errCode],p1,p2,p3,p4,p5,p6);
                    pprintf("\n\n\n\n\n\n\n");
                    scrn_restoreScreen(MSGPOSITIONS,pbuff);
                    free(pbuff);
                }
                if ((ch = toupper(util_getch()))!='P')
                    break;
            }       
            win_close(w);
            scrn_showCursor();
        }    
    }
    else
        printf(errorMsg[errCode],p1,p2,p3,p4,p5,p6,p7);

//    _settextcolor (GRAY);
    eprintf("\n");
    if (ch=='D')
    {
        test_disable(_current_grp,_current_id);
        ch = 'C';
    }
	if (errCode != ERR_ABORT)
		if ((global.verbose & V_DEBUG) && global.manuf)
//            if (scrn_yes("Go to command prompt?"))
    			cmd_line_interface(NULL);
    return (ch=='C'? NO_ERROR: errCode);
} 

/*************************************************
 * function test_title
 *   returns test group title
 *
 * input: grp -- test group
 *        id  -- test id
 *
 * output: none
 *
 * return: the name of test
 *
 *************************************************/
const s08 * test_title(int grp, int id)
{
    return tests[grp].subTest[id].name;
}

/*************************************************
 * function test_groupTitle
 *   returns test group title
 *
 * input: grp -- test group
 *
 * output: none
 *
 * return: the name of the test group
 *
 *************************************************/
const s08 * test_groupTitle(int grp)
{
    return tests[grp].name;
}

/*************************************************
 * function test_maxTest
 *   returns a constant number of tests available in that group
 *
 * input: grp - group number
 *  
 * output: none
 *
 * return: The number of tests availabe in that group
 * 
 **************************************************/
int test_maxTest(int grp)
{
	return tests[grp].test_count;
} 

/*************************************************
 * function test_runTest
 *   performs the test
 *
 * input: grp -- test group
 *        id - test id
 *        para -- test parameter
 *
 * output: none
 *
 * return: test result
 *
 * side effect: Once this function is called, it set "tested" flag
 *              internally to indicate this test has been run before
 *
 *************************************************/
int test_runTest(u32 req_iteration, int grp, int id, void * para)
{
    int retc;
    
    if ((retc=do_test(req_iteration, grp, id, para, 1))==NO_ERROR)
        printf("\n");
    return retc;
}

/*************************************************
 * function test_testGroup
 *   performs all tests
 *
 * input: req_iteration -- requested iteration
 *        devMap -- it passes the device need to be tested. This passes in
 *                  bit map. If bit0 is set, it indicates device 0 is selected
 * output: none
 *
 * return: test result
 *************************************************/
int test_testGroup(u32 req_iteration, int grp, u32 sel)
{
    u32 iteration = 0;
    u32 old_select = test_selectedGrpTests(GROUP_REG);
    int errorCode = NO_ERROR;

    test_setGrpTests(grp,sel);
    do 
    {
        showIteration(++iteration, req_iteration);
        errorCode = test_grp(grp);
    } while ((!req_iteration || (iteration < req_iteration)) && (errorCode==NO_ERROR));
             
    if (errorCode==NO_ERROR)
        printf("\n");
    test_setGrpTests(grp,old_select);
    return errorCode;             
}        

/*************************************************
 * function test_doAllTests
 *   performs all tests
 *
 * input: req_iteration -- requested iteration
 *        devMap -- it passes the device need to be tested. This passes in
 *                  bit map. If bit0 is set, it indicates device 0 is selected
 * output: none
 *
 * return: test result
 *************************************************/
int test_doAllTests(u32 req_iteration, u32 devMap, u32 update_WOL_PXE, int eepProg)
{
    int rtv=HAS_ERROR;
    int dev = get_device_detected();
    
    if (!dev)
    {
        scrn_dispWait("Could not find any device");
    }
    else
    {
//        ScreenPtr saveScrn = scrn_save();
        rtv = startAllTests(req_iteration, devMap, update_WOL_PXE, eepProg);
//        free (saveScrn);
    }
    return rtv;
}        

/*************************************************
 * function test_enable 
 *   Enables the test
 *
 * input: grp, id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_enable(int grp, int id)
{
    if (grp < MAX_TEST_GROUP)
        if (id < tests[grp].test_count)
        {
            SET_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED);
            return 1;
        }
    return 0;
}

/*************************************************
 * function test_enableGrpTests 
 *   Enables the test, parameter are passed as bit mapped
 *
 * input: grp, bits
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_enableGrpTests(int grp, u32 bits)
{
    int id=0;
    
    if (grp < MAX_TEST_GROUP) 
    {
        while (id < tests[grp].test_count)
        {
            if (bits & 1)
                SET_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED);
            id++;
            bits >>= 1;    
        }
        return 1;
    }    
    return 0;
}

/*************************************************
 * function test_disable 
 *   disable the test
 *
 * input: id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_disable(int grp, int id)
{
    if (grp < MAX_TEST_GROUP)
        if (id < tests[grp].test_count)
        {
            CLEAR_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED);
            return 1;
        }
    return 0;
}

/*************************************************
 * function test_disableGrpTests 
 *   Disables the test, parameter are passed as bit mapped
 *
 * input: grp, bits
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_disableGrpTests(int grp, u32 bits)
{
    int id=0;
    
    if (grp < MAX_TEST_GROUP) 
    {
        while (id < tests[grp].test_count)
        {
            if (bits & 1)
                CLEAR_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED);
            id++;
            bits >>= 1;    
        }
        return 1;
    }    
    return 0;
}

/*************************************************
 * function test_enabled
 *   return zero if test is not enabled else it returns
 *          a non-zero number 
 *
 * input: grp,id - test id
 *  
 * output: none
 *
 * return: non-zero if enabled, zero if disabled
 * 
 **************************************************/
int test_enabled(int grp, int id)
{
    if (grp < MAX_TEST_GROUP)
        if (id < tests[grp].test_count)
        {
            return IS_FLAG(tests[grp].subTest[id].flags,TEST_ENABLED);
        }
    return 0;
}

/*************************************************
 * function test_tested
 *   returns true if the test is run previousely
 *
 * input: grp,id -- test id
 *  
 * output: none
 *
 * return: zero if the test has never been tested
 *         otherwise, returns non-zero value
 **************************************************/


/**************************************************************
 * function test_checkTestStr
 * reads a test select as string in enable/disable the test.
 *
 * input: enable - 1:enable, 0:disable tests
 *        p      - parameter string
 *
 * output: prints out error message if error is detected
 *
 * return: 0 or no error, 1 for error detected
 *
 * side effects: none
 *************************************************************/
int test_checkTestStr(int enable, s08 * p)
{
    u32 tests;
    u08 grp;
    
    while (*p) 
    {       
        if (*p >= 'a')
           *p &= 0xdf;  // convert to upper case
        grp = *p++ - 'A';
        if (grp >= MAX_TEST_GROUP)
        {
            printf("ERROR: Invalid parameter to enable/disable tests\n");
            return 1; // group error 
        }
        tests = 0;
        while (*p >= '1' && *p <= '9')
        {
            tests |= 1 << (*p - '1');
            p++;
        }
        if (!tests) // any test selected?
          tests =0xffffffff;  // select all tests
        if (enable)
            test_enableGrpTests(grp, tests);
        else
            test_disableGrpTests(grp, tests);
    }   
    return 0;
}

/*************************************************
 * function reset_tests
 *   reset all tests to not tested state
 *
 * input: none
 *  
 * output: none
 *
 * return: non
 **************************************************/

/*************************************************
 * function test_notAvailable
 *   display 'test_passed' and return NO_ERROR
 *
 * input: none
 *  
 * output: none
 *
 * return: NO_ERROR
 **************************************************/
int test_notAvailable(void)
{
    if (!_failed)
    {
        _settextcolor (PASS_COLOR);
        printf("N/A");
        _settextcolor (GRAY);
    }
    return NO_ERROR;
}

int link_status(u08 stat)
{
    if (!_failed)
    {
        _settextcolor (PASS_COLOR);
		if (stat)
			printf("Link UP");
		else
			printf("Link DOWN");
        _settextcolor (GRAY);
    }
    return NO_ERROR;
}
/*************************************************
 * function test_passed
 *   display 'test_passed' and return NO_ERROR
 *
 * input: none
 *  
 * output: none
 *
 * return: NO_ERROR
 **************************************************/
int test_passed(void)
{
    if (!_failed)
    {
        _settextcolor (PASS_COLOR);
        printf("passed             ");
        _settextcolor (GRAY);
    }
    return NO_ERROR;
}
/*************************************************
 * function test_passed
 *   display 'test_passed' and return NO_ERROR
 *
 * input: none
 *  
 * output: none
 *
 * return: NO_ERROR
 **************************************************/
int test_passed0(void)
{
    _settextcolor (PASS_COLOR);
    printf("passed             ");
    _settextcolor (GRAY);
    
    return NO_ERROR;
}
/**************************************************/
set_current_test(int g,int i) 
{
    _current_grp=g;
    _current_id=i;
}

/*****************************************************************
 * showIteration
 *     if req_iteartion is not 1, then show the itearation 
 *
 * in - iteration : current iteration
 *      req_iteration : requested iteration
 *****************************************************************/ 
void showIteration(u32 iter, u32 req_iteration)
{
    if (req_iteration != 1)
    {
        _settextcolor (HELP_COLOR);
        printf("\nIteration %d",iter);
        if (req_iteration)
            printf(" of %d",req_iteration);
        _settextcolor (GRAY);
    }
}
        
/*************************************************
 *
 **************************************************/
int test_setError(int code)
{
    return (_failed = code);
} 