/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ U T I L . H
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_util.h 12    7/19/02 10:07a Ahui $
 *
 *  Description: This file contains utility routines for T3.
 *
 *  AUTHOR: Kevin Tran
 *****************************************************************************/
 
#ifndef __T3_UTIL_H__
#define __T3_UTIL_H__

#include "eeprom.h"
#include "t3_tests.h"
#include "t3_reg.h"
#include "t3_util.h"
#include "fw_img.h"
#include "tigon3.h"
#include "um.h"
#include "protocol.h"
#include "t3_tx.h"
#include "blast.h"
#include "Cl4xsb.h"
#define MAX_PATH_NAME_LEN 80


//#define DIAG_LOG(y,msg) if (y <= (global.diag_cb.verbose_level)) printf msg
#define DIAG_LOG_FATAL  1
#define DIAG_LOG_ERROR  2
#define DIAG_LOG_INFO   3

#define DIAG_FAILURE   -1
#define DIAG_SUCCESS   0
#define DIAG_ABORTED   -2
#define DEBUG_MODE_OFF  0
#define DEBUG_MODE_ON   1

#define EEPROM_MODE_AUTO (t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.LocalCtrl)) & GRC_MISC_LOCAL_CTRL_AUTO_SEEPROM)
//#define SEEPROM_MODE_AUTO     1
//#define SEEPROM_MODE_BIT_BAN  2
#define SEEPROM_TOTAL_SIZE  (256 * 1024)
#define SEEPROM_CHIP_SIZE   (64  * 1024)
#define SEEPROM_WAIT        100000

#ifdef IKOS
#define SEEPROM_CLOCK_PERIOD  0x8
#define CORE_CLOCK_FREQ       190000L
#else
#define CORE_CLOCK_FREQ       66000000L
#define SEEPROM_CLOCK_PERIOD  0x60
#endif

#define SEEPROM_VPD_START      0x100
#define VPD_READ_START         0
#define VPD_READ_SIZE          128
#define VPD_WRITE_START        0x80
#define VPD_WRITE_SIZE         128
#define VPD_STORAGE_SIZE       (VPD_WRITE_SIZE+VPD_READ_SIZE)

/* T3 supports 8 MSI messages. Parameters are powered of 2 */
#define MSI_MSG_COUNT          3

#define MAGIC_WORD          (((u32)'K' << 24) | ((u32)'e' << 16) | \
			     ((u32)'v' << 16) | ((u32)'T'))

#define ALL_DEV_BIT (util_power(2,global.device_detected) - 1)

/* Memory block */
#define MEM_BLOCK_SRAM_BD           (1 << 0)
#define MEM_BLOCK_SRAM_DMA          (1 << 1)
#define MEM_BLOCK_SRAM_MBUF         (1 << 2)
#define MEM_BLOCK_SPAD              (1 << 3)
#define MEM_BLOCK_EXT_SRAM          (1 << 4)
#define MEM_BLOCK_SRAM_MBUF_VIA_DMA (1 << 5)
#define MEM_BLOCK_SRAM_MBUF_SPECIAL (1 << 6)
#define MEM_RANDOM_ADDR_DATA_TEST   (1 << 7)
#define MAX_MEM_BLOCKS              8

#define MEM_BD_BASE1                  0x0000
#define MEM_BD_SIZE1                  (4*1024)
#define MEM_BD_TOP1                   (MEM_BD_BASE1 + MEM_BD_SIZE1-1)

#define MEM_BD_BASE2                  0x4000
#define MEM_BD_SIZE2                  (16 *1024)
#define MEM_BD_TOP2                   (MEM_BD_BASE2 + MEM_BD_SIZE2-1)

#define MEM_DMA_BASE                  0x2000
#define MEM_DMA_SIZE                  (8 *1024)
#define MEM_DMA_TOP                  (MEM_DMA_BASE + MEM_DMA_SIZE-1)


#define MEM_INTERNAL_MBUF_BASE       0x8000
#define MEM_INTERNAL_MBUF_SIZE       T3_NIC_MBUF_POOL_SIZE
#define MEM_INTERNAL_MBUF_TOP        (CURRCARD.MbufTopAddr)

#define MEM_SPAD_BASE                0x30000
#define MEM_SPAD_SIZE                (32 *1024)
#define MEM_SPAD_TOP                 (MEM_SPAD_BASE + MEM_SPAD_SIZE-1)

#define MEM_EXT_SRAM_BASE            0x20000
#define MEM_EXT_SRAM_SIZE            (16*1024*1024 -0x20000)
#define MEM_EXT_SRAM_TOP             (MEM_EXT_SRAM_BASE + MEM_EXT_SRAM_SIZE-1)
#define MEM_EXT_SELECTION			 0x01129201
#define PME_MAX_DATA_ENTRY 16

#define DMA_TEST_DESC_START          0x2000
#define DMA_TEST_NUM_DESC            6
#define DMA_TEST_DESC_END            (DMA_TEST_DESC_START + DMA_TEST_NUM_DESC * 0x20)

#define T3_MAX_NUM_REG         4096
#define T3_MAX_NUM_MII_REG     31

#define CLOCK_ERROR_MARGIN      15

#define PXE_MEM_START       0x10000
#define PXE_MAX_IMAGE_SIZE (64*1024)

#define DISPLAY_PROGRESS() if (!(i & 0xff)) { putchar(progress_bar[++global.progress_bar_index % 4]); \
                           putchar('\b'); }


#define NUM_MBUF_CHAINS         8
#define MAX_LOST_MBUFS          64
#define NUM_CROSS_LINKED_MBUFS  8
#define MBUF_MAX_REF_COUNT 16

typedef struct PCI_DEVICE_INFO
{
  u16 vendor_id;
  u16 device_id;
  u32 class;
  u08 revision;
  u32 base_addr_l;
  u32 base_addr_h;
  int irq;
}PCI_DEVICE_INFO,*pPCI_DEVICE_INFO;

#define T3_SPEED_10MBPS     0
#define T3_SPEED_100MBPS    1
#define T3_SPEED_1000MBPS   2
#define T3_SPEED_AUTONEG    3

#define T3_MODE_FULL_DUPLEX 0
#define T3_MODE_HALF_DUPLEX 1

#define PHY_54XX_AUTO_NEG    (1 << 9)

#define PHY_54XX_SPEED_MASK   ((1 << 13) | (1 << 6))
#define PHY_54XX_SPEED_10MPS   0
#define PHY_54XX_SPEED_100MPS  (1 << 6) 
#define PHY_54XX_SPEED_1000MPS (1 << 13) 

#define PHY_54XX_MODE_MASK           (1 << 8)
#define PHY_54XX_MODE_FULL_DUPLEX    (1 << 8)
#define PHY_54XX_MODE_HALF_DUPLEX    0

#define PCI_BUS_TYPE_CONVENTIONAL_PCI  1
#define PCI_BUS_TYPE_PCI_X             2

typedef struct DIAG_MEM_CB
{
  u32 sram_bd1_start;
  u32 sram_bd1_end;
  u32 sram_bd2_start;
  u32 sram_bd2_end;
  u32 sram_dma_start;
  u32 sram_dma_end;
  u32 sram_mbuf_start;
  u32 sram_mbuf_end;
  u32 sram_spad_start;
  u32 sram_spad_end;
  u32 ext_mem_start;
  u32 ext_mem_end;
  int mbuf_bank;
}DIAG_MEM_CB,*pDIAG_MEM_CB;



typedef struct DRV_CFG_CB
{
  int NicSendBd;
  int TaskOffloadCap;
  int TxPacketDescCnt;
  int RxStdDescCnt;
  int RxCoalescingTicks;
  int TxCoalescingTicks;
  int RxMaxCoalescedFrames;
  int TxMaxCoalescedFrames;
  int RxCoalescingTicksDuringInt;
  int TxCoalescingTicksDuringInt;
  int RxMaxCoalescedFramesDuringInt;
  int TxMaxCoalescedFramesDuringInt;
  int StatsCoalescingTicks;
  int TxCopyBufferSize;
  int RxJumboDescCnt;
  int RxJumboBufferSize;
  int QueueRxPackets;
  int ExtMemExists;
  u32 MbufBase;
  u32 MbufLength;
  int FlowControl;
  int auto_link_speed;
  int EnableMiniRing;
  int RxMiniBufferSize;
}DRV_CFG_CB,*pDRV_CFG_CB;

typedef struct DIAG_CB
{
  int version;
  int abort_on_fail;         /* Abort on fail or not */
//  int verbose_level;         /* Verbose Level      */
  DIAG_MEM_CB mem;           /* Memory test        */
//  DIAG_MF_CB mft;            /* Manufacturing test */
  u32 testSelect[MAX_TEST_GROUP];
  DRV_CFG_CB drv_cfg;        /* LM driver configuration */
}DIAG_CB,*pDIAG_CB;

#define DRV_CFG_FILE_NAME   "drvcfg.txt" /* Driver configuration files */
#define DIAG_CFG_FILE_NAME   "diagcfg.bin"  /* Diagnostics configuration files */
#define DIAG_MII_TEST_FILE_NAME  "MiiReg.txt" /* MII register definition files */
#define DIAG_CPU_TEST_FILE_NAME  "cpu.bin" /* Firmware file for CPU test */
#define DIAG_CPU_DIAG_FILE_NAME  "cpudiag.bin" /* Firmware file for CPU diag test */

typedef struct DRV_CFG_ENTRY
{
  s08 *tag;
  int id;
}DRV_CFG_ENTRY,*pDRV_CFG_ENTRY;

#define DRV_CFG_TAG_NIC_SEND_BD      0
#define DRV_CFG_TAG_TASK_OFFLOAD     1
#define DRV_CFG_TAG_REQ_MEDIA_TYPE   2
#define DRV_CFG_TAG_TX_DESC_CNT      3
#define DRV_CFG_TAG_RX_STD_DESC_CNT  4
#define DRV_CFG_TAG_RX_MINI_DESC_CNT 5
#define DRV_CFG_TAG_RX_JMBO_DESC_CNT 6
#define DRV_CFG_TAG_RX_COALES_TICK   7
#define DRV_CFG_TAG_TX_COALES_TICK   8
#define DRV_CFG_TAG_RX_COALES_FRAME  9
#define DRV_CFG_TAG_TX_COALES_FRAME  10
#define DRV_CFG_TAG_TX_COPY_BUF_SIZE 11
#define DRV_CFG_TAG_RX_JMBO_BUF_SIZE 12
#define DRV_CFG_TAG_USE_EXT_MEM      13
#define DRV_CFG_TAG_STATS_TICKS      14
#define DRV_CFG_TAG_MBUF_BASE        15
#define DRV_CFG_TAG_MBUF_LENGTH      16
#define DRV_CFG_TAG_RX_MINI_BUF_SIZE 17
#define DRV_CFG_TAG_Q_RX_PACKETS     18

#define DRV_CFG_TAG_MAX_ENTRY        19

#define RX_CPU  0
#define TX_CPU  1
#define RX_CPU_CODE_SPACE 0x00030000L
#define TX_CPU_CODE_SPACE 0x00034000L

#define PARM_TYPE_BYTE    0
#define PARM_TYPE_WORD    1
#define PARM_TYPE_DWORD   2
#define PARM_TYPE_STRING  3
#define PARM_TYPE_NO_DATA 4

typedef struct ARGUMENT_PARM
{
  int valid;
  s08 opt;
  int data_type;
  int radix;
  union 
  {
    u08 byte;
    u16 word;
    u32 dword;
    s08 *string;
  }data;
}ARGUMENT_PARM,*pARGUMENT_PARM;

#define CPU_TEST_MAGIC_NUM       0x0BEEF0
#define CPU_TEST_MAGIC_NUM_RESP  0xAA0000
#define CPU_TEST_IS_COMPLETE(code) ((code & 0xff0000) == CPU_TEST_MAGIC_NUM_RESP)
#define CPU_TEST_RETC(code)      (code & 0xffff)

#define CPU_TEST_WAIT_TIME  20

#define MAX_IMAGE_SIZE (16 * 1024)

#define DIAG_REGISTER_TEST      (1 << 0)
#define DIAG_MEMORY_TEST        (1 << 2)
#define DIAG_PACKET_XCHG_TEST   (1 << 3)
#define DIAG_EEPROM_TEST        (1 << 4)
#define DIAG_INTERRUPT_TEST     (1 << 5)
#define DIAG_CPU_TEST           (1 << 6)
#define DIAG_DMA_TEST           (1 << 7)
#define DIAG_VPD_TEST           (1 << 8)
#define DIAG_MII_TEST           (1 << 9)
#define DIAG_BIST_TEST          (1 << 10)

/******************* TPROT Stuffs ***************************/

#define TPROT_PACKET_HEADER_SIZE                5
#define TPROT_PACKET_SIG                        0x636b6e7a

#define TPROT_PACKET_OVERHEAD_SIZE              22

#define TPROT_ETHERNET_MAX_FRAME_SIZE           1514
#define TPROT_ETHERNET_HEADER_SIZE              14
#define TPROT_ETHERNET_MIN_FRAME_SIZE           60

#define TPROT_GET_PACKET_SIG(_p)        (*((u32 * ) \
                                        (((u08 *) (_p)) + \
                                        TPROT_ETHERNET_HEADER_SIZE)))

#define TPROT_GET_PACKET_TYPE(_p)       (*(((u08 *) (_p)) + \
                                        TPROT_ETHERNET_HEADER_SIZE + \
                                        TPROT_PACKET_HEADER_SIZE - 1))

#define TPROT_PACKET_PAT_INC_NO_CHK_ERROR 1
#define TPROT_PACKET_PAT_INC_CHK_ERROR    2

// Packet formats.
#define TPROT_PACKET_FORMAT_UNSPECIFIED      0x00 // Current buffer content.
#define TPROT_PACKET_FORMAT_DEFAULT          0x01 // Some default pattern.
#define TPROT_PACKET_FORMAT_ASCENDING        0x02 // Some default pattern.
#define TPROT_PACKET_FORMAT_LAST_ITEM        0x03 // must be the last item here.

// Tprot packet type.
typedef enum {
    // Packet acknowledgment.  This packet serves the purpose of acknowledging
    // the previous packet.  Not all commands requires acknowledgment.
   //
    // This packet has the following format.
    //    1 byte -- Packet type of the packet the station received.
    TPROT_PACKET_TYPE_PACKET_ACK                    = 0,

    // Packet data.  This packet is sent in various tests.  It's purpose is to
    // define the format of the data and upon reception whether to check
    // for errors.
    //
    // This packet has the following format.
    //    1 byte  -- Packet format.
    //       0 -- Packet data is random.
    //       1 -- Packet data is the byte-value sequence 0, 1, 2, 3, ...,
    //          but don't check for errors.
    //       2 -- Packet data is the byte-value sequence 0, 1, 2, 3, ...,
    //          check for errors.
    //    2 bytes -- Data size.
    //    x bytes -- Packet data as defined in packet format.
    TPROT_PACKET_TYPE_DATA_PACKET                   = 1,

    // Loopback packet.  Upon reception, the remote station loops back this
    // packet to the local station.  If the packet format indicates error
    // checking to be performed, the packet will be looped back if there is
    // an error.
    //
    // The new packet will have the old packet's source address as it's
    // destination, and the source address of the new packet is it's address.
    // Packet data remain the same.
    //
    // This packet has the following format:
    //    1 byte  -- Packet format. (see DATA_PACKET).
    //    2 bytes -- Data size.
    //    x bytes -- Packet data which contains the sequence 0, 1, 2, and so
    //               on up to the data size.
    TPROT_PACKET_TYPE_LOOPBACK                      = 2,

    // Remote address packet.  This packet is used to find the remote station's
    // address.  Upon reception, the remote station replys with
    // the REMOTE_ADDR_REPLY packet.
    //
    // This packet has the following format:
    //    None.
    TPROT_PACKET_TYPE_REMOTE_ADDR                   = 3,

    // Remote address reply packet.  This packet is a reply to the
    // REMOTE_ADDR packet.  This packet is sent out by the remote station.
    //
    // This packet has the followin format:
    //    None.
    TPROT_PACKET_TYPE_REMOTE_ADDR_REPLY             = 4,

    // Send packets.  This packet instructs the remote station to send
    // out a series of DATA_PACKET at full speed.
    //
    // The remote station keeps the following statistics:
    //    Elapsed time in milliseconds.
    //    Bytes sent.
    //    Packets sent.
    //
    // This packet has the following format:
    //    4 bytes -- Number of packets to send.
    //    1 bytes -- Packet format.
    TPROT_PACKET_TYPE_SEND_PACKETS                  = 5,

    // Send packets statistics.  This packet is sent to the remote station
    // to retrieve the statistics that were created during the SEND_PACKETS 
    // session.
    //
    // This packet has the following format.
    //    None.
    TPROT_PACKET_TYPE_GET_STATISTICS                = 6,

    // Send packets statistics reply.  This packet is sent by the remote
    // station in reponse to SEND_PACKET_STATS.
    //
    // This packet has the following format.
    //    4 bytes -- Elapsed time in milliseconds.
    //    4 bytes -- Bytes sent.
    //    4 bytes -- Packets sent.
    TPROT_PACKET_TYPE_GET_STATISTICS_REPLY          = 7,

    // Receive packets.  This packet inform the remote station to begin
    // receive DATA_PACKET packets.
    //
    // The remote station keeps track of the following statistics during this
    // session.
    //    Elapsed time in milliseconds.
    //    Bytes received without error.
    //    Packets received without error.
    //    Packets received with error.
    //
    // This packet has the following format.
    //    None.
    TPROT_PACKET_TYPE_RECEIVE_PACKETS               = 8,

    // This packet has the following format.
    //    None.
    TPROT_PACKET_TYPE_RESET_SERVER_STATS            = 9,

    // Must be the last item in this enum.
    TPROT_PACKET_TYPE_LAST_ITEM                     = 10,
} TPROT_PACKET_TYPE, *PTPROT_PACKET_TYPE;

#define MICRO_SECONDS(_x)    (_x)*global.clk_scale
#define DIAG_MAGIC_PACKET_TYPE  		0xdead
#define TPROT_MAGIC_PACKET_TYPE 		0x2301
#define MAX_STATIC_TX_BUFFER_SIZE	    0x4000
#define MAX_STATIC_RX_BUFFER_SIZE	    0x4000

typedef struct card {
    pRegs macRegs;
    int  phy_device;
    media_manufact_region_t manufact;
    void (*isr_callback)();
    void *lm_cb_ptr;
    u32  loadd_count;
    u32  base_address;
    u32  base_address_offset;
    u32  ASIC_version;
    u32  cmd_stat;
    u32  bist_lat;
    U32  ReceiveMask;
    s08  *mem_pool_start_addr;
    u32  mem_pool_handle;
    u32  eeprom_size;
    u32  external_memory_size;
    int  phy_type;
    int  phy_led_mode;
    u32  MbufTopAddr; //  = (MEM_INTERNAL_MBUF_BASE + MEM_INTERNAL_MBUF_SIZE-1);
    blastCfg   blast;
    TX_TEST_INFO txInfo;
    int (*rx_ind_users[MAX_RX_IND_USERS])(PUM_PACKET pPacket);
    int (*tx_comp_ind_users[MAX_TX_COMP_IND_USERS])(PUM_PACKET pPacket);
	SWMIBCOUNTERS swmibcounts;
    u32  msiArea[4]; 

/* 16 bit variables */    
    u16  intr_count;
    u16  vendor_id;
    u16  device_id;
    u16  subsystem_vendor_id;
    u16  subsystem_device_id;

/* 8 bit variables */
    u08  rel_mem_write;
//    u08  pcix_work_around; 
    s08  *dev_str;
    u08  driver_loaded;
	u08  tx_buffer[MAX_STATIC_TX_BUFFER_SIZE];
	u08  rx_buffer[MAX_STATIC_RX_BUFFER_SIZE];
	u08  rx_extd_buffer[MAX_ETHERNET_PACKET_SIZE];
	u08  pattern_file_buffer[MAX_JUMBO_TX_BUFFER_SIZE];
    u08  boundary_offset;
    u08  opt_not_straddle_4k_boundary;
    u08  interrupt_line;
    u08  bus_number;
    u08  device_and_function;
    u08  poll_mode;
    u08  mem_pool_initialized;
    u08  fw_loaded;
    u08  CRC_patched;
    u08  card_opt_selected;
    u08  ODI_driver_loaded;
    u08  voltage_level;
    u08  isRef;
    u08  dma_workaround_done;
    u08  Speed10MBS;
    u08  Speed100MBS;
    u08  Speed1000MBS;
    u08  initialized;
    u08  indirect_access;
	u08 gottestint;
	u08 inttest;
} card_t;
 
#define TBIMODE() (CURRCARD.phy_type == NIC_HW_CONFIG_PHY_TYPE_FIBER)
#define CRC_MAGIC_VALUE     0xDEBB20E3
#define CRC32_POLYNOMIAL    0xEDB88320
#define get_currcard_index() (global.current_card)
//#define get_currcard() (global.device_detected ? &CURRCARD : NULL)
#define get_currcard() (global.current_card < global.device_detected ? &CURRCARD : NULL)
#define get_device_detected() (global.device_detected)
#define util_getkey(x) win_getkey(x)

/************************************************************************
 *      Functions exported from t_util.c                                *
 ************************************************************************/

/* Function Prototypes */


int     util_wait_ms(int milliseconds);
void    util_wait_1us(int wait);
void    util_fill_miiregs(u32 beg, u32 end, u16 data);
void    util_fill_conf (u32 beg, u32 end, u32 data, int);
void    util_write_reg (u32 addr, u32 data);
void    util_fill_reg (u32 beg, u32 end, u32 data);
void    util_install_default_handler();
void    util_print_miiregs(u32 beg, u32 end);
void    util_fill_direct_mem(u32 beg, u32 end, u32 data,int type);
void    util_intr_info(void);
void    util_print_tabular (u32 beg, u32 end, u32 (*func)(u32));
void    util_pci_cfg_write_word(u32 address,u32 data,int *failed);
void    util_pci_cfg_write_short(u32 address,u16 data,int *failed);
void    util_write_sram (u32 addr, u32 data);
void    util_write_isram (u32 addr, u32 data);
void    util_init_default_cfg(void);
void    util_init_pci_and_find_tigons(int,int);
void    util_print_banner(void);
void    util_select_devices(void);
void    util_system_init(void);
void    util_show_device(int i);
void    util_beep(void);
void    util_open_device_windows(void);
int     util_hw_init(int pci_init);

int     util_user_abort(void);
int     util_switch_card(int dev);
int     util_pause(s08 * msg);

int     util_isDigit(s08 c);
int     util_openLog(s08 * logFileName);
int     util_openBin(s08 * binFileName);
int     util_pcicfg_init(void);
int     util_pci_scan(void);
int     util_system_exit(int);
int     util_resetPhy(void);
int     util_printer_ready(void);
int     util_activateDevice(int dev);
int     util_readFile(s08 * filename, int offset, u32 len, u32 * size, u08 *buffer);
int     util_readFileMemAlloc(s08 * filename,int offset,u32 len,u32 limit,u32 * size, u08 **buffer, int verbose);
int     util_readMacAddress();
int     util_getPass(s08 * msg, s08 * buff, int len);
int     util_fileExists(s08 * logFileName);
int     util_is5703_or_later(void);
int     util_poll_cpu_messages(void);
double  util_get_clock();


u08     util_char2num(u08 c);



u32     util_read_reg(u32 addr);
u32     util_read_sram (u32 addr);
u32     util_read_isram (u32 addr);
u32     util_read_sram_blk(u32 addr, u32 *buffer,int num_word);

u32     util_read_isram_blk(u32 addr, u32 *buffer,int num_word);
u32     util_power(u32 base,u32 n);
u32     util_read_direct_mem(u32 addr);
u08     util_read_direct_mem_byte(u32 addr);
u16     util_read_direct_mem_short(u32 addr);
void    util_printTabularByte(u32 beg, u32 end, u08 (*func)(u32));
void    util_printTabularShort(u32 beg, u32 end, u16 (*func)(u32));
int     util_DriverLoaded(void);
int     util_CheckDriverLoaded(void);
int     util_kbhit(void);
int     util_getch(void);
int     util_AccessMacAddrFile(u08 AccessMode);
int     util_ProcessMacAddr(s08 *buff);
/* for misc.asm prototyping */

u16 disable_intr();


#endif /* __T3_UTIL_H__ */

 /*****************************************************************************
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_util.h $
 * 
 * 12    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 11    7/11/02 1:02p Ahui
 * Clean up unused code.
 * 
 * 10    6/13/02 10:02a Ahui
 * Clean up unused code
 * 
 * 9     6/11/02 11:50a Ahui
 * Added functions to support fmac option
 * 
 * 8     5/16/02 6:00p Mnima
 * use input eeprom name
 * 
 * 7     4/25/02 11:03a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

