#ifndef __T3_VAR_H__
#define __T3_VAR_H__

#include "types.h"
#include "asf.h"
#include "pet.h"


#define SYSTEM_STACK_SIZE 0x200              /* 1K bytes for stack */
#define SYSTEM_VAR_SIZE   0x600              /* 1.5K for system variable */
#define SYSTEM_RAM        0x4000             /* each CPU has only 16K RAM */
#define CBUFSIZE          0x80               /* console buffer size */

#define BUFSIZE           48
#define ADDRBUFSIZE       16
#define TOTALADDRESSES    104


#define SMB_HW_EVENT                RX_CPU_EVT_SW1
#define CMD_SW_EVENT                RX_CPU_EVT_SW3
#define STOP_WD_EVENT               RX_CPU_EVT_SW4
#define ETH_SW_EVENT                RX_CPU_EVT_SW5
#define SMB_SW_EVENT                RX_CPU_EVT_SW6
#define DRIVER_EVENT                RX_CPU_EVT_SW7

#define SMB_ATTN                    TX_CPU_EVT_SW0
#define POLL_LEGACY_TIMEOUT_EVENT   TX_CPU_EVT_SW1
#define POLL_ASF_TIMEOUT_EVENT      TX_CPU_EVT_SW2
#define TX_CMD_SW_EVENT             TX_CPU_EVT_SW3
#define RMCP_SMBUS_EVENT            TX_CPU_EVT_SW4

/* RX */
#define SMB_SW_EVENT_BIT   (1 << SMB_SW_EVENT)
#define SMB_HW_EVENT_BIT   (1 << SMB_HW_EVENT)
#define STOP_WATCHDOG_BIT  (1 << STOP_WD_EVENT)
#define ETH_SW_EVENT_BIT   (1 << ETH_SW_EVENT)
#define CMD_SW_EVENT_BIT   (1 << CMD_SW_EVENT)
#define DRIVER_EVENT_BIT   (1 << DRIVER_EVENT)

/* TX */
#define SMB_ATTN_BIT       (1 << SMB_ATTN)
#define POLL_LEGACY_TIMOUT_BIT (1 << POLL_LEGACY_TIMEOUT_EVENT)
#define POLL_ASF_TIMEOUT_BIT   (1 << POLL_ASF_TIMEOUT_EVENT)
#define TX_CMD_SW_EVENT_BIT  (1 << TX_CMD_SW_EVENT)
#define RMCP_SMBUS_EVENT_BIT  (1 << RMCP_SMBUS_EVENT)

#define ASF_EVENT_TX_MASK (RMCP_SMBUS_EVENT_BIT | SMB_ATTN_BIT | POLL_LEGACY_TIMOUT_BIT | POLL_ASF_TIMEOUT_BIT | TX_CMD_SW_EVENT_BIT)
#define ASF_EVENT_RX_MASK (STOP_WATCHDOG_BIT | SMB_SW_EVENT_BIT | SMB_HW_EVENT_BIT | ETH_SW_EVENT_BIT | CMD_SW_EVENT_BIT | RX_CPU_EVENT_RLP | DRIVER_EVENT_BIT)

typedef struct _IPHOST
{
   BYTE  ip_addr[4];
   BYTE  mac_addr[6];
} IPHOST;

typedef struct {
    u32     db_ctrl;
#define DB_CTRL_BUSY    BIT_0
#define DB_CTRL_ERR     BIT_1
    u32     db_para1;
    u32     db_para2;
    BYTE    db_data_out[CBUFSIZE];
} db_print_t;

typedef enum {
    ONESHOT,
    REPEAT,
} PET_TYPE;

/*
typedef struct {
    BYTE        data[BUFSIZE];
    BYTE        Length;
    BYTE        MsgType;
} PET_MSG_t, * pPETMSG;
*/
/*
#define MAX_PET_MSG 2
*/

typedef enum {
    OS_PRESENT,
    OS_ABSENT,
    OS_WoL
} sysMode_t;    

typedef enum {
    ASFNormal,
    ASFPause,
} asfMode_t;    

typedef struct {
    PET_DATA petData;

    BYTE     MsgType;
    BYTE     Count;
	BYTE     filler[2];

    WORD     WatchdogTime;
    WORD     SequenceNum;

} PET_INFO, * pPET_INFO;

/* common structure for both CPUA and CPUB */
typedef struct {
/* debug application interface */
    db_print_t  prn;
    PET_INFO    pet;
    sysMode_t   sysMode;
    asfMode_t   asfMode;
} commonVar_t, * pCommVar;

typedef void (*callBackFunc)(void);

/* Timer structure for Heartbeat, WatchDog and Software Timers. */
/* The Software Timer is implemented using Retransmission Timer */
/* at 1 second interval.                                        */
#define MAX_SW_TIMERS	6	/* number of Software Timers */

typedef struct {
	callBackFunc  callback;

	u16           time;
	u16           count;
} SwTimer_t;

typedef struct {
    callBackFunc    HeartBeatCallBack;

    u16             idControl;
    u16             HeartBeatTime;

	callBackFunc	WatchDogCallBack;

	SwTimer_t       SwTimer[MAX_SW_TIMERS];

	u16				WatchDogTime;
	u16             DriverTimerID;
} asfTimer_t;

/*
typedef union {
    RMCP_PKT    rmcp;
    ARP_PKT     arp;
    PET_PKT     pet;

//    IMCP_PKT    imcp;

} ASF_PKT;
*/
//#define ASF_PACKET_SIZE sizeof(ASF_PKT)

/* smbus variables */
typedef struct {
    BYTE    bufferin[BUFSIZE];
    u32     bufferindex;
    BYTE    bufferout[BUFSIZE];
    u32     bufferoutdex;
    u32     bufferoutlen;
    BYTE    UsedAddresses[ADDRBUFSIZE];
    BYTE    ASFAddresses[TOTALADDRESSES];
    BYTE    ASFStates[TOTALADDRESSES]; /* support 8 devices per address to start       */
    BYTE    bufferoutsem;
    BYTE    bufferoutvalid;
    BYTE    ExpectedInput;
    BYTE    commands;
    BYTE    bufferinvalid;	
	BYTE    filler[3];		   /* word align */
/*    PET_MSG_t PETMsg[MAX_PET_MSG]; */
} smVar, *pSmVar;

/* ip variables */
#define ARP_CACHE_SIZE     2


/* Pet Retransmission structure */
#define MAX_PET_RETRAN 4
typedef struct {
	BYTE	 count;		/* how many times to retransmit */
	BYTE     filler[3];
	PET_INFO petInfo;	/* pointer to MBuf */
} Pet_Retran_t;

/* The sys-> data structure for CPUA.  All elements in here are accessed via sys-> */
typedef struct {
    commonVar_t  comm;   /* must be at the top of structure */
//    asf_t        asf;
    BYTE         NicMacAddr[8];
    BYTE         MgntConsMacAddr[8];
//    ASF_PKT      txPktBuff, rxPktBuff;  /* these variables must be 32-bit aligned */
    u16          txPktSize, rxPktSize;
    asfTimer_t   tm;
	PET_INFO     WatchdogPet;
	Pet_Retran_t PetRetran[MAX_PET_RETRAN];
    IPHOST       arp_cache[ARP_CACHE_SIZE];
} SystemAVar, *pSysAVar;

/* The sys-> data structure for CPUB.  All elements in here are accessed via sys-> */
typedef struct {
    commonVar_t comm;  /* must be at the top of the structure */
//    asf_t       asf;
    BYTE        NicMacAddr[8];
    u32         rmcpCmd;
    smVar       sm;
} SystemBVar, *pSysBVar;

#endif